#ifndef __BROWSEIO_H_
#define __BROWSEIO_H_

#define IMAGE_ICON_SIZE		120

#define IMAGE_ICON_DIR_INFO	256
#define IMAGE_ICON_FILENAME	128
#define IMAGE_ICON_FILESIZE	64
#define IMAGE_ICON_IMAGESIZE	32
#define IMAGE_ICON_FILEDATE	16
#define IMAGE_ICON_SELECTED	8
#define IMAGE_ICON_SUB_DIR	4
#define IMAGE_ICON_NO_SUFFIX	2
#define IMAGE_ICON_REPEAT	1


#define MAX_FILE_HISTORY	20 /* maximum number of entries in recently used files history */

/* this does not belong here, but since the interface part of the icon library does redraws, it needs to know about it */
#define IMG_BUF_Y              350 /* height of screen buffer for redraws */

typedef struct
 {
  char *Fname;
  unsigned short imageX, imageY;
  unsigned short xdpi, ydpi; /* use 0 to indicate unknown value; not multiplied by 1000 like in load/print interface! */
  unsigned char FnameLen; /* the length of Fname string */
  unsigned char loaderID;
  unsigned char usedColors; /* valid only for colormap images */
  unsigned char iconX, iconY;
  unsigned char imageType;
  unsigned char iconType;
  unsigned char cmapSize; /* number of colors in the colormap, if used */
  off_t fileSize;
  time_t modifiedTime;
  unsigned short compressedIconLen;
  unsigned short subDirID;
  unsigned char *iconData;
  unsigned char *iconCmap;
  char filterOK;
 } imageInfo;

typedef struct
 {
  char *name; /* directory name */
  int numDirs; /* number of directories in this directory */
  int numFiles; /* number of files in this directory */
 } dirBrowse;

typedef struct
 {
  int numDir; /* number of used directories */
  int numAllocDir; /* number of allocated directories */
  dirBrowse *dirB; /* pointer to array of structures with directory names and number of files */
 } directoryInfo;

#ifndef BROWSEIO
#define DBROWSEIO extern
#else
#define DBROWSEIO
#endif

/* calls the function to display progress during initial reading of image icons */
typedef void (*displayReadProgressProc) (
        int numFound /* number of valid image files found so far */
	);

/* registers the function to display the progress during reading image directories */
DBROWSEIO void registerBrowseIconProgress(displayReadProgressProc readProgress);

/* set subdirectory browsing on or off */
DBROWSEIO void setImageBrowseIncludeSubDir(int set);

/* enable printout of filename before it is loaded on or off */
DBROWSEIO void setImageBrowseDebug(int set);

/* gets the variable which keeps the number of days that unused icons should be kept in cache */
/* should be called and variable set before widget initialisation */
DBROWSEIO int *getIconCacheDays(void);

/* return string with current browse path */
DBROWSEIO char *getImageBrowsePath(void);

/* return string with current browse filename (with path) */
DBROWSEIO char *getImageBrowseFile(void);

/* return the maximum size of image icons */
DBROWSEIO int getImageIconSize(void);

/* returns the pointer to data of image used of icon creation and full screen image display */
DBROWSEIO unsigned char *getBrowseImageData(void);

/* frees the data data of image used of icon creation and full screen image display */
DBROWSEIO void freeBrowseImageData(void);

/* returns the pointer to the directory which contains image icons */
DBROWSEIO char *getImageIconPath(void);

/* delete all icons which were not accessed since clearIconCacheDays */
DBROWSEIO void clearIconCache(int clearIconCacheDays);

/* initialises image browsing library */
DBROWSEIO void initBrowseIO(void);

/* check can remove old entries from icon cache on average every 255nd  run */
DBROWSEIO void clearImageIconCacheSometimes(void);

/* you must set the browse path prior to mapping the browse window */
DBROWSEIO void setBrowsePath(char *path);

/* generic function for reversing images in Y direction */
DBROWSEIO void reverseImageY(unsigned char *data, int bpl, int sizeY);

/* return the directory name for the given ID */
DBROWSEIO char *getImageDir(int numDir);

/* return the number of directories in the current directory */
DBROWSEIO int getImageDirNum(void);

/* return the subdirectory path for the given ID */
DBROWSEIO char *getImageSubDir(int numDir);

/* return the number of currently loaded subirectories */
DBROWSEIO int getImageSubDirNum(void);

/* loads the image in the browser's internal image and creates the corresponding icon */
DBROWSEIO int loadImageMakeIcon(imageInfo *iI, aColor *browseImageC, int *browseImageBPL);

/* loads the image icon, for the cases when we are sure that the icon already exists in the cache */
DBROWSEIO int loadImageIcon(imageInfo *iI);

/* return true if file .nomedia does not exist in given directory */
DBROWSEIO int mediaDir(char *dir);

/* swaps two image icon structurs */
DBROWSEIO void swapImageIcon(imageInfo *i1, imageInfo *i2);

/* free the image info data */
DBROWSEIO void freeImageI(imageInfo *iI);

/* for loading of icons which involves loading of images we must provide colormap before calling loadImageI() */
DBROWSEIO void setImageBrowseCmap(aColor *browseImageC);

/* load the image into image info structure (if loadable) and create the icon, or load it from icon cache */ 
DBROWSEIO int loadImageI(char *filen, imageInfo *iI);

/* load all the images in readable formats from specified directory to the browser */
/* it allocates and sets the imageI icon stucture array, fills in dirB array of directory names, */
/* sets returns number of loaded images and number of directories */
/* browseImageC is a pre-allocated 256 array of aColor needed for loading of colormapped images */
DBROWSEIO void loadImageIcons(char *loadPath, imageInfo **imageI, int *numIconImages, directoryInfo **dirInfo, aColor *browseImageC, int iconDisplayBits);

/* take 32 bytes hex string and insert a "/" or "\\" (Windows case) after second byte, so that we divide */
/* icon cache in 256 directories and icon name is 2 bytes shorter, as subdir is the first part of the name */
DBROWSEIO void insertHexDir(char *md5);

/* provide a way to replace and make a better icon, in cases when image loader is not able make good icon */
/* this is useful mainly for ArahDrape, where drape project is rendered, and image browser is unable to do it on its own */
DBROWSEIO void updateAndSaveImageIcon(imageInfo *iI, char *path);

/* function reduces given image to a smaller icon. All data must be allocated. If source image is of type LOAD_BW */
/* program generates grayscale 8-bit icon; otherwise the icon is always 24 bits per pixel */
/* usedColors returns number of used colors, if cmap image */
DBROWSEIO int reduceImageToIcon(unsigned char *srcData, int srcBPL, int srcX, int srcY, int srcType, aColor *srcCol,
                               unsigned char *dstData, int dstBPL, int dstX, int dstY, unsigned char *usedColors);

/* compress the icon data */
DBROWSEIO void compressIconData(unsigned char **iconData, int iconLen, unsigned short *compressedIconLen);

/* renders the icon image into the passed buffer in the specified image depth, repeat mode and zoom */
DBROWSEIO int renderBrowseIconImage(unsigned char *iconData, unsigned short compressedIconLen,
 unsigned char iconType, unsigned char iconX, unsigned char iconY, unsigned char *iconCmap,
 int iconSize, int magnifyIcon, int dispXY, int noRepeatImage, int bytesPerPixel, unsigned char *dstData, int dstbpl);

/* directory comparison function */
DBROWSEIO int dirBrowseCmp(const void *arg1, const void *arg2);

/* count the number of directories and files in the given path */
DBROWSEIO void countDirFiles(const char *loadPath, int *numDir, int *numFiles);

/* return true if image browser is currently loading the image, so we don't start another loading from the main program */
DBROWSEIO int imageBrowserNowLoading(void);

#ifndef BRUTIL
#define DBRUTIL extern
#else
#define DBRUTIL
#endif

/* Compute the md5 hash on a buffer and return md5 as 32 bytes long ascii encoded hex string */
DBRUTIL char *md5string(const unsigned char *buf, int buflen);


/* save the information about recently read or written files. */
/* we pass the history file, since the same functions are used for files of types */
/* pass maxHistory to prevent reading the list files over the allocated space */
DBRUTIL void saveRecentFile(char *fileName, int onlyRead, char *historyFile, int maxHistory);

/* get the current recently used file history */
/* separate sections for files which have been written, read, and the directories */
/* list of read and written files are used from the menu of recently used files, while list of directories is used for browsers */
/* eliminate the files which can not be accessed at the time of reading */
/* list of arrays to pointers for readFileList and saveFileList must be pre-allocated and empty */
/* pass maxHistory to prevent reading the list files over the allocated space */
/* getRecentFiles already eliminates files, which can not be accessed */
DBRUTIL int getRecentFiles(char **readFileList, char **writeFileList, char **dirList, char *historyFile, int maxHistory);

/* clear the history of recently saved or loaded files */
DBRUTIL void clearRecentFiles(char *historyFile);

/* in-memory image icon reduction by integer divisor factors, copying of reduced icon to new location in selected RGB packing mode */
/* the source icon is damaged due to in memory reduction, so icon must be created just for this purpose and discarded afterwards */
/* source icon must always be in 24 bpp r,g,b mode */
/* parameters wid, hei are the final reduced icon size, so the input icon size must be at least wid*divideFactor and hei*divideFactor */
/* function does not perform any range checking or memory allocations - all sizes must be big enough */
/* paddedWid is only used for copying the icon into destination, where it is made wider by that area and filled with backgound color */
/* in this way icons of various widht can be made to fill the same widht and line up vertically in the menu */
DBRUTIL void reduceIconByFactor(int divideFactor, unsigned char *iconData, int iconBPL, int wid, int hei, int paddedWid,
 unsigned char *backgroundRGB, unsigned char *resultData, int resultBPL, int resultBitsPerPixel);

/* RGB to HSL transformation by Ken Fiskhin of Pixar, Graphics Gems I p. 763 */
/* r,g,b: 0..255, h: 0..359, s,l: 0..1000 */
DBRUTIL void RGBtoHSL (int *rgb, int *hsl);

/* HSL to RGB transformation by Ken Fiskhin of Pixar, Graphics Gems I p. 763 */
/* r,g,b: 0..255, h: 0..359, s,l: 0..1000 */
DBRUTIL void HSLtoRGB (int *hsl, int *rgb);

#endif /* __BROWSEIO_H_ */
