/* loadimg.h */
#ifndef _LOADIMG_H_
#define _LOADIMG_H_

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

#ifndef min
#define max(a,b) 	((a)<(b) ? (b) : (a))
#define min(a,b) 	((a)>(b) ? (b) : (a))
#endif

#define LOAD_BW			0 /* 1=black, 128 = 10000000 binary = leftmost pixel black */
#define LOAD_CMAP		1 /* colormapped image; up to 256 colors */
#define LOAD_TRUE_COL		2 /* rgbrgbrgb... probably should be quantized */
#define LOAD_GREYSCALE		3 /* not really suported, just for conversion to CMAP */
#define LOAD_DEFAULT		4 /* calling program can specify it, if default image type is desired, not one of the above */
#define LOAD_GREY		5 /* a new load type, just for icons 8-bit greyscale without colormap */
#define LOAD_BLACKLIST		6 /* fake image iconType, to signal damaged image file which can not be loaded */
#define LOAD_RGBA		7 /* rgbargba... RGB with 8 bit alpha channel */
#define LOAD_LAB		8 /* LabLab... L scaled from 0-100 to 0-255, a and b scaled from +-127 to 0-255 */

#define MAX_LOAD_IMAGE	65520 /* maximal x and y dimensions of images we load with this library */
#define MIN_IMAGE_FILSIZE    20 /* assume we can't have smaller image files than this, so reject them on loading */

#define LOAD_OK               0
#define LOAD_USER_ABORT       1
#define LOAD_NO_MEMORY        2
#define LOAD_IO_ERROR         3
#define LOAD_INTERNAL_ERROR   4
#define LOAD_WRONG_IMG        5
#define LOAD_FILETYPE_UNKNOWN 6
#define LOAD_FILE_CORRUPT     7
#define LOAD_NOT_IMAGE        8
/* NOTE! remember to add new image type also in imageTypeColors in browsei.c */

/* defines 0-6 are equal to PRT_OK .. PRT_WRONG_IMG */
#define LTXT_WAIT		10
#define LTXT_LOADING_WAIT	11
#define LTXT_CANCEL_LOAD	12

#define MAX_LTXT 		20

#define UNKNOWN_LOADER	255

/* colors for checkered transparency */
#define DARK_CHECK	102
#define LIGHT_CHECK	153

/* checks if image is in selected format and reads the header */
/* TRUE on success, FALSE otherwise */
typedef int (*tryLoadProc) (
	char *filename,
	int *loadType, /* LOAD_BW ... LOAD_TRUE_COL */
	int *xpixels,
	int *ypixels,
	int *xdpi, /* horizontal dots per inch *1000 */
	int *ydpi, /* vertical dots per inch *1000 */
	unsigned char *cmap, /* caller pre-allocated return colormap */
	char *info, /* caller preallocated loader info only 500 bytes!!! */
	char *reverse, /* return TRUE if bottom line is first in this format */
	int *numInterlace, /* number of interlace passes, 1 for no interlace */
	int *coreFile, /* FALSE only for extra files, making part of a multiple-file format design */
	int realLoad /* TRUE if we are really loading, FALSE if we just want info */
	);

/* loader reads a band */
typedef int (*loadBandProc) (
	unsigned char *data, /* put the data in here */
	int  lines, /* number of lines in this band */
        int  bpl /* bytes per line */
	);

/* closes the input file and releases any temporary storage */
typedef int (*endLoadProc) ();

/* initializes the destination data for loading */
/* TRUE on success, FALSE otherwise */
typedef int (*initConsumeProc) (
	int loadType, /* LOAD_BW ... LOAD_TRUE_COL */
	int xpixels,
	int ypixels,
	int xdpi, /* horizontal dots per inch *1000 */
	int ydpi, /* vertical dots per inch *1000 */
	char reverse, /* TRUE if producer wants to feed image in reverse y order */
	int numInterlace, /* # of interlace passes; if different than 1 then band height must be whole image */
	unsigned char *cmap, /* caller pre-allocated colormap */
	unsigned char **data, /* allocate destination data */
	int *bpl, /* bytes per line */
	int *bandh, /* height of a band */
	int *progress /* advance in the data buffer or stay put */
	);

/* consume a band */
typedef int (*consumeBandProc) (
	unsigned char *data, /* get data from here */
	int  lines, /* number of lines in this band */
        int  bpl, /* bytes per line */
        int  ypos /* y position */
	);

/* frees any temporary storage */
typedef int (*endConsumeProc) (int status);

extern void registerLoadGIF(void);
extern void registerLoadPNG(void);
extern void registerLoadPCX(void);
extern void registerLoadJPEG(void);
extern void registerLoadTIFF(void);
extern void registerLoadPBM(void);
extern void registerLoadWIF(void);
extern void registerLoadBMP(void);

#define NUM_LOADERS	9 /* must match the number of functions above + 1 */

extern void registerLoader(tryLoadProc tLoad, loadBandProc lBand, endLoadProc eLoad);

extern void initImageLoadLibrary();

extern void initLoadFabric();
extern void initLoadWeave();

/* P U B L I C   F U N C T I O N S */

/* if numLoader is negative, returns number of loader functions, otherwise selects the */
/* one which is passed as parameter */
/* loader 0 is uses heuristics on filename suffix to use the most probable loader first. */
/* if no such loader can be found, loader "0" returns "-1" and you should not use it */
/* and keep calling loaders 1...NUM_LOADERS */
/* if a valid heuristics cna be used, loader 0 returns the number of loader which it is using */
int returnLoaderFunctions(int numLoader, char *filename);

/* returns the string that should explain the loader's file type */
char *returnLoaderString(int numLoader);

/* returns the string that should explain the loader's file type, but without text which is repeated in logo */
char *returnLoaderStringNoLogo(int numLoader);

/* returns the string that which contains the logo filename string */
char *returnLoaderStringJustLogo(int numLoader);



#ifndef LUTIL
#define DLUTIL extern
#else
#define DLUTIL
#endif

/* set and get functions for temporary directory, gzip and unzip commands */
DLUTIL void llibSetTmpDir(char *value);

DLUTIL void llibSetGzipCommand(char *value);

DLUTIL void llibSetUnzipCommand(char *value);

DLUTIL char *llibGetTmpDir(void);

DLUTIL char *llibGetGzipCommand(void);

DLUTIL char *llibGetUnzipCommand(void);

/* removes keyword identifier in a line of settings; delimiter is '=' */
DLUTIL char *removeHead(char *inpstr);

/* used to remove last character, like /n */
DLUTIL char *removeLast(char *textString);

/* reverse and, if necessary, negate bytes in the passed data field */
DLUTIL void reverseNegateBytes(unsigned char *data, int length, int negate);

/* reimplementation of fgets which reads MSDOS, UNIX and DOS ASCII files (end of line mandess) */
DLUTIL char *fgetsDosMac(char *inpStr, int limit, FILE *fp);

/* simple ascii version of tolower, since on Windows it does not work properly, and for our needs converting A-Z to a-z is enough */
DLUTIL char tolowerAZ(char in);

/* convert to lowercase*/
DLUTIL void stringToLower(char *line);

/* unified function to generate real view image for both screen and print in any depth */
/* there are no implicit parameters, can be extraced and used as generic function */
/* some repetition of code, but can not afford a switch within inner loop */
DLUTIL void renderRealImg(unsigned char *dstData, int dstBPL, int bytesPerPixel,
 int xpos, int ypos, int width, int height, int firstBottomPix, int dstxdpi, int dstydpi,
 unsigned char *srcData, int srcBPL, int srcWid, int srcHei, int srcImageType,
 unsigned char *srcCmap, int srcxdpi, int srcydpi, float zoomFactor);

/* generate a list of points which are randomly distributed across the image */
/* but with similar distance to other points, with random oscillation */
/* extremely high values of numPoints should be avoided, as it becomes very slow */
/* caller must test the result and free the allocated array */
DLUTIL unsigned short *selfAvoidTable(int numPoints, int wid, int hei);

/* stretch the contrast on cmap or true color images. */
/* on true color images modify the image, on colormap only modify the colormap */
DLUTIL void contrastStretch(unsigned char *imageData, int imageBPL, int imageType, int wid, int hei, unsigned char *cmap);

/* interpret background as white or check and convert RGBA to RGB in memory, also realloc memory */
DLUTIL unsigned char *collapseRGBA(unsigned char *data, int wid, int hei, int *bpl, int doCheck);

/* examines the line looking for <tag>value</tag> and return pointer to value, or NULL if not found; will modify line */
DLUTIL char *parseOneLineTagXML(char *line, char *tag);

/* tuneable color quantization function: trueData is input data in RGB, cmapData is colormapped
   output image. You set the requested number of colors, and the program returns the actual
   number. You can previously put in the colormap the colors you want to have
   in the image (regardless of histogram), and indicate this by setting the number of fixColors>0.
   If cmapData is NULL, then we just want back the reduced colormap, without remapping the image.
   If trueData is NULL, then we are doing quantization of colormapped image; do not actually
   change the data, ony return the colormap and actual colors; not yet implemented */
int colorQuantize(unsigned char *trueData, int trueBPL, unsigned char *cmapData, int cmapBPL,
 int sizeX, int sizeY, unsigned char *cmap, int fixColors, int *requestColors);

/* remap a true color image to colormap image, using the passed colormap, without any colormap compression */
/* dither noise controls the level of random dithering, valid values ar 0-100 */
int trueColorRemapToCmap(unsigned char *trueData, int trueBPL, unsigned char *cmapData, int cmapBPL,
 int sizeX, int sizeY, unsigned char *cmap, int cmapColors, int ditherNoise);

/* remap a true color image to palette image using give colormap and Floyd/Steinberg dithering */
/* for finding closest color, use color cube mapping instead of search throught all palette colors for each pixel */
/* ditherIntensity is 16-0, 16 is normal dither, puts less dithering transitino, and 0 is no dithering */
/* ditherNoise is level of noise for dithering, 0 for no noise, 100 is maximum; will add random values of +-100 */ 
void colorQuantizeDitherFS(unsigned char *trueData, int trueBPL, unsigned char *cmapData, int cmapBPL,
 int srcX, int srcY, int dstX, int dstY, unsigned char *cmap, int cmapColors, int ditherIntensity, int ditherNoise);

/* same as above, but with dithering when we have more than 256 input colors; will  give result as 16 bit pallete index, instead of 8 bit */
/* so cmapData is unsigned short instead of unsigned char */ 
void colorQuantizeDitherFS16(unsigned char *trueData, int trueBPL, unsigned short *cmapData, int cmapBPL,
 int srcX, int srcY, int dstX, int dstY, unsigned char *cmap, int cmapColors, int ditherIntensity, int ditherNoise);

#endif /* _LOADIMG_H_ */
