/***************************************************************************
                          apaintdraw.cpp  -  description
                             -------------------
    begin                : Mon Mar 10 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "apaintdraw.h"
#include "apaintview.h"
#include "apaintBrushList.h"


ApaintDraw::ApaintDraw(ApaintDoc *docp, ApaintView *viewp)
{                      
 docIsLoaded=false;
 zoomX=zoomY=1;
 Q_CHECK_PTR(docp);
 Q_CHECK_PTR(viewp);
 doc = docp;
 view = viewp;
 drawFatLines = false;
 maxRectList.setAutoDelete(true);
 connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer *)));
 connect(doc, SIGNAL(refreshSig(int, int, int, int)), this,  SLOT(screenRefresh(int, int, int, int)));
 connect(doc, SIGNAL(refreshLayerSig(const QRect&, const QRect&, int)), this, SLOT(screenRefresh(const QRect&, const QRect&, int)));
 resetCtrlData();
}

ApaintDraw::~ApaintDraw()
{
}

  
void ApaintDraw::layerChangedSlot(Layer *sel)
{
  if (!sel)
  {
    docIsLoaded=false;
    return;
   }  
  docIsLoaded=sel->isValid();
}

void ApaintDraw::zoomChangedSlot(double zx, double zy)
{
  zoomX=zx;
  zoomY=zy;
}

QRect ApaintDraw::getDrawArea()
{
  return view->getContentsSize();
}

QRect ApaintDraw::getScaleRect(QRect norm)
{
  int x1, y1, x2, y2;
  
  x1=(int)(norm.x()*zoomX);
  y1=(int)(norm.y()*zoomY);
  x2=(int)((norm.right()+1)*zoomX);
  y2=(int)((norm.bottom()+1)*zoomY);
  return QRect(x1, y1, x2-x1, y2-y1);
}

QRect ApaintDraw::getScaleRect(int x, int y, int wid, int hei)
{
  int x1, y1, x2, y2;

  x1=(int)(x*zoomX);
  y1=(int)(y*zoomY);
  x2=(int)((x+wid)*zoomX);
  y2=(int)((y+hei)*zoomY);
  return QRect(x1, y1, x2-x1, y2-y1);
}

void ApaintDraw::refreshImg(int x, int y, int wid, int hei)
{  
  QRect *nr;
  QRect sr;
  Layer *selLayer;
  selLayer = doc->getLayer();
  if (!selLayer) return;
  
  if (selLayer->isUsingTiles())
  {
   QPtrList<QRect>& rList = selLayer->getLayerRefreshArea(QRect(x, y, wid, hei));
   for (int j =0; j<(int)rList.count(); j++)
   {
     nr = rList.at(j);
     screenRefreshRepeat(selLayer, nr->x(), nr->y(), nr->width(), nr->height());
   }
  }
  else
  {
    sr = getScaleRect(x, y, wid, hei);
    view->refresh(sr.x(), sr.y(), sr.width(), sr.height());
  }
}

void ApaintDraw::screenRefresh(int x, int y, int w, int h, int border)
{
  if (!docIsLoaded) return;

  QRect ref = getScaleRect(x, y, w, h);
  view->refresh(ref.x()-border, ref.y()-border, ref.width()+2*border, ref.height()+2*border);
}

void ApaintDraw::screenRefresh(int x, int y, int w, int h)
{
 screenRefresh(x, y, w, h, 0) ;
}

void ApaintDraw::screenRefreshRepeat(int x, int y, int w, int h)
{
 screenRefreshRepeat(doc->getLayer(), x, y, w, h);
}

void ApaintDraw::screenRefreshRepeat(Layer *layer, int x, int y, int w, int h)
{
 if (!layer) return;
 if (layer->isUsingTiles())
 {
   QRect tileRect, tileRectSec, sr;
   Tile *tile;
   int sx, sy;
   QSize imgs = QSize(layer->imageBuf.width(), layer->imageBuf.height());
   QPtrList<Tile>& list = layer->getTileList(); 
   for (int i = 0; i< (int)list.count(); i++)
   {
       tile = list.at(i);
       tileRect = tile->getPosMain();
       tileRectSec = tile->getPosSecond();
       sx = tileRect.x();
       sy = tileRect.y();
       if (tile->mirrorHor())
          sx+=imgs.width()-x-w;
       else
          sx+=x;
       if (tile->mirrorVert())
          sy+= imgs.height()-y-h;
       else
          sy+=y;
       sr = getScaleRect(sx, sy, w, h);
       view->refresh(sr.x(), sr.y(), sr.width(), sr.height());
       
       /*refresh second rect*/
       if ((sx+w>tileRect.right()+1) && tileRectSec.isValid())
       {
         int nw;
         nw =sx+w-tileRect.right()-1;
         sr = getScaleRect(0, sy, nw, h);
         view->refresh(sr.x(), sr.y(), sr.width(), sr.height());
       }
       else if ((sy+h>tileRect.bottom()+1) && tileRectSec.isValid())
       {
         int nh;
         nh =sy+h-tileRect.bottom()-1;
         sr = getScaleRect(sx, 0, w, nh);
         view->refresh(sr.x(), sr.y(), sr.width(), sr.height());
       }
   }
  }
}

void ApaintDraw::refreshIntersectedArea(const QRect &oldRec, const QRect &newRec, int border)
{
  int x1,y1,x2,y2;
  int xo1, yo1, xo2, yo2;

  x1=newRec.x();
  y1=newRec.y();
  x2= newRec.x()+newRec.width();
  y2= newRec.y()+newRec.height();
  
  xo1=oldRec.x();
  yo1=oldRec.y();
  xo2=oldRec.x()+oldRec.width();
  yo2=oldRec.y()+oldRec.height();    

  screenRefresh(x1, y1, x2-x1, y2-y1, border);
  
  if (xo1<x1)
    screenRefresh(xo1, yo1, x1-xo1, yo2-yo1, border);

  if (yo1<y1)
   screenRefresh(xo1 ,yo1 , xo2-xo1, y1-yo1, border);

  if (xo2>x2)
   screenRefresh(x2, yo1, xo2-x2, yo2-yo1, border);

  if (yo2>y2)
   screenRefresh(xo1, y2, xo2-xo1, yo2-y2, border);
    
}

void ApaintDraw::screenRefresh(const QRect &oldRec, const QRect &newRec, int border)
{
 refreshIntersectedArea(oldRec, newRec, border);
}

void ApaintDraw::refreshDrawImage(QRect &ref)
{
 int x1,y1,x2,y2;
  int xo1, yo1, xo2, yo2;

  x1=ref.x();
  y1=ref.y();
  x2= ref.x()+ref.width();
  y2= ref.y()+ref.height();

  xo1=oldRefresh.x();
  yo1=oldRefresh.y();
  xo2=oldRefresh.x()+oldRefresh.width()+1;
  yo2=oldRefresh.y()+oldRefresh.height()+1;

  refreshImg(x1, y1, x2-x1, y2-y1);

  if (xo1<x1)
    refreshImg(xo1, yo1, x1-xo1, yo2-yo1);

  if (yo1<y1)
   refreshImg(xo1 ,yo1 , xo2-xo1, y1-yo1);

  if (xo2>x2)
   refreshImg(x2, yo1, xo2-x2, yo2-yo1);

  if (yo2>y2)
   refreshImg(xo1, y2, xo2-xo1, yo2-y2);
} 

void  ApaintDraw::clearRectSlot(QRect imgArea, QRect drawArea)
{  
  if (!docIsLoaded) return;
  Layer *selLayer;
  selLayer = doc->getLayer();
  if (!selLayer) return;
  
  QPainter p(selLayer->getPixmap());
  QRect *nr, pnr;
  selLayer = doc->getLayer();
  if (!selLayer) return;
  double ax = doc->getAspectX();
  double ay = doc->getAspectY();
  
  QRect view=selLayer->layerPos().intersect(imgArea);
  QRect darea = selLayer->drawAreaPos().intersect(drawArea);
  
  if (selLayer->isUsingTiles())
  {
   QPtrList<QRect>& rList = selLayer->getLayerRefreshArea(view);
   for (int i=0; i<(int)rList.count(); i++)
   {
     nr = rList.at(i);
     pnr = QRect((int)(nr->x()*ax), (int)(nr->y()*ay), (int)(nr->width()*ax), (int)(nr->height()*ay));
     refreshOldData(nr->x(), nr->y(), nr->right()+1, nr->bottom()+1, selLayer);
     screenRefreshRepeat(nr->x(), nr->y(), nr->width(), nr->height());
     p.eraseRect(pnr.x(), pnr.y(), pnr.width(), pnr.height());
   }
  }
  else
  {
   refreshOldData(view.x(), view.y(), view.right()+1, view.bottom()+1, selLayer);
   screenRefresh(view.x(), view.y(), view.width(), view.height());
   p.eraseRect(darea.x(), darea.y(), darea.width(), darea.height());
  }

}

void ApaintDraw::refreshRBand(QRect ref, QRect drawArea, bool useFg, bool finalDraw)
{  
  if (!docIsLoaded) return;
  int i;
  QRect *nr, pnr;
  Layer *selLayer;
  selLayer = doc->getLayer();
  if (!selLayer) return;
  
  double ax = doc->getAspectX();
  double ay = doc->getAspectY();
  
  QRect view = selLayer->layerPos().intersect(ref);
  QRect drawRec = selLayer->drawAreaPos().intersect(drawArea);

  if (oldRefresh.isValid())
  {
    if (selLayer->isUsingTiles())
    {
      QPtrList<QRect>& oldr = selLayer->getLayerRefreshArea(oldRefresh);
      for (i=0; i<(int)oldr.count(); i++)
      {
       nr = oldr.at(i);
       refreshOldData(nr->left(), nr->top(), nr->right()+1, nr->bottom()+1, selLayer);
      }
    }
    else
    {
     refreshOldData(oldRefresh.left(), oldRefresh.top(), oldRefresh.right()+1, oldRefresh.bottom()+1, selLayer);
    }
  }
  else
    oldRefresh = view;

  QPtrList<QRect>& newr = selLayer->getLayerRefreshArea(ref);
  if (selLayer->isUsingTiles())
  {    
    for (i=0; i<(int)newr.count(); i++)
    {
      nr = newr.at(i);
      pnr = QRect((int)(nr->x()*ax), (int)(nr->y()*ay), (int)(nr->width()*ax), (int)(nr->height()*ay));
      drawToImage(pnr, *nr, useFg, finalDraw, false);
    }
  }
  else
   drawToImage(drawRec, view, useFg, finalDraw, false);
      
  
  refreshDrawImage(ref);
  oldRefresh = view;
  
  if (finalDraw)
  {
    resetAndUndo(drawRec, view);
    if (selLayer->isUsingTiles())
    {
      for (i=0; i<(int)newr.count(); i++)
      {
       nr = newr.at(i);
       finalizeDraw(nr->x(), nr->y(), nr->width(), nr->height());
      }
    }
    else
    {
     finalizeDraw(view.x(), view.y(), view.width(), view.height()); 
    }
  }   
}

void ApaintDraw::refreshImageDraw(int x, int y, int wid, int hei, bool finalDraw)
{  
  if (!docIsLoaded) return;
  QRect *nr;
  Layer *selLayer;
  selLayer = doc->getLayer();
  if (!selLayer) return;

  double ax = doc->getAspectX();
  double ay = doc->getAspectY();
  
  QRect ref(x, y, wid, hei);

  QRect view=selLayer->layerPos().intersect(ref);
  QRect drawRec = QRect((int)(view.x()*ax), (int)(view.y()*ay), (int)(view.width()*ax), (int)(view.height()*ay));
    
  
  setGlobalMaxRect(view, drawRec);
  
  if (selLayer->isUsingTiles())
  {
    QPtrList<QRect>& rList = selLayer->getLayerRefreshArea(view);
    for (int i=0; i<(int)rList.count(); i++)
    {
      nr = rList.at(i);
      screenRefreshRepeat(nr->x(), nr->y(), nr->width(), nr->height());
      setMaxRect(i, *nr);
    }
    
    if (finalDraw)
    {
       resetAndUndo(globalMaxDraw, globalMaxRef);
       for (int i=0; i<(int)maxRectList.count(); i++)
       {
         nr = rList.at(i);
         finalizeDraw(nr->x(), nr->y(), nr->width(), nr->height());
       }
       maxRectList.clear();
       globalMaxRef = QRect();
       globalMaxDraw = QRect();
    }    
  }
  else
  {
    screenRefresh(view.x(), view.y(), view.width(), view.height());    

    if (finalDraw)
    {
     resetAndUndo(globalMaxDraw, globalMaxRef);
     finalizeDraw(globalMaxRef.x(), globalMaxRef.y(), globalMaxRef.width(), globalMaxRef.height());
     globalMaxRef = QRect();
     globalMaxDraw = QRect();
    }   
  }
  
}

void ApaintDraw::refreshDirectDraw(QRect ref, bool useFg, bool doDraw, bool finalDraw)
{   
  if (!docIsLoaded) return;
  QRect *nr, pnr;
  Layer *selLayer;
  selLayer = doc->getLayer();
  if (!selLayer) return;

  double ax = doc->getAspectX();
  double ay = doc->getAspectY();
  int  dx, dy;
  dx = dy = 0;
  if (ax>1) dx = 1;
  if (ay>1) dy = 1;

  QRect view=selLayer->layerPos().intersect(ref);
  QRect drawRec = QRect((int)(view.x()*ax), (int)(view.y()*ay), (int)(view.width()*ax)+dx, (int)(view.height()*ay)+dy);
    
  setGlobalMaxRect(view, drawRec);

  if (selLayer->isUsingTiles())
  {
    QPtrList<QRect>& rList = selLayer->getLayerRefreshArea(globalMaxRef);
    for (int i=0; i<(int)rList.count(); i++)
    {
     nr = rList.at(i);
     pnr = QRect((int)(nr->x()*ax), (int)(nr->y()*ay), (int)(nr->width()*ax), (int)(nr->height()*ay));
     if (doDraw)
     {
      refreshOldData(nr->left(), nr->top(), nr->right()+1, nr->bottom()+1, selLayer);
      drawToImage(pnr, *nr, useFg, true, true);
     }
     screenRefreshRepeat(nr->x(), nr->y(), nr->width(), nr->height());
     setMaxRect(i, *nr);
    }  

    if (finalDraw)
    {
      resetAndUndo(globalMaxDraw, globalMaxRef);
      for (int i=0; i<(int)maxRectList.count(); i++)
      {
         nr = rList.at(i);
         finalizeDraw(nr->x(), nr->y(), nr->width(), nr->height());
      }
      maxRectList.clear();
      globalMaxRef = QRect();
      globalMaxDraw = QRect();
    }
  }
  else
  {
    if (doDraw)
    {
      refreshOldData(globalMaxRef.left(), globalMaxRef.top(), globalMaxRef.right()+1, globalMaxRef.bottom()+1, selLayer);
      drawToImage(globalMaxDraw, globalMaxRef, useFg, true, true);
    }
    screenRefresh(globalMaxRef.x(), globalMaxRef.y(), globalMaxRef.width(), globalMaxRef.height());
    if (finalDraw)
    {
     resetAndUndo(globalMaxDraw, globalMaxRef);
     finalizeDraw(globalMaxRef.x(), globalMaxRef.y(), globalMaxRef.width(), globalMaxRef.height());
     globalMaxRef = QRect();
     globalMaxDraw = QRect();
    } 
  }  
}

void ApaintDraw::doFatLines(QImage &img, QPainter *p, int xs, int ys, bool final)
{
 int x, y;

 for (y = 0; y< img.height()-1; y++)
  for (x = 0; x< img.width()-1; x++)
  {
   if (PIX_ON(img, x, y) && PIX_ON(img, x+1, y+1) && !PIX_ON(img, x, y+1) && !PIX_ON(img, x+1, y))
   {
    img.setPixel(x, y+1, 1);
    if (final)
      p->drawPoint(xs+x, ys+y+1);
   }
   else if (PIX_ON(img, x, y+1) && PIX_ON(img, x+1, y) && !PIX_ON(img, x, y) && !PIX_ON(img, x+1, y+1))
   {
    img.setPixel(x, y, 1);
    if (final)
      p->drawPoint(xs+x, ys+y);
   }    
  }
}

void ApaintDraw::drawToImage(QRect srcRec, QRect &dstRec, bool useFg, bool final, bool useRelativ)
{
  QImage img1Bit;  
  int x, y, xs, ys, wids, heis, srcx, srcy;
  uchar *bitLine, *imgLine;
  uchar **dataBit, **imgData;
  uchar colIndx;
  uint rcol, bcol, gcol;
  QPixmap pixRect;
  Layer *selLayer;
  ApaintBrushList *bList = ApaintBrushList::getObject();
  
  selLayer = doc->getLayer();
  if (!selLayer) return;
  
  colIndx = 0;
  rcol = bcol = gcol =0;  
  xs = dstRec.x() - selLayer->layerPos().x();
  ys = dstRec.y() - selLayer->layerPos().y();
  wids = dstRec.width();
  heis = dstRec.height();
  srcx = srcRec.x() - selLayer->drawAreaPos().x();
  srcy = srcRec.y() - selLayer->drawAreaPos().y();  
  pixRect = QPixmap(srcRec.width(), srcRec.height(), 1);
  if (pixRect.isNull()) return;
  bitBlt(&pixRect, 0, 0, selLayer->getPixmap(), srcx, srcy, srcRec.width(), srcRec.height());
  img1Bit=pixRect.convertToImage();

  /* this code should be obsolete once we correct drawing with different pen x and y size */
  if ((srcRec.width() != wids) || (srcRec.height() != heis))
   {
    img1Bit=scaleQImage(srcx, srcy, img1Bit, selLayer->drawAreaPos(), selLayer->layerPos(), 
                        selLayer->getPixSrcZoomX(), selLayer->getPixSrcZoomY(), xs, ys, wids, heis);
    if (!wids || !heis)
      return; 
    dstRec = QRect(xs, ys, wids, heis);    
   }
     
  imgData=selLayer->imageBuf.jumpTable();
  dataBit=img1Bit.jumpTable();

  if (drawFatLines)
   {
    QPainter p(selLayer->getPixmap());   
    doFatLines(img1Bit, &p, srcx, srcy, final);
   }

  if (bList->useBrush())
   {
    drawToImageWithBrush(srcRec, dstRec, useRelativ,  bList, img1Bit, selLayer);
    return;
   }
    
  if (selLayer->getImageType() == LOAD_TRUE_COL)
   {
    if (useFg)
     {
      rcol = redfg;
      bcol = bluefg;
      gcol = greenfg;
     }
    else
     {
      rcol = redbg;
      bcol = bluebg;
      gcol = greenbg;
     }   
   }
  else if (selLayer->getImageType() == LOAD_CMAP)
  {
    if (useFg)
    colIndx = colorIndexFg;
   else
    colIndx = colorIndexBg;
  }

  switch (selLayer->getImageType())
  {
    case LOAD_BW:
    for (y=0; y<heis; y++)
     {
      bitLine=*(dataBit+y);
      imgLine=*(imgData+y+ys);
      if (img1Bit.bitOrder()==QImage::LittleEndian)
       {
	for (x=0; x<wids; x++)
         if (*(bitLine+(x>>3)) & 1 << (x & 7))
           *(imgLine+(x>>3)) |= 1 << (x & 7);
         else
           *(imgLine+(x>>3)) &= 254 << (x & 7);              
       }
      else /*big endian*/
       {
	for (x=0; x<wids; x++)
         if (*(bitLine+(x>>3)) & 1<<(7-(x & 7)))
           *(imgLine+(x>>3)) |= 1<<(7-(x & 7));
         else
           *(imgLine+(x>>3)) &= 254 << (7 -(x & 7));
       }
     }
    break;

    case LOAD_CMAP:
     for (y=0; y<heis; y++)
      {
       bitLine=*(dataBit+y);
       imgLine=*(imgData+y+ys);
       if (img1Bit.bitOrder()==QImage::LittleEndian)
       {
        x=0; 
        while (x<wids)
        {
         if (*(bitLine+(x>>3))==0)
          {
           x+=8;
           continue;
          }
         if (*(bitLine+(x>>3)) & 1<<(x & 7))
          {
           if (!selLayer->isProtectedColor(*(imgLine+x+xs)))
            *(imgLine+x+xs)=(uchar)colIndx;
          }
         x++;
        }
       }
       else /*big endian*/
       {
        x=0; 
        while (x<wids)
         {
          if (*(bitLine+(x>>3))==0)
           {
            x+=8;
            continue;
           }            
          if (*(bitLine+(x>>3)) & 1<<(7-(x & 7)))
           {
            if (!selLayer->isProtectedColor(*(imgLine+x+xs)))
             *(imgLine+x+xs)=(uchar)colIndx;      
           }
          x++;             
         }
       }
      }
     break;

    case LOAD_TRUE_COL:
     for (y=0; y<heis; y++)
      {
       bitLine=*(dataBit+y);
       imgLine=*(imgData+y+ys)+(xs*3);
       if (img1Bit.bitOrder()==QImage::LittleEndian)
       {
        x=0;
        while (x<wids)
        {
         if (*(bitLine+(x>>3)) == 0)
          {
           x+=8;
           imgLine+=24;
           continue;
          }  
         if (*(bitLine+(x>>3)) & 1<<(x & 7))
          {
           *imgLine++=rcol;
           *imgLine++=gcol;
           *imgLine++=bcol;
          }
         else
          {
           imgLine++; imgLine++; imgLine++;
          }
         x++;
        }
       }
      else /*big endian*/
       {
        x=0;
        while (x<wids)
        {
          if (*(bitLine+(x>>3)) == 0)
           {
            x+=8;
            imgLine+=24;
            continue;
           } 
          if (*(bitLine+(x>>3)) & 1<<(7-(x & 7)))
           {
            *imgLine++=rcol;
            *imgLine++=gcol;
            *imgLine++=bcol;
           }
          else
           {
            imgLine++; imgLine++; imgLine++;
           }
          x++;
         }
       }
     }
   break;
  }
}

void ApaintDraw::drawToImageWithBrush(QRect srcRec, QRect dstRec, bool useRelativ, ApaintBrushList *bList, QImage &img1Bit, Layer *selLayer)
{
  uchar **imgData, **dataBit;
  uchar *bitLine, *imgLine, *brushLine, *brushMaskLine;
  int x, y, xsrc, ysrc, bwid, bhei, xsrcMask, xd, yd, xs, ys, wids, heis;
  int *transTable;
  uint *brushTab; 
  
  xs = dstRec.x() - selLayer->layerPos().x();
  ys = dstRec.y() - selLayer->layerPos().y();
  wids = dstRec.width();
  heis = dstRec.height();
  ApaintImage dimg = selLayer->imageBuf;  
  imgData=dimg.jumpTable();
  dataBit=img1Bit.jumpTable();

  ApaintImage brushImg = bList->getBrush();
  QImage brushMask = bList->getBrushMask();
  if (brushImg.isNull()) return;
  if (brushMask.isNull()) return;
  bwid = brushImg.width();
  bhei = brushImg.height();
  brushTab = brushImg.colorTable();
  transTable = bList->getTransTable();

  if (bList->getAbsOffset() || useRelativ)
   {
    xd = xs;
    yd = ys-dimg.height()%bhei;
   }
  else
   xd = yd = 0; 

  if (!bList->getAbsOffset())
   {
    int startOffsetX = INT_MAX;
    int startOffsetY = INT_MAX;

    /* find first bit which is set, top down */
     for (y=0; y<heis; y++)
      {
       bitLine=*(dataBit+heis-y-1);
       if (img1Bit.bitOrder()==QImage::LittleEndian)
	{
	 x=0;
	 while (x<wids)
	  {
	   if (*(bitLine+(x>>3))==0)
	    {
             x+=8;
             continue;
	    }
	   if (*(bitLine+(x>>3)) & 1<<(x & 7))
	    {
	     if (startOffsetX>x)
	      startOffsetX=x;
	     if (startOffsetY>y)
	      startOffsetY=y;
	    }
	   x++;
	  }
	}
       else
	{
	 x=0;
	 while (x<wids)
	  {
	   if (*(bitLine+(x>>3))==0)
	    {
             x+=8;
             continue;
	    }
	   if (*(bitLine+(x>>3)) & 128>>(x & 7))
	    {
	     if (startOffsetX>x)
	      startOffsetX=x;
	     if (startOffsetY>y)
	      startOffsetY=y;
	    }
	   x++;
	  }
	}
      }
    xd-=startOffsetX;
    yd = bhei - (heis % bhei) + startOffsetY;
   }

  xd += bList->getOffsetX();
  yd -= bList->getOffsetY();

  while (xd<0)
   xd+=bwid;
  while (yd<0)
   yd+=bhei;

//printf("abs %d start xd %d yd %d xs %d ys %d wids %d heis %d dstRec.x %d dstRec.y %d selLayer.x %d selLayer.y %d\n", bList->getAbsOffset(), xd, yd, xs, ys, wids, heis, dstRec.x(), dstRec.y(), selLayer->layerPos().x(), selLayer->layerPos().x());   
   if (dimg.type()==LOAD_CMAP)
    {
     //merge color tables
     if (bList->isColorTabDifferent())
      {
       doc->putUndoStack(ChangeColorTable);
       bList->mergeColorTables(selLayer);
       emit colorTableChanged(selLayer);
      }

     for (y=0; y<heis; y++)
      {
       bitLine=*(dataBit+y);
       imgLine=*(imgData+y+ys);
       ysrc = (y + yd)%bhei;
       brushLine = brushImg.scanLine(ysrc);
       brushMaskLine = brushMask.scanLine(ysrc);
       if (img1Bit.bitOrder()==QImage::LittleEndian)
       {
        x=0;
        xsrc = xd%bwid;
        while (x<wids)
        {
          if (*(bitLine+(x>>3))==0)
          {
            x+=8;
            xsrc = (x+xd)%bwid;
            continue;
          }
          if ((*(bitLine+(x>>3)) & 1<<(x & 7)) && (*(brushMaskLine+(xsrc>>3)) & 1<<(xsrc & 7)))
          {
            if (!selLayer->isProtectedColor(*(imgLine+x+xs)))
             *(imgLine+x+xs)=transTable[brushLine[xsrc]];
          }
          x++;
          xsrc = (x+xd)%bwid;
        }
       }
       else /*big endian*/
       {
        x=0;
        xsrc = xd%bwid;
        while (x<wids)
        {
          if (*(bitLine+(x>>3))==0)
          {
            x+=8;
            xsrc = (x+xd)%bwid;
            continue;
          }
          if ((*(bitLine+(x>>3)) & 128>>(x & 7)) && (*(brushMaskLine+(xsrc>>3)) & 1<<(xsrc & 7)))
            if (!selLayer->isProtectedColor(*(imgLine+x+xs)))
             *(imgLine+x+xs)=transTable[brushLine[xsrc]];
          x++;
          xsrc = (x+xd)%bwid;
        }
       }
     }
    }
   else /* LOAD_TRUE_COL */
    {
     for (y=0; y<heis; y++)
      {
        bitLine=*(dataBit+y);
        imgLine=*(imgData+y+ys)+(xs*3);
        ysrc = (y + yd)%bhei;
        brushLine = brushImg.scanLine(ysrc);
        brushMaskLine = brushMask.scanLine(ysrc);
        if (img1Bit.bitOrder()==QImage::LittleEndian)
         {
          if (brushImg.type() == LOAD_TRUE_COL)
           {
             x=0;
             xsrc = (xd%bwid)*3;
             xsrcMask = xd%bwid;
             while (x<wids)
              {
               if (*(bitLine+(x>>3)) == 0)
                {
                 x+=8;
                 xsrc = ((x+xd)%bwid)*3;
                 xsrcMask = (x+xd)%bwid;
                 imgLine+=24;
                 continue;
                }
               if ((*(bitLine+(x>>3)) & 1<<(x & 7)) && (*(brushMaskLine+(xsrcMask>>3)) & 1<<(xsrcMask & 7)))
                {
                 *imgLine++=brushLine[xsrc];
                 *imgLine++=brushLine[xsrc+1];
                 *imgLine++=brushLine[xsrc+2];
                }
               else
                { imgLine++; imgLine++; imgLine++; }
               x++;
               xsrc = ((x+xd)%bwid)*3;
               xsrcMask = (x+xd)%bwid;
             }
           }
           else if (brushImg.type() == LOAD_CMAP)
           {
             x=0;
             xsrc = xd%bwid;
             while (x<wids)
             {
               if (*(bitLine+(x>>3)) == 0)
               {
                 x+=8;
                 xsrc = ((x+xd)%bwid);
                 imgLine+=24;
                 continue;
               }
               if ((*(bitLine+(x>>3)) & 1<<(x & 7)) && (*(brushMaskLine+(xsrc>>3)) & 1<<(xsrc & 7)))
               {
                 *imgLine++=qRed(brushTab[brushLine[xsrc]]);
                 *imgLine++=qGreen(brushTab[brushLine[xsrc]]);
                 *imgLine++=qBlue(brushTab[brushLine[xsrc]]);
               }
               else
               {
               imgLine++; imgLine++; imgLine++;
               }
               x++;
               xsrc = ((x+xd)%bwid);
             }
           }
        }
       else /*big endian*/
        {
         if (brushImg.type() == LOAD_TRUE_COL)
          {
           x=0;
           xsrc = (xd%bwid)*3;
           xsrcMask = xd%bwid;
           while (x<wids)
            {
             if (*(bitLine+(x>>3)) == 0)
              {
               x+=8;
               xsrc = ((x+xd)%bwid)*3;
               xsrcMask = (x+xd)%bwid;
               imgLine+=24;
               continue;
              }
             if ((*(bitLine+(x>>3)) & 128>>(x & 7)) && (*(brushMaskLine+(xsrcMask>>3)) & 1<<(xsrcMask & 7)))
              {
               *imgLine++=brushLine[xsrc];
               *imgLine++=brushLine[xsrc+1];
               *imgLine++=brushLine[xsrc+2];
              }
              else
               { imgLine++; imgLine++; imgLine++; }
              x++;
              xsrc = ((x+xd)%bwid)*3;
              xsrcMask = (x+xd)%bwid;
            }
           }
          else if (brushImg.type() == LOAD_CMAP)
           {
            x=0;
            xsrc = xd%bwid;
            xsrcMask = xd%bwid;
            while (x<wids)
             {
              if (*(bitLine+(x>>3)) == 0)
               {
                x+=8;
                xsrc = ((x+xd)%bwid);
                imgLine+=24;
                continue;
               }
              if ((*(bitLine+(x>>3)) & 128>>(x & 7)) && (*(brushMaskLine+(xsrc>>3)) & 1<<(xsrc & 7)))
               {
                *imgLine++=qRed(brushTab[brushLine[xsrc]]);
                *imgLine++=qGreen(brushTab[brushLine[xsrc]]);
                *imgLine++=qBlue(brushTab[brushLine[xsrc]]);
               }
               else
                { imgLine++; imgLine++; imgLine++; }
               x++;
               xsrc = ((x+xd)%bwid);
             }
           }
        }   
      }
  }
}

void ApaintDraw::refreshOldData(int x1, int y1, int x2, int y2, Layer *layer)
{
   int line;
   uchar **dataOrig, **dataDoc;
   int wid, hei, bpp;
   
   x1-=layer->layerPos().x();
   y1-=layer->layerPos().y();
   x2-=layer->layerPos().x();
   y2-=layer->layerPos().y();
   dataOrig=layer->origBuf.jumpTable();
   dataDoc=layer->imageBuf.jumpTable();
   hei=y2-y1;
   wid=x2-x1;
   bpp=layer->imageBuf.depth()/8;
   wid*=bpp;
   
   for (line=y1; line<y1+hei; line++)
    memcpy(*(dataDoc+line)+x1*bpp, *(dataOrig+line)+x1*bpp, wid);
}

void ApaintDraw::finalizeDraw(int xs, int ys, int wids, int heis)
{
 int line, bpl;
 uchar **dataOrig, **dataDoc;
 Layer *selLayer = doc->getLayer();
 if (!selLayer) return;

 xs-=selLayer->layerPos().x();
 ys-=selLayer->layerPos().y();
 dataOrig=selLayer->origBuf.jumpTable();
 dataDoc=selLayer->imageBuf.jumpTable();
 bpl=selLayer->imageBuf.bytesPerLine();


 for (line=ys; line<ys+heis; line++)
  memcpy(*(dataOrig+line), *(dataDoc+line), bpl);
}

void ApaintDraw::resetAndUndo(QRect &srcRec, QRect &dstRec)
{
  Layer *selLayer = doc->getLayer();
  if (!selLayer) return;

  QRect sRec = QRect(srcRec.x() - selLayer->drawAreaPos().x(), srcRec.y() - selLayer->drawAreaPos().y(),
                     srcRec.width(), srcRec.height());
  QRect dRec = QRect(dstRec.x() - selLayer->layerPos().x(), dstRec.y() - selLayer->layerPos().y(),
                     dstRec.width(), dstRec.height());
  emit unRedoData(sRec, dRec);
  resetCtrlData(); 
}

void ApaintDraw::resetCtrlData()
{
  Layer *selLayer = doc->getLayer();
  if (!selLayer) return;
  
  if (selLayer->getPixmap())
   selLayer->getPixmap()->fill(Qt::black);  
  oldRefresh = QRect();  
}

void ApaintDraw::setMaxRect(int rnum, QRect r)
{  
  if (rnum<(int)maxRectList.count())
  {
    int mx1, my1, mx2, my2; 
   QRect *tr = maxRectList.at(rnum); 
   mx1 = QMIN(tr->x(), r.x());
   my1 = QMIN(tr->y(), r.y());
   mx2 = QMAX(tr->right()+1, r.right()+1);
   my2 = QMAX(tr->bottom()+1, r.bottom()+1);
   tr->setRect(mx1, my1, mx2-mx1, my2-my1);
  }
  else
   maxRectList.append(new QRect(r));
 
}

void ApaintDraw::setGlobalMaxRect(QRect &refRect, QRect &drawRect)
{
   int mx1, my1, mx2, my2; 
  /*setUp globalMax*/
  if (globalMaxRef.isValid())
  {
   mx1 = QMIN(globalMaxRef.x(), refRect.x());
   my1 = QMIN(globalMaxRef.y(), refRect.y());
   mx2 = QMAX(globalMaxRef.right()+1, refRect.right()+1);
   my2 = QMAX(globalMaxRef.bottom()+1, refRect.bottom()+1);
   globalMaxRef.setRect(mx1, my1, mx2-mx1, my2-my1);
  }
  else
    globalMaxRef = refRect;

  if (globalMaxDraw.isValid())
  {
   mx1 = QMIN(globalMaxDraw.x(), drawRect.x());
   my1 = QMIN(globalMaxDraw.y(), drawRect.y());
   mx2 = QMAX(globalMaxDraw.right()+1, drawRect.right()+1);
   my2 = QMAX(globalMaxDraw.bottom()+1, drawRect.bottom()+1);
   globalMaxDraw.setRect(mx1, my1, mx2-mx1, my2-my1);
  }
  else
    globalMaxDraw = drawRect;       
}

void ApaintDraw::setFgColorIndx(int indx)
{
  colorIndexFg=indx;
}

void ApaintDraw::setBgColorIndx(int indx)
{
  colorIndexBg=indx;
}

void ApaintDraw::setFgColor(QRgb rgb)
{
  redfg = qRed(rgb);
  greenfg = qGreen(rgb);
  bluefg = qBlue(rgb);
}

void ApaintDraw::setBgColor(QRgb rgb)
{
  redbg = qRed(rgb);
  greenbg = qGreen(rgb);
  bluebg = qBlue(rgb);
}

void ApaintDraw::setFatLines(bool enable)
{
  drawFatLines = enable;
}

