#ifndef __ApaintIO_h_Included__
#define __ApaintIO_h_Included__

#include <qpainter.h>
#include <qbitmap.h>
#include <apaintimage.h>
#include "apaintdoc.h"

class QPixmap;
class QImage;
class QFontMetrics;
class ApaintIOPrivate;

class IconList; 
class ApaintIO
{
private:
    ApaintIO();  
    ~ApaintIO();
public:  
 
    static ApaintIO* getObject();
    static void deleteObject();
    
    void fastDocDraw(ApaintDoc *doc, QPainter *p, 
                              int dx, int dy, int dw, int dh, double scaleX, double scaleY, bool useGrid);
    void fastImageDraw(ApaintImage& img, int sx, int sy, QPainter *p,
                       int dx, int dy, int dw, int dh, double scaleX, double scaleY);
    void fastIconDraw(IconList *ilist, QPainter *p, QFontMetrics &fm, int dx, int dy, int dw, int dh);                          
    void setGridSize(int wid, int hei){ gridWid = wid; gridHei = hei;}    
    void setGridColor(uchar col){ gridCol = col;}
private:
    
    bool m_bShm;
    void initXImage(int w, int h);
    void doneXImage();
    void createXImage(int w, int h);
    void destroyXImage();
    void createShmSegment(int size);
    void destroyShmSegment();
    void convertToXImage(int dstx, int dsty, 
                         int imageWidth, int imageHeight, int imageType,
                         int imageDepth, uchar** imageLine, QRgb* imageColorTab, int numColors,
                         int sx, int sy, int sw, int sh, int srcOffY,
                         bool mirrorx, bool mirrory, double scaleX, double scaleY,
                         bool useMask, QImage *mask, bool *transColors, bool *protectColors,
			 bool semiTransparent = false);
    QImage copyScaleImage( ApaintDoc *doc, int dx, int dy, int dw, int dh, double scaleX, double scaleY);
    void drawGrid(int dx, int dy, int dw, int dh, double scaleX, double scaleY, int imgHei);
    
private:
    QRect getScaleRect(QRect pos, double scaleX, double scaleY);
    ApaintIOPrivate* d;
    QPixmap *pix;
    QImage *transProtTable;
    int gridWid, gridHei;
    uchar gridCol;
};

#endif // __ApaintIO_h_Included__
