/***************************************************************************
                          apaintunredo.cpp  -  description
                             -------------------
    begin                : Sun Mar 16 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <qptrlist.h>
#include <qdir.h>
#include "apaintunredo.h"
#include "apaintimageUtil.h"
#include "apaintconfig.h"
#include "apaint.h"

#define PFX    "apaint"
#define UNDO_STACK    0
#define REDO_STACK    1

static void buildTextureBitmap(TextureBitmap *w,int width, int height);
static void freeTextureBitmap(TextureBitmap *w);
static int pixelOn(TextureBitmap *w, int x, int y);
static void setPixel(TextureBitmap *w, int x, int y);

void buildTextureBitmap(TextureBitmap *w,int width, int height)
{
 w->bpl=(width+7)>>3;
 w->data=(uchar *)calloc((w->bpl*height),1);
}

void freeTextureBitmap(TextureBitmap *w)
{
 w->bpl=0;
 free(w->data);
 w->data=0;
}

int pixelOn(TextureBitmap *w, int x, int y)
{
 if (w->data[w->bpl*y+(x>>3)]&(128>>(x&7))) /* do not just return this because we */
  return(true);                          /* want to compare results for == */
 else
  return(false);
}

void setPixel(TextureBitmap *w, int x, int y)
{
 w->data[w->bpl*y+(x>>3)] |= (128>>(x&7));
}

UndoObject::UndoObject(int typ)
{
 type=typ;
}

UndoObject::~UndoObject()
{
  
}

ApaintUnRedo::ApaintUnRedo(ApaintDoc *docp, TransformTool *transToolp, ColPaletteDlg *colp)
{
 undoStackNum=redoStackNum=0;
 docLoaded=false;
 undoStackWhenSaved = 0;
 stack.setAutoDelete(false);
 undoDirOk=ApaintConfig::makeTmpDir(TMPDIR, PFX, undoDir);
 doc = docp;
 transTool = transToolp;
 Q_CHECK_PTR(colp);
 colP = colp;
 connect(doc, SIGNAL(newDocSig()), this, SLOT(newDocSlot()));
 connect(doc, SIGNAL(putUndoStackSig(Layer*, LayerChangeType)), this, SLOT(putLayerData(Layer*, LayerChangeType)));
 connect(this, SIGNAL(emptyUndo(bool)), apaint, SLOT(emptyUndo(bool)));
 connect(this, SIGNAL(emptyRedo(bool)), apaint, SLOT(emptyRedo(bool)));
 connect(this, SIGNAL(documentChanged(bool)), apaint, SLOT(documentChanged(bool)));
}

ApaintUnRedo::~ApaintUnRedo()
{ 
 if (undoDirOk)
  ApaintConfig::clearTmpDir(undoDir);
}

// if nothing is on UNDO stack the image isn't changed!
bool ApaintUnRedo::isDocChanged()
{
 bool ret = stack.count() != undoStackWhenSaved;

 return ret;
}

void ApaintUnRedo::resetDocChanged()
{
 undoStackWhenSaved = stack.count();
 emit documentChanged(false);
}

void ApaintUnRedo::newDocSlot()
{
  if (!stack.isEmpty())
   clearStack(&stack);
  if (!redoStack.isEmpty())
   clearStack(&redoStack);
  docLoaded=true;
}


void ApaintUnRedo::removeMyData(UndoObject *uobj)
{
  QDir dir;
  if (!uobj) return;
  
  if (uobj->type==IMAGE_CHANGE)
  {
   dir.remove(uobj->data.imgData->fname, true); 
   freeTextureBitmap(&uobj->data.imgData->bmp);
   delete uobj->data.imgData;
  }
  else if (uobj->type==LAYER_CHANGE || uobj->type==UNSELECT)  
  {
   dir.remove(uobj->data.ldata->fname, true);
   delete uobj->data.ldata;
  }
  else if (uobj->type == TRANSFORM_CHANGE)
   delete uobj->data.tdata;
  else if (uobj->type == CHANGE_LAYER_TILES)
  {
    if (uobj->data.tileData->useTiles)
      delete[] uobj->data.tileData->mirrorTab;
  }  
  else if (uobj->type == LAYER_FILTER)
   delete uobj->data.layerFilterData;
  delete uobj;
}

void ApaintUnRedo::clearStack(UnRedoStack *stackIn)
{
 UndoObject *uobj;

 while (!stackIn->isEmpty())
  {
   uobj=stackIn->pop();
   removeMyData(uobj);
  }
 if (stackIn == &stack)
  {
   emit emptyUndo(true);
   emit documentChanged(stack.count() != undoStackWhenSaved);
  }
 else
  if (stackIn == &redoStack)
   emit emptyRedo(true);
  
}

void ApaintUnRedo::putOnStack(UndoObject *uobj, short which)
{
 if (which==UNDO_STACK)
  {
   if (stack.isEmpty())
    emit emptyUndo(false);
   stack.push(uobj);
   undoStackNum++;
   emit documentChanged(stack.count() != undoStackWhenSaved);
  }
 else if (which==REDO_STACK)
  {  
   if (redoStack.isEmpty())
    emit emptyRedo(false);
   redoStack.push(uobj);
   redoStackNum++;
 }
// emit showDocCaption(); unreahable
}

UndoObject* ApaintUnRedo::getFromStack(short which)
{
 if (which==UNDO_STACK)
  {
   if (stack.isEmpty()) return NULL;
   undoStackNum--;
   if (stack.count() == 1)
    emit emptyUndo(true);
   emit documentChanged(stack.count() != undoStackWhenSaved + 1);
   return stack.pop();
  }
 else if (which==REDO_STACK)
  {
   if (redoStack.isEmpty()) return NULL;
   redoStackNum--;
   if (redoStack.count() == 1)
    emit emptyRedo(true);
   return redoStack.pop();
  }
 return NULL;
}

void ApaintUnRedo::putInsRemData(bool horizontal, int pos, int num, bool doInsert)
{
 if (!docLoaded || !undoDirOk)  return;

 if (!redoStack.isEmpty())
   clearStack(&redoStack);
 
  putInsRemData(horizontal, pos, num, doInsert, UNDO_STACK);
}

void ApaintUnRedo::putInsRemData(bool horizontal, int pos, int num, bool doInsert, int which)
{ 
 UndoObject *uobj;
 int size;
 uchar *buf;
 gzFile f;
 QString fname;
 Layer *layer = doc->getLayer();
 if (!layer) return;
   
 if (which==REDO_STACK)
   fname=QString("%1%2%3").arg(undoDir).arg("redo").arg(redoStackNum);
 else
   fname=QString("%1%2%3").arg(undoDir).arg("undo").arg(undoStackNum);
 f=gzopen((char *)fname.ascii(), "wb6");
 if (!f) return;
  
 uobj = new UndoObject(CHANGE_INS_REM_PIX);
 if (!uobj)
 {
   qDebug("ApaintUnRedo pointer NULL!");
   return;
 }
 uobj->data.pixInsRem = new PixelInsRemData;
 uobj->data.pixInsRem->doInsert = doInsert;
 uobj->data.pixInsRem->pos = pos;
 uobj->data.pixInsRem->num = num;
 uobj->data.pixInsRem->horizontal = horizontal;
 uobj->data.pixInsRem->fname = fname;
 uobj->data.pixInsRem->size = 0;

 if (!doInsert)
 {
   if (horizontal)
    buf = ApaintImageUtil::getHorLines(&layer->imageBuf, pos, num, size);
   else
    buf = ApaintImageUtil::getVertLines(&layer->imageBuf, pos, num, size);

   uobj->data.pixInsRem->size = size;
   gzwrite(f, buf, size);
   gzclose(f);
   delete buf;
 }
 putOnStack(uobj, which); 
}

void ApaintUnRedo::putSpecificData()
{
 putOnStack(new UndoObject(SPECIFIC), UNDO_STACK);
}

void ApaintUnRedo::clearSpecificData()
{
 while (stack.isEmpty() == false && stack.top()->type == SPECIFIC)
  delete getFromStack(UNDO_STACK);

 while (redoStack.isEmpty() == false && redoStack.top()->type == SPECIFIC)
  delete getFromStack(REDO_STACK);
}

void ApaintUnRedo::getInsRemData(UndoObject *uobj)
{
 gzFile f;
 uchar *buf = 0;
 Layer *layer = doc->getLayer();
 if (!layer)  return;
 
 if (!uobj->data.pixInsRem->doInsert)
 {
   f=gzopen(uobj->data.pixInsRem->fname, "rb6");
   if (!f) return;
   int size = uobj->data.pixInsRem->size;
   buf = new uchar[size];
   gzread(f, buf, size);
   gzclose(f);
 } 
 if (uobj->data.pixInsRem->horizontal)
   layer->insertRemovePixelsHor(uobj->data.pixInsRem->pos, uobj->data.pixInsRem->num, !uobj->data.pixInsRem->doInsert,
                                false, (buf!=0), buf);
 else
   layer->insertRemovePixelsVert(uobj->data.pixInsRem->pos, uobj->data.pixInsRem->num, !uobj->data.pixInsRem->doInsert,
                                false, (buf!=0), buf);

 if (doc->getLayerType() == MainLayer)
   doc->sendSignals(true, true);
 else
   doc->sendSignals(true, false);

 doc->refreshLayer();
 if (buf)
  delete[] buf;                                                               
}

void ApaintUnRedo::putTileData()
{
 if (!docLoaded || !undoDirOk)  return;

 if (!redoStack.isEmpty())
   clearStack(&redoStack);
   
 putTileData(UNDO_STACK);
}

void ApaintUnRedo::putTileData(int which)
{
 UndoObject *uobj;
 Tile *tile;
 Layer *layer = doc->getMainLayer();
 
 uobj = new UndoObject(CHANGE_LAYER_TILES);
 if (!uobj)
 {
   qDebug("ApaintUnRedo pointer NULL!");
   return;
 }
 uobj->data.tileData = new TileData;
 if (!uobj->data.tileData)
 {
   qDebug("ApaintUnRedo pointer NULL!");
   return;
 }
 uobj->data.tileData->useTiles = layer->isUsingTiles();
 if (layer->isUsingTiles())
 {
  layer->getTileInfo(uobj->data.tileData->sizex, uobj->data.tileData->sizey, uobj->data.tileData->offx, uobj->data.tileData->offy);
  QPtrList<Tile>& tlist = layer->getTileList();
  uobj->data.tileData->mirrorTab = new MirrorType[tlist.count()];
  if (!uobj->data.tileData->mirrorTab)
  {
   qDebug("ApaintUnRedo pointer NULL!");
   return;
  }
  for (int i=0; i<(int)tlist.count(); i++)
  {
    tile = tlist.at(i);
    if (tile->mirrorHor())
      uobj->data.tileData->mirrorTab[i].mirrorHor = true;
    else
      uobj->data.tileData->mirrorTab[i].mirrorHor = false;

    if (tile->mirrorVert())
      uobj->data.tileData->mirrorTab[i].mirrorVert = true;
    else
      uobj->data.tileData->mirrorTab[i].mirrorVert = false;
  }
 }
 putOnStack(uobj, which); 
}

void ApaintUnRedo::getTileData(UndoObject *uobj)
{
  Tile *tile;
  Layer *layer = doc->getMainLayer();
  bool useTiles = uobj->data.tileData->useTiles;
  if (useTiles)
  {
    doc->createMainLayerTiles(uobj->data.tileData->sizex, uobj->data.tileData->sizey,
                              uobj->data.tileData->offx, uobj->data.tileData->offy, false);
    QPtrList<Tile>& tlist = layer->getTileList();
    for (int i=0; i<(int)tlist.count(); i++)
    {
      tile = tlist.at(i);
      tile->setMirrorHor(uobj->data.tileData->mirrorTab[i].mirrorHor);
      tile->setMirrorVert(uobj->data.tileData->mirrorTab[i].mirrorVert);
    }
    doc->refreshLayer();
  }
  else
  {
    /*dont use tiles anymore*/
    doc->enableMainLayerTiles(false);
  }
}

void ApaintUnRedo::putLayerNum(int lnum)
{
 if (!docLoaded || !undoDirOk)  return;

 if (!redoStack.isEmpty())
   clearStack(&redoStack);

  putLayerNum(lnum, UNDO_STACK);
}


void ApaintUnRedo::putLayerNum(int lnum, int which)
{
  UndoObject *uobj;

  uobj= new UndoObject(CHANGE_LAYER);
  if (!uobj)
  {
    qDebug("ApaintUnRedo pointer NULL!");
    return;
  }
  uobj->data.layerNum = lnum;

  putOnStack(uobj, which);
}

void ApaintUnRedo::getLayerNum(UndoObject *uobj)
{
  int lnum = uobj->data.layerNum;
  if (doc->getLayerNum() != lnum)
   doc->setActiveLayer(lnum);
}


void ApaintUnRedo::putColorData(QRgb col, int indx)
{
 if (!docLoaded || !undoDirOk)  return;

 if (!redoStack.isEmpty())
   clearStack(&redoStack);

 putColorData(col, indx, UNDO_STACK);
}

void ApaintUnRedo::putColorData(QRgb col, int indx, int which)
{
  UndoObject *uobj;

  uobj= new UndoObject(COLOR_CHANGE);
  if (!uobj)
  {
    qDebug("ApaintUnRedo pointer NULL!");
    return;
  }
  uobj->data.col = new ColorData;
  if (!uobj->data.col)
  {
    qDebug("ApaintUnRedo pointer NULL!");
    return;
  }
  uobj->data.col->color = col;
  uobj->data.col->colorIndx = indx;
  putOnStack(uobj, which);
}

void ApaintUnRedo::getColorData(UndoObject *uobj)
{
  colP->setColorPos(uobj->data.col->colorIndx);
  colP->setColor(uobj->data.col->color, false);
}

void ApaintUnRedo::putTransformData(double sx, double sy, double angle, int dstWid, int dstHei)
{
 if (!docLoaded || !undoDirOk)  return;

 if (!redoStack.isEmpty())
   clearStack(&redoStack);

 putTransformData(sx, sy, angle, UNDO_STACK, dstWid, dstHei);
}

void ApaintUnRedo::putTransformData(double sx, double sy, double angle, int which, int dstWid, int dstHei)
{
  UndoObject *uobj;

  uobj= new UndoObject(TRANSFORM_CHANGE);
  if (!uobj)
  {
    qDebug("ApaintUnRedo pointer NULL!");
    return;
  }
  uobj->data.tdata= new TransFormData;
  if (!uobj->data.tdata)
  {
    qDebug("ApaintUnRedo pointer NULL!");
    return;
  }

  uobj->data.tdata->sx = sx;
  uobj->data.tdata->sy = sy;
  uobj->data.tdata->angle = angle;
  uobj->data.tdata->dstWid = dstWid;
  uobj->data.tdata->dstHei = dstHei;
  putOnStack(uobj, which);
}

void ApaintUnRedo::getTransformData(UndoObject *uobj)
{
 transTool->scaleRotImg(uobj->data.tdata->sx, uobj->data.tdata->sy,
  uobj->data.tdata->angle, uobj->data.tdata->dstWid,
  uobj->data.tdata->dstHei, true, transTool->getFilterType());
}

void ApaintUnRedo::putLayerData(Layer *sel, LayerChangeType ctyp)
{
  if (!docLoaded || !undoDirOk)  return;

  if (!redoStack.isEmpty())
    clearStack(&redoStack);
   
  putLayerData(sel, ctyp, UNDO_STACK);
}

void ApaintUnRedo::putLayerData(Layer *sel, LayerChangeType ctyp, int which)
{
  UndoObject *uobj;
  QString fname;
  gzFile f;
  QImage mask;
  ApaintImage layerImage, mainImage;
  FHPoint *ctrlPts=0;
  int numCtrlPts;
  bool layerDataSaved, layerColorTabSaved;
  QRect pos;
  
  if (!sel)
  {
    qDebug("ApaintUnRedo pointer NULL!");
    return;
  }

  if (which==REDO_STACK)
   fname=QString("%1%2%3").arg(undoDir).arg("redo").arg(redoStackNum);
  else
   fname=QString("%1%2%3").arg(undoDir).arg("undo").arg(undoStackNum);
  f=gzopen((char *)fname.ascii(), "wb6");
  if (!f) return;

  if (ctyp == CreateLayer)
    pos = sel->layerPos();
  else
    pos = sel->realLayerPos();

  layerDataSaved = ctyp == Unselect || ctyp == DestroyLayer  || ctyp == ChangeLayerForm || ctyp == ChangeSelToMainLayer;
  layerColorTabSaved = ctyp == ChangeColorTable;
  
  if (ctyp == CreateLayer || ctyp == DestroyLayer || ctyp == Unselect)
   {
    if (doc->mainLayer->isUsingTiles())
     mainImage = doc->mainLayer->getAllTileData(pos, false);
    else    
     mainImage = doc->mainLayer->origBuf.copy(pos.x(), pos.y(), pos.width(), pos.height());
   }

  layerImage = sel->origBuf;
  mask = sel->getMask();
  
  numCtrlPts = sel->numCtrlPoints();
  
  uobj= new UndoObject(ctyp==Unselect ? UNSELECT : LAYER_CHANGE);
  if (!uobj)
  {
    qDebug("ApaintUnRedo pointer NULL!");
    return;
  }
  uobj->data.ldata= new LayerData;
  if (!uobj->data.ldata)
  {
    qDebug("ApaintUnRedo pointer NULL!");
    return;
  }
  uobj->data.ldata->x=pos.x();
  uobj->data.ldata->y=pos.y();
  uobj->data.ldata->xdpi = layerImage.xdpi();  
  uobj->data.ldata->ydpi = layerImage.ydpi();
  uobj->data.ldata->wid=pos.width();
  uobj->data.ldata->hei=pos.height();
  uobj->data.ldata->usingMask = sel->usingMask();
  uobj->data.ldata->ltyp = sel->getType();
  uobj->data.ldata->lg = sel->getGeometry();
  uobj->data.ldata->ctyp = ctyp;
  uobj->data.ldata->shiftx = pos.width() - sel->getShiftX();
  uobj->data.ldata->shifty = pos.height() - sel->getShiftY();
  uobj->data.ldata->swapColor1 = sel->getSwapColor1();
  uobj->data.ldata->swapColor2 = sel->getSwapColor2();
  uobj->data.ldata->numCol = layerImage.numColors();
  uobj->data.ldata->mainNumCol = mainImage.numColors();
  uobj->data.ldata->numCtrlPoints = numCtrlPts;
  uobj->data.ldata->fname=fname.copy();
  if (ctyp == CreateLayer)
   uobj->data.ldata->maskBpl=0;
  else
   uobj->data.ldata->maskBpl = mask.bytesPerLine();
  uobj->data.ldata->layerDataSaved = layerDataSaved;
  uobj->data.ldata->imgBpl = layerImage.bytesPerLine();
  uobj->data.ldata->imgDepth = layerImage.depth();
  uobj->data.ldata->mainImgBpl = mainImage.bytesPerLine();
  memcpy(uobj->data.ldata->transColors, sel->getTransColors(), 256*sizeof(bool));
  memcpy(uobj->data.ldata->protectColors, sel->getProtectColors(), 256*sizeof(bool));
  putOnStack(uobj, which);

  /*get Border points*/
  if (layerDataSaved && (numCtrlPts>0))
  {
   QPointArray *borderPts = sel->getBorderPoints();
   ctrlPts = (FHPoint*) malloc(sizeof(FHPoint)*numCtrlPts);
   for (int i =0; i<numCtrlPts; i++)
   {
     ctrlPts[i].x = borderPts->point(i).x();
     ctrlPts[i].y = borderPts->point(i).y();
   }
  }

  /*save data on hard drive*/
  if (layerDataSaved && sel->usingMask())
    gzwrite(f, mask.scanLine(0), mask.bytesPerLine()*mask.height());

  if (layerDataSaved && (numCtrlPts>0))
  {
    gzwrite(f, ctrlPts, sizeof(FHPoint)*numCtrlPts);
    free(ctrlPts);
  }
   
  if (layerDataSaved || layerColorTabSaved)
    gzwrite(f, layerImage.colorTable(), sizeof(QRgb)*layerImage.numColors());

  if (ctyp == DestroyLayer || ctyp == Unselect)
    gzwrite(f, mainImage.colorTable(), sizeof(QRgb)*mainImage.numColors());

  if (layerDataSaved)   
   gzwrite(f, layerImage.scanLine(0), layerImage.bytesPerLine()*layerImage.height());

  if (ctyp == DestroyLayer || ctyp == Unselect)
    gzwrite(f, mainImage.scanLine(0), mainImage.bytesPerLine()*mainImage.height());
  
  gzclose(f); 
   
}

/*QPointArray must allready be setup*/
void ApaintUnRedo::fhPts2Qpts(FHPoint *fhPts, QPointArray *qPts, int num)
{
 for (int i=0; i<num; i++)
  qPts->setPoint(i, fhPts[i].x, fhPts[i].y);
}

#include <iostream>

using namespace std;

void ApaintUnRedo::getLayerData(UndoObject *uobj, bool undo)
{
  int x, y, wid, hei, shx, shy, xdpi, ydpi, swapColor1, swapColor2;
  QString fname;
  gzFile f;
  int imgBpl, maskBpl, imgDepth, numCol, mainNumCol, mainImgBpl;
  LayerChangeType ctyp;
  ApaintImage image, mainImage;
  QImage mask;
  QRect newRect, oldRect;
  LayerType typ;
  LayerGeometry lg;
  bool usingMask;
  QRgb colTab[256], colTabMain[256];
  QPointArray borderPts;
  FHPoint *ctrlPts;
  int numCtrlPts;
  
  if (!docLoaded || !undoDirOk)  return;

  if (!doc->getLayer()) return;

  fname=uobj->data.ldata->fname;
  x=uobj->data.ldata->x;
  y=uobj->data.ldata->y;
  xdpi = uobj->data.ldata->xdpi;
  ydpi = uobj->data.ldata->ydpi;
  wid=uobj->data.ldata->wid;
  hei=uobj->data.ldata->hei;
  usingMask = uobj->data.ldata->usingMask;
  typ = uobj->data.ldata->ltyp;
  lg = uobj->data.ldata->lg;
  ctyp = uobj->data.ldata->ctyp;
  imgBpl = uobj->data.ldata->imgBpl;
  maskBpl = uobj->data.ldata->maskBpl;
  imgDepth = uobj->data.ldata->imgDepth;
  shx = uobj->data.ldata->shiftx;
  shy = uobj->data.ldata->shifty;
  swapColor1 = uobj->data.ldata->swapColor1;
  swapColor2 = uobj->data.ldata->swapColor2;
  numCol = uobj->data.ldata->numCol;
  numCtrlPts = uobj->data.ldata->numCtrlPoints;
  mainNumCol = uobj->data.ldata->mainNumCol;
  mainImgBpl = uobj->data.ldata->mainImgBpl;

  newRect = QRect(x, y, wid, hei);
  oldRect = doc->getLayer()->layerPos();
  f=gzopen(fname, "rb6");
  if (!f) return;
 
  if (ctyp == CreateLayer)
   {    
    /*destroy layer*/
    doc->deleteLayer();
    emit refresh(x, y, wid, hei, RECT_SIZE);
   }
  else if (ctyp == DestroyLayer || ctyp == Unselect)
  {    
    if (usingMask)
     {
      mask = QImage(wid, hei, 1, 2, QImage::LittleEndian);
      gzread(f, mask.scanLine(0), maskBpl*hei);     
     }
    if (numCtrlPts>0)
    {
      ctrlPts = (FHPoint*) malloc(sizeof(FHPoint)*numCtrlPts);
      gzread(f, ctrlPts, sizeof(FHPoint)*numCtrlPts);
      borderPts = QPointArray(numCtrlPts);
      fhPts2Qpts(ctrlPts, &borderPts, numCtrlPts);      
      free(ctrlPts);
    }

    gzread(f, colTab, sizeof(QRgb)*numCol); 
    gzread(f, colTabMain, sizeof(QRgb)*mainNumCol);
    if (doc->mainLayer->getImageType() == LOAD_CMAP)
      doc->mainLayer->setColorTable(colTabMain, mainNumCol ,false, false);
    image.newImage(wid, hei,imgDepth);
    image.setXdpi(xdpi);
    image.setYdpi(ydpi);
    gzread(f, image.scanLine(0), imgBpl*hei);
    if (image.type() == LOAD_CMAP)
      image.setExactColorTable(colTab, numCol);

    /*load mainLayer data*/
    mainImage = doc->mainLayer->imageBuf.resize(wid, hei, 0);
    gzread(f, mainImage.scanLine(0), mainImgBpl*hei);          

    /*create old Layer*/
    if (usingMask)
      doc->createNewLayer(image, &mask,  x, y);
    else
     doc->createNewLayer(image, 0,  x, y);
    doc->setLayerType(typ);

    doc->getLayer()->setGeometry(lg);
    if (numCtrlPts>0)
      doc->setLayerBorderPoints(borderPts);

    /*copy old data to main image*/
    doc->copyImageToLayer(doc->mainLayer, QRect(x, y, wid, hei), mainImage, mainImage, true, false, 0, 0, FALSE);
    if (doc->mainLayer->isUsingTiles())
    {
      QPtrList<QRect>& rList = doc->mainLayer->getLayerRefreshArea(QRect(x, y, wid, hei));
      QRect *nr;
      for (int i=0; i<(int)rList.count(); i++)
      {
        nr = rList.at(i);
        emit refreshRepeat(doc->mainLayer, nr->x()-RECT_SIZE, nr->y()-RECT_SIZE, nr->width()+2*RECT_SIZE, nr->height()+2*RECT_SIZE);
      }
    }
    else
      emit refresh(x, y, wid, hei,RECT_SIZE);
  }
  else if (ctyp == ChangeLayerPos)
  { 
    doc->setLayerPos(x, y);
    newRect = doc->getLayerPos();
    doc->refreshLayer(oldRect, newRect, 3);
  }
  else if (ctyp == ChangeLayerShiftX)
  {
    doc->getLayer()->horShift(shx);
    emit refresh(oldRect.x(), oldRect.y(), oldRect.width(), oldRect.height(), 0);
  }
  else if (ctyp == ChangeLayerShiftY)
  { 
    doc->getLayer()->vertShift(shy);
    emit refresh(oldRect.x(), oldRect.y(), oldRect.width(), oldRect.height(), 0);
  }
  else if (ctyp == SwapColors)
   doc->getLayer()->swapColors(swapColor1, swapColor2);
  else if (ctyp == ChangeLayerMirrorX)
    doc->layerMirrorVertical();    
  else if (ctyp == ChangeLayerMirrorY)
    doc->layerMirrorHorizontal();
  else if (ctyp == ChangeLayerFlip)
   {
    doc->layerMirrorFlip();
    if (undo) /* flip it three times so we get back to original */
     {
      doc->layerMirrorFlip();
      doc->layerMirrorFlip();
     }
   }
  else if (ctyp == ChangeLayerExplode)
    doc->layerMirrorExplode();
  else if (ctyp == InvertAllColors)
    doc->layerInvertColors(INVERT_COLORS_ALL);
  else if (ctyp == InvertRed)
    doc->layerInvertColors(INVERT_COLORS_RED);
  else if (ctyp == InvertGreen)
    doc->layerInvertColors(INVERT_COLORS_GREEN);
  else if (ctyp == InvertBlue)
    doc->layerInvertColors(INVERT_COLORS_BLUE);
  else if (ctyp == InvertCyan)
    doc->layerInvertColors(INVERT_COLORS_CYAN);
  else if (ctyp == InvertMagenta)
    doc->layerInvertColors(INVERT_COLORS_MAGENTA);
  else if (ctyp == InvertYellow)
    doc->layerInvertColors(INVERT_COLORS_YELLOW);
  else if (ctyp == SwapRGB_BRG)
    doc->layerInvertColors(SWAP_COLORS_RGB_GBR); /* BRG swap is inverse of GBR */
  else if (ctyp == SwapRGB_GBR)
    doc->layerInvertColors(SWAP_COLORS_RGB_BRG); /* GBR swap is inverse of BRG */
  else if (ctyp == SwapRG)
    doc->layerInvertColors(SWAP_COLORS_RG);
  else if (ctyp == SwapRB)
    doc->layerInvertColors(SWAP_COLORS_RB);
  else if (ctyp == SwapGB)
    doc->layerInvertColors(SWAP_COLORS_GB);

  else if (ctyp == ChangeLayerForm)
  {
   if (usingMask)
    {
     mask = QImage(wid, hei, 1, 2, QImage::LittleEndian);
     gzread(f, mask.scanLine(0), maskBpl*hei);
    }
   if (numCtrlPts>0)
    {
     ctrlPts = (FHPoint*) malloc(sizeof(FHPoint)*numCtrlPts);
     gzread(f, ctrlPts, sizeof(FHPoint)*numCtrlPts);
     borderPts = QPointArray(numCtrlPts);
     fhPts2Qpts(ctrlPts, &borderPts, numCtrlPts);
     doc->setLayerBorderPoints(borderPts);
     free(ctrlPts);
    }

    gzread(f, colTab, sizeof(QRgb)*numCol);
    image.newImage(wid, hei,imgDepth);
    image.setXdpi(xdpi);
    image.setYdpi(ydpi);
    gzread(f, image.scanLine(0), imgBpl*hei);
    if (image.type() == LOAD_CMAP)
      image.setExactColorTable(colTab, numCol);

    if (usingMask)
      doc->layerSetNewData(image, x, y, &mask);
    else
     doc->layerSetNewData(image, x, y, 0);
     doc->refreshLayer(oldRect, newRect, 3);
  }
  else if (ctyp == ChangeSelToMainLayer)
  {
    gzread(f, colTab, sizeof(QRgb)*numCol);
    image.newImage(wid, hei,imgDepth);
    image.setXdpi(xdpi);
    image.setYdpi(ydpi);
    gzread(f, image.scanLine(0), imgBpl*hei);
    if (image.type() == LOAD_CMAP)
      image.setExactColorTable(colTab, numCol);
    Layer *nl = new Layer(image, 0, 0, 0, MainLayer, doc->getAspectX(), doc->getAspectY());
    doc->mainLayerToSelection(nl);
  }
  else if (ctyp == ChangeMainToSelLayer)
  {
    doc->selectionToMainLayer();
  }
  else if (ctyp == ChangeColorTable)
  {
    gzread(f, colTab, sizeof(QRgb)*numCol);
    doc->setLayerColorTable(colTab, numCol, false, false);
  }
  
  gzclose(f);

  doc->setLayerTransColors(uobj->data.ldata->transColors);
  doc->setLayerProtectColors(uobj->data.ldata->protectColors);

}

void ApaintUnRedo::putLayerFilterData(int what, int fgCol, int bgCol, int penX, int penY, int value)
{
 if (!docLoaded || !undoDirOk) return;

 selLayer = doc->getLayer();
 if (!selLayer) return;

 if (!redoStack.isEmpty())
   clearStack(&redoStack);

 UndoObject *uobj = new UndoObject(LAYER_FILTER);
 LayerFilterData *data = uobj->data.layerFilterData = new LayerFilterData;
 data->what = what;
 data->selectedLayer = selLayer;
 data->fgCol = fgCol;
 data->bgCol = bgCol;
 data->penX = penX;
 data->penY = penY;
 data->value = value;
 putOnStack(uobj, UNDO_STACK);
}

void ApaintUnRedo::putImageData(QRect &srcRec, QRect &dstRec)
{
 if (!docLoaded || !undoDirOk)  return;

 selLayer = doc->getLayer();
 if (!selLayer) return;
 
 if (!redoStack.isEmpty())
   clearStack(&redoStack);

 if (selLayer->isUsingTiles())
    putMultipleImageData(srcRec, dstRec);
 else
    putImageData(srcRec, dstRec, 0, UNDO_STACK);
  
}

void ApaintUnRedo::putMultipleImageData(QRect &srcRec, QRect &dstRec)
{
 int which = UNDO_STACK;
 QRect *rn, prn;
 double ax = doc->getAspectX();
 double ay = doc->getAspectY();
 
 selLayer = doc->getLayer();
 if (!selLayer) return;
 
 QPtrList<QRect>& rList=selLayer->getLayerRefreshArea(dstRec);
 for (int i=0; i<(int)rList.count(); i++)
 {
  rn = rList.at(i);
  prn = QRect((int)(rn->x()*ax), (int)(rn->y()*ay), (int)(rn->width()*ax), (int)(rn->height()*ay));
  putImageData(prn, *rn, 0, which);
 }
 UndoObject *uobj= new UndoObject(MULTIPLE_IMAGE_CHG);
 uobj->data.multipleImgNum = rList.count();
 putOnStack(uobj, which);
}

void ApaintUnRedo::putImageData(QRect &srcRec, QRect &dstRec, uchar *bmp, int which)
{
 UndoObject *uobj; 
 QString fname;
 gzFile f;
 int  depth, bpp, size, x, y, xs, ys, ws, hs;
 uchar *buf;
 QImage img1Bit;
 uchar *bitLine;
 uchar **dataBit;
 QPixmap maskRect;

 if (!docLoaded || !undoDirOk)  return;

 selLayer = doc->getLayer();
 if (!selLayer) return;

 //dst rect
 xs = dstRec.x();
 ys = dstRec.y();
 ws = dstRec.width();
 hs = dstRec.height();
 depth=selLayer->origBuf.depth();
 bpp=depth>>3;
   
 if (which==REDO_STACK)
  fname=QString("%1%2%3").arg(undoDir).arg("redo").arg(redoStackNum);
 else
  fname=QString("%1%2%3").arg(undoDir).arg("undo").arg(undoStackNum);  
 f=gzopen((char *)fname.ascii(), "wb6");
 if (!f) return;
   
 uobj= new UndoObject(IMAGE_CHANGE);
 if (!uobj)
  {
    qDebug("ApaintUnRedo pointer NULL!");
    return;
  }
 uobj->data.imgData= new ImageData;
 if (!uobj->data.imgData)
  {
    qDebug("ApaintUnRedo pointer NULL!");
    return;
  }

 maskRect = QPixmap(srcRec.width(), srcRec.height(), 1);
 if (maskRect.isNull()) return;
 bitBlt(&maskRect, 0, 0, selLayer->getPixmap(), srcRec.x(), srcRec.y(), srcRec.width(), srcRec.height());
 img1Bit=maskRect.convertToImage();
 if ((srcRec.width() != ws) || (srcRec.height() != hs))
 {
    img1Bit = scaleQImage(srcRec.x(), srcRec.y(), img1Bit, selLayer->drawAreaPos(), selLayer->layerPos(),
                         selLayer->getPixSrcZoomX(), selLayer->getPixSrcZoomY(), xs, ys, ws, hs);
 } 
 
 dataBit=img1Bit.jumpTable(); 
 uobj->data.imgData->x=xs;
 uobj->data.imgData->y=ys;
 uobj->data.imgData->width=ws;
 uobj->data.imgData->height=hs;
 uobj->data.imgData->fname=fname.copy(); 
 buildTextureBitmap(&uobj->data.imgData->bmp, ws, hs);
 if (!bmp)
 {
  for (y=0; y<hs; y++)
  {
   bitLine=*(dataBit+y);
   x =0;
   if (img1Bit.bitOrder()==QImage::LittleEndian)
   {     
     while (x<ws)
     {
      if (*(bitLine+(x>>3)) == 0)
      {
        x+=8;
        continue;
      }
      if (*(bitLine+(x>>3)) & 1<<(x&7))
       setPixel(&uobj->data.imgData->bmp, x, y);
       x++;
      }
    }
    else /*Big endian*/
    {
     while (x<ws)
     {
      if (*(bitLine+(x>>3)) == 0)
      {
        x+=8;
        continue;
      }
      if (*(bitLine+(x>>3)) & 1<<(7-(x & 7)))
       setPixel(&uobj->data.imgData->bmp, x, y);
       x++;
     }
    }
   }
 }
 else
  {
   free(uobj->data.imgData->bmp.data);
   uobj->data.imgData->bmp.data=bmp;
  }

 gzwrite(f, uobj->data.imgData->bmp.data, uobj->data.imgData->bmp.bpl*hs);
 size=0;
 buf=(uchar *)&compBuf;
 for (y=ys; y<ys+hs; y++)
  for (x=xs; x<xs+ws; x++)
   if (pixelOn(&uobj->data.imgData->bmp, x-xs, y-ys))
   {
    memcpy(buf+size, (uchar *)selLayer->origBuf.scanLine(y)+x*bpp, bpp);
    size+=bpp;
    if (size>BUF_SIZE-bpp)
    {
      gzwrite(f, (int *)&size, sizeof(size));
      gzwrite(f, buf, size);
      size=0;
    }
   }
 if (size>0)
 {
  gzwrite(f, (int *)&size, sizeof(size));
  gzwrite(f, buf, size);
 }

 free(uobj->data.imgData->bmp.data);
 
 uobj->data.imgData->bmp.data=0;
 gzclose(f);   
 putOnStack(uobj, which);
}

void ApaintUnRedo::getMultipleImageData(UndoObject *uobj, int fromWhich, int toWhich)
{
 uchar *bmp;
 QRect dstRec;
 UndoObject *srcUobj;
 
 for (int i=0; i<uobj->data.multipleImgNum; i++)
 {
  srcUobj=getFromStack(fromWhich);
  if (!srcUobj) return;
  bmp=getBitmap(srcUobj);
  dstRec = QRect(srcUobj->data.imgData->x, srcUobj->data.imgData->y, srcUobj->data.imgData->width, srcUobj->data.imgData->height);
  if (bmp!=0)
   putImageData(dstRec, dstRec, bmp, toWhich);
  unRedoImage(srcUobj);
  removeMyData(srcUobj);
 }
 UndoObject *dstUobj= new UndoObject(MULTIPLE_IMAGE_CHG);
 if (!dstUobj)
 {
   qDebug("ApaintUnRedo pointer NULL!");
   return;
 }
 dstUobj->data.multipleImgNum = uobj->data.multipleImgNum;
 putOnStack(dstUobj, toWhich);  
}

void ApaintUnRedo::unRedoImage(UndoObject *uobj)
{ 
  int bpp, x, y, w, h, size, xi, yi, dx, dy, bsize;
  QString fname;
  gzFile f;
  uchar *buf;

  if (!docLoaded || !undoDirOk)  return;

  selLayer = doc->getLayer();
  if (!selLayer) return;
 
  bpp=selLayer->imageBuf.depth()>>3;  
  fname=uobj->data.imgData->fname;
  x=uobj->data.imgData->x;
  y=uobj->data.imgData->y;
  w=uobj->data.imgData->width;
  h=uobj->data.imgData->height;
   
  f=gzopen(fname, "rb6");
  if (!f) return;

  size=uobj->data.imgData->bmp.bpl*uobj->data.imgData->height;
  uobj->data.imgData->bmp.data=(uchar *)calloc(size,1);
  gzread(f, uobj->data.imgData->bmp.data, size);
  buf=(uchar *)&compBuf;
  bsize=size=0;
  for (yi=y; yi<y+h; yi++)
   for (xi=x; xi<x+w; xi++)
    if (pixelOn(&uobj->data.imgData->bmp, xi-x, yi-y))
    {
     if (size<=0 && !gzeof(f))
      {
       bsize=0;
       gzread(f, (int *)&size, sizeof(size));
       if (size>0)
        gzread(f, buf, size);       
      }
     
     if (size>0)     
      {
       memcpy(selLayer->imageBuf.scanLine(yi)+xi*bpp, buf+bsize, bpp);
       memcpy(selLayer->origBuf.scanLine(yi)+xi*bpp, buf+bsize, bpp);
       bsize+=bpp;
       size-=bpp;
      }
    }
 
  gzclose(f);
  dx = selLayer->layerPos().x();
  dy = selLayer->layerPos().y();
  if (selLayer->isUsingTiles())
    emit refreshRepeat(x+dx, y+dy, w, h);
  else
    emit refresh(x+dx, y+dy, w, h, 1);
  
}

uchar* ApaintUnRedo::getBitmap(UndoObject *uobj)
{
 uchar *bmp;
 int size;
 gzFile f;

 f=gzopen(uobj->data.imgData->fname,"rb6");
 if (!f)
  return(0);  
 size=uobj->data.imgData->bmp.bpl*uobj->data.imgData->height;
 bmp=(uchar *)calloc(size,1);
 gzread(f, bmp, size);
 gzclose(f);
 return bmp; 
}

/*used for UNDO/REDO function*/
void ApaintUnRedo::putLayerBackOnStack(int which, LayerChangeType typ)
{
 selLayer = doc->getLayer();
 if (!selLayer) return;

 switch (typ)
  {
   case CreateLayer:
    putLayerData(selLayer, DestroyLayer, which);
    break;
   case DestroyLayer:
    putLayerData(selLayer, CreateLayer, which);
    break;
   case Unselect:
    putLayerData(selLayer, Unselect, which);
    break;
   case ChangeLayerPos:
    putLayerData(selLayer, ChangeLayerPos, which);
    break;
   case ChangeLayerShiftX:
    putLayerData(selLayer, ChangeLayerShiftX, which);
    break;
   case ChangeLayerShiftY:
    putLayerData(selLayer, ChangeLayerShiftY, which);
    break;
   case SwapColors:
    putLayerData(selLayer, SwapColors, which);
    break;
   case ChangeLayerMirrorX:
    putLayerData(selLayer, ChangeLayerMirrorX, which);
    break;
   case ChangeLayerMirrorY:
    putLayerData(selLayer, ChangeLayerMirrorY, which);
    break;
   case ChangeLayerFlip:
    putLayerData(selLayer, ChangeLayerFlip, which);
    break;
   case ChangeLayerExplode:
    putLayerData(selLayer, ChangeLayerExplode, which);
    break;
   case InvertAllColors:
   case InvertRed:
   case InvertGreen:
   case InvertBlue:
   case InvertCyan:
   case InvertYellow:
   case InvertMagenta:
   case SwapRGB_GBR:
   case SwapRGB_BRG:
   case SwapRG:
   case SwapRB:
   case SwapGB:
    putLayerData(selLayer, typ, which);
    break;

   case ChangeLayerForm:
    putLayerData(selLayer, ChangeLayerForm, which);
    break;
   case ChangeColorTable:
    putLayerData(selLayer, ChangeColorTable, which);
    break;
   case ChangeSelToMainLayer:
    putLayerData(selLayer, ChangeMainToSelLayer, which);
    break;
   case ChangeMainToSelLayer:
    Layer *l = doc->getMainLayer();
    putLayerData(l, ChangeSelToMainLayer, which);
    break;
  }
}

void ApaintUnRedo::undoLast()
{
 UndoObject *uobj;
 uchar *bmp;
 double sx, sy, angle;
 QRect dstRec;
 int dstWid, dstHei;
  
 uobj=getFromStack(UNDO_STACK);
 if (!uobj) return;

 if (uobj->type == SPECIFIC)
  putOnStack(uobj, REDO_STACK);
 else if (uobj->type == LAYER_FILTER)
 {
  LayerFilterData &data = *(uobj->data.layerFilterData);
  int i;

  for (i=0; i<doc->numLayers(); i++)
   if (doc->getLayer(i) == data.selectedLayer) break;

  if (i >= doc->numLayers())
   {
    putOnStack(uobj, UNDO_STACK);
    return;
   }

  putOnStack(uobj, REDO_STACK);
  int oldLayer = doc->getLayerNum();
  doc->setLayerNum(i);
  int op;

  if (data.what == FILTER_MOVE_LEFT) op = FILTER_MOVE_RIGHT;
  else if (data.what == FILTER_MOVE_RIGHT) op = FILTER_MOVE_LEFT;
  else if (data.what == FILTER_MOVE_UP) op = FILTER_MOVE_DOWN;
  else if (data.what == FILTER_MOVE_DOWN) op = FILTER_MOVE_UP;
  else if (data.what == FILTER_TILT_LEFT) op = FILTER_TILT_RIGHT;
  else if (data.what == FILTER_TILT_RIGHT) op = FILTER_TILT_LEFT;
  else if (data.what == FILTER_TILT_UP) op = FILTER_TILT_DOWN;
  else if (data.what == FILTER_TILT_DOWN) op = FILTER_TILT_UP;

  doc->layerApplyFilter(op, data.fgCol, data.bgCol, data.penX, data.penY, data.value, FALSE, FALSE, FALSE);
  doc->setLayerNum(oldLayer);
 }
 else if (doc->inTransformMode())
 {
   if (uobj->type == TRANSFORM_CHANGE)
   {
     transTool->getScaleRotData(sx, sy, angle, dstWid, dstHei);
     putTransformData(sx, sy, angle, REDO_STACK, dstWid, dstHei);    
     getTransformData(uobj);
     removeMyData(uobj);     
   }
   else /*DONT UNDO YET, PUT IT BACK ON STACK*/
     putOnStack(uobj, UNDO_STACK);
 }
 else
 {
   bool transUndo=uobj->type == TRANSFORM_CHANGE;
   while (transUndo)
    {
     uobj=getFromStack(UNDO_STACK);
     if (!uobj) return;
     if (uobj->type == TRANSFORM_CHANGE)
      removeMyData(uobj);
     else
      transUndo = false;
    }
   if (uobj->type==IMAGE_CHANGE)
    {
     bmp=getBitmap(uobj);
     dstRec = QRect(uobj->data.imgData->x, uobj->data.imgData->y, uobj->data.imgData->width, uobj->data.imgData->height);
     if(bmp!=0)
      putImageData(dstRec, dstRec, bmp, REDO_STACK);
     unRedoImage(uobj);
    }
   else if (uobj->type==LAYER_CHANGE || uobj->type==UNSELECT)
    {
     QString fileName = doc->getMainLayer()->imageBuf.imageName();

     putLayerBackOnStack(REDO_STACK, uobj->data.ldata->ctyp);
     getLayerData(uobj, TRUE);
     doc->getMainLayer()->imageBuf.imageName() = fileName;
     doc->sendSignals(true, false);
    }
   else if (uobj->type == COLOR_CHANGE)
    {
     putColorData(colP->getColor(uobj->data.col->colorIndx), uobj->data.col->colorIndx, REDO_STACK);
     getColorData(uobj);
    }
   else if (uobj->type == MULTIPLE_IMAGE_CHG)
    {
     getMultipleImageData(uobj, UNDO_STACK, REDO_STACK);
    }   
   else if (uobj->type == CHANGE_LAYER)
    {
     putLayerNum(doc->getLayerNum(), REDO_STACK);
     getLayerNum(uobj);
    }
   else if (uobj->type == CHANGE_LAYER_TILES)
    {
     putTileData(REDO_STACK);
     getTileData(uobj); 
    }
   else if (uobj->type == CHANGE_INS_REM_PIX)
    {    
     putInsRemData(uobj->data.pixInsRem->horizontal, uobj->data.pixInsRem->pos, uobj->data.pixInsRem->num,
                   !uobj->data.pixInsRem->doInsert, REDO_STACK);
     getInsRemData(uobj);    
    }   
   removeMyData(uobj);
 }
 emit undo();
}

void ApaintUnRedo::redoLast()
{
 UndoObject *uobj;
 uchar *bmp;
 QRect dstRec;
 double sx, sy, angle;
 int  dstWid, dstHei;
 
 uobj=getFromStack(REDO_STACK);
 if (!uobj) return;

 if (uobj->type == SPECIFIC)
  putOnStack(uobj, UNDO_STACK);
 else if (uobj->type == LAYER_FILTER)
 {
  LayerFilterData &data = *(uobj->data.layerFilterData);
  int i;

  for (i=0; i<doc->numLayers(); i++)
   if (doc->getLayer(i) == data.selectedLayer) break;

  if (i >= doc->numLayers())
  {
   putOnStack(uobj, REDO_STACK);
   return;
  }

  putOnStack(uobj, UNDO_STACK);
  int oldLayer = doc->getLayerNum();
  doc->setLayerNum(i);
  doc->layerApplyFilter(data.what, data.fgCol, data.bgCol, data.penX, data.penY, data.value, FALSE, FALSE, FALSE);
  doc->setLayerNum(oldLayer);
 }
 else if (doc->inTransformMode())
 {
   if (uobj->type == TRANSFORM_CHANGE)
   {
     transTool->getScaleRotData(sx, sy, angle, dstWid, dstHei);
     putTransformData(sx, sy, angle, UNDO_STACK, dstWid, dstHei);
     getTransformData(uobj);
     removeMyData(uobj);
   }
   else
     putOnStack(uobj, REDO_STACK);
 }
 else
 {
   bool transUndo=uobj->type == TRANSFORM_CHANGE;
   while (transUndo)
   {
     uobj=getFromStack(REDO_STACK);
     if (!uobj) return;
     if (uobj->type == TRANSFORM_CHANGE)
      removeMyData(uobj);
     else
      transUndo = false;
   }
   if (uobj->type==IMAGE_CHANGE)
   {
    bmp=getBitmap(uobj);
    dstRec = QRect(uobj->data.imgData->x, uobj->data.imgData->y, uobj->data.imgData->width, uobj->data.imgData->height);
    if (bmp!=0)
     putImageData(dstRec, dstRec, bmp, UNDO_STACK);
    unRedoImage(uobj);
   }
   else if (uobj->type == LAYER_CHANGE)
   {
     QString fileName = doc->getMainLayer()->imageBuf.imageName();

     putLayerBackOnStack(UNDO_STACK, uobj->data.ldata->ctyp);
     getLayerData(uobj, FALSE);
     doc->getMainLayer()->imageBuf.imageName() = fileName;
     doc->sendSignals(true, false);
   }
   else if (uobj->type == UNSELECT)
    {
     if (!doc) return;
     if (!doc->getLayer()) return;
     if (doc->inTransformMode()) return;
     if (doc->getLayerType() == MainLayer) return;
     if (doc->isProtectMode()) return;

     QRect r=doc->getLayerPos();
     if (doc->isLayerDataLoaded())
     {
      putLayerData(doc->getLayer(), Unselect, UNDO_STACK);
      doc->collapsSelectedLayer(false);
     }
     doc->refreshLayer(r);
    }
   else if (uobj->type == COLOR_CHANGE)
   {
     putColorData(colP->getColor(uobj->data.col->colorIndx), uobj->data.col->colorIndx, UNDO_STACK);
     getColorData(uobj);
   }
   else if (uobj->type == MULTIPLE_IMAGE_CHG)
   {
     getMultipleImageData(uobj, REDO_STACK, UNDO_STACK); 
   }
   else if (uobj->type == CHANGE_LAYER)
   {
    putLayerNum(doc->getLayerNum(), UNDO_STACK); 
    getLayerNum(uobj); 
   }
   else if (uobj->type == CHANGE_LAYER_TILES)
   {
    putTileData(UNDO_STACK);
    getTileData(uobj);
   }
   else if (uobj->type == CHANGE_INS_REM_PIX)
   {
    putInsRemData(uobj->data.pixInsRem->horizontal, uobj->data.pixInsRem->pos, uobj->data.pixInsRem->num,
              !uobj->data.pixInsRem->doInsert, UNDO_STACK);
    getInsRemData(uobj);
   }  
   removeMyData(uobj);
 }
 emit redo();
}
