/***************************************************************************
                          apaintunredo.h  -  description
                             -------------------
    begin                : Sun Mar 16 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef APAINTUNREDO_H
#define APAINTUNREDO_H

#include <qobject.h>
#include <qptrstack.h>
#include <qpixmap.h>
#include <stdlib.h>
#include <qdir.h>
#include <qfile.h>
#include <qimage.h>
#include "apaintdoc.h"
#include "toolTransform.h"
#include "dialogColorPalette.h"
#include <zlib.h>

#define IMAGE_CHANGE          0
#define LAYER_CHANGE          1
#define TRANSFORM_CHANGE      2
#define COLOR_CHANGE          3
#define MULTIPLE_IMAGE_CHG    4
#define CHANGE_LAYER          5
#define CHANGE_LAYER_TILES    6
#define CHANGE_INS_REM_PIX    7
#define UNSELECT              8
#define SPECIFIC              9
#define LAYER_FILTER         10
 
#define BUF_SIZE      32768

typedef struct
{
  uchar *data;
  int bpl;
} TextureBitmap;

typedef struct
{
  int x,y, width, height;
  TextureBitmap bmp;
  QString fname;
} ImageData;

typedef struct
{
  int x, y;
} FHPoint;

typedef struct
{
  int x, y, wid, hei, shiftx, shifty;
  int swapColor1, swapColor2;
  QString fname;
  int xdpi, ydpi;
  int maskBpl;
  int imgBpl;
  int imgDepth;
  int mainImgBpl;
  bool usingMask;
  int numCol;
  int mainNumCol;
  int numCtrlPoints;
  /*if selection is out of main image or layer is ImageLayer we save layer data on HD*/
  bool layerDataSaved;
  LayerType ltyp;
  LayerGeometry lg;
  LayerChangeType ctyp;
  bool transColors[256];
  bool protectColors[256];
} LayerData;

typedef struct
{
 double sx, sy, angle;
 int dstWid, dstHei;
} TransFormData;

typedef struct
{
 uchar colorIndx;
 QRgb color; 
} ColorData;

typedef struct
{
 bool mirrorHor;
 bool mirrorVert;
} MirrorType;

typedef struct
{
 bool useTiles;
 int sizex, sizey, offx, offy;
 MirrorType *mirrorTab;
} TileData;

typedef struct
{
  bool doInsert, horizontal;
  int pos, num, size;
  QString fname;
} PixelInsRemData;

typedef struct
{
  Layer *selectedLayer;
  int what;
  int fgCol;
  int bgCol;
  int penX;
  int penY;
  int value;
} LayerFilterData;
                                
class UndoObject {
public:
  UndoObject(int typ);
  ~UndoObject();

public:    
 int type;
 union 
 {
   ImageData *imgData;
   LayerData *ldata;
   TransFormData *tdata;
   ColorData *col;
   TileData *tileData;
   PixelInsRemData *pixInsRem;
   LayerFilterData *layerFilterData;
   int multipleImgNum;
   FHPoint pos;
   int layerNum;
 } data;   
};    

typedef QPtrStack<UndoObject> UnRedoStack;
 class ApaintApp;
    
class ApaintUnRedo : public QObject  {
   Q_OBJECT
public: 
  ApaintUnRedo(ApaintDoc *docp, TransformTool *transToolp, ColPaletteDlg *colp);
  ~ApaintUnRedo();

  void undoLast();
  void redoLast();
  bool isDocChanged();
  void resetDocChanged();

signals:
  void refresh(int x, int y, int w, int h, int border);
  void refreshRepeat(int x, int y, int w, int h);
  void refreshRepeat(Layer *layer, int x, int y, int w, int h);
  void emptyUndo(bool onOff);
  void emptyRedo(bool onOff);
  void undo();
  void redo();
  void documentChanged(bool);
  
public slots:
  void putLayerFilterData(int what, int fgCol, int bgCol, int penX, int penY, int value);
  void putImageData(QRect &srcRec, QRect &dstRec);
  void putLayerData(Layer *sel, LayerChangeType ctyp);
  void putTransformData(double sx, double sy, double angle, int dstWid, int dstHei);
  void putColorData(QRgb col, int indx);
  void putLayerNum(int lnum);  
  void putTileData();
  void newDocSlot();
  void putInsRemData(bool horizontal, int pos, int num, bool doInsert);
  void putSpecificData();
  void clearSpecificData();
    
private:
  void putImageData(QRect &srcRec, QRect &dstRec, uchar *bmp, int which);
  void putMultipleImageData(QRect &srcRec, QRect &dstRec);  
  void getMultipleImageData(UndoObject *uobj, int fromWhich, int toWhich);
  void unRedoImage(UndoObject *uobj);
  void clearStack(UnRedoStack *stackIn);
  void removeMyData(UndoObject *uobj);
  void putOnStack(UndoObject *uobj, short which);
  UndoObject* getFromStack(short which);
  uchar* getBitmap(UndoObject *uobj);
  void putLayerData(Layer *sel, LayerChangeType ctyp, int which);
  void getLayerData(UndoObject *uobj, bool undo);
  void putTransformData(double sx, double sy, double angle, int which, int dstWid, int dstHei);
  void getTransformData(UndoObject *uobj);
  void putLayerBackOnStack(int which, LayerChangeType typ);
  void putColorData(QRgb col, int indx, int which);
  void getColorData(UndoObject *uobj);
  void fhPts2Qpts(FHPoint *fhPts, QPointArray *qPts, int num);
  void putLayerNum(int lnum, int which);  
  void getLayerNum(UndoObject *uobj);
  void putTileData(int which);
  void getTileData(UndoObject *uobj);
  void putInsRemData(bool horizontal, int pos, int num, bool doInsert, int which);
  void getInsRemData(UndoObject *uobj);
  
  UnRedoStack stack;
  UnRedoStack redoStack;        
  bool docLoaded;
  bool undoDirOk;
  QString undoDir;
  int undoStackNum, redoStackNum;
  uchar compBuf[BUF_SIZE];
  Layer *selLayer, *tmpLayer;
  ApaintDoc *doc;
  TransformTool *transTool;
  ColPaletteDlg *colP;
  int undoStackWhenSaved;
};

#endif
