/***************************************************************************
                          dialogBrushList.cpp  -  description
                             -------------------
    begin                : Thu Jul 15 2004
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dialogBrushList.h"
#include <qlayout.h>
#include <qgroupbox.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qtooltip.h>
#include <qprocess.h>
#include "widgetIconView.h"
#include "apaintBrushList.h"
#include "apaintconfig.h"
#include "iconViewData.h"
#include "apaintdoc.h"

extern char *getCommonPNG(const char *pixName);

BrushListWidget::BrushListWidget(QWidget *parent, const char *name, ApaintDoc *docp):BrushSelectDialogUI(parent, name)
{ 
  gbProp->setTitle(ApaintConfig::getText(100));
  gbBrush->setTitle(ApaintConfig::getText(722)+":");
  cbAbs->setText(ApaintConfig::getText(ABSOLUTE_MSG));
  gbBrush->setColumnLayout(0, Qt::Vertical );
  gbBrush->layout()->setSpacing( 1 );
  gbBrush->layout()->setMargin( 1 );
  vbox = new QVBoxLayout(gbBrush->layout());
  iconView = new IconListView(gbBrush, "iconViewList2");
  iconView->setMaximumWidth(170);
  iconView->setMinimumSize(100, 20);
  iconView->setSizePolicy( QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Preferred ));
  vbox->addWidget(iconView);
  Q_CHECK_PTR(docp);
  doc = docp;
  ApaintBrushList *bList = ApaintBrushList::getObject();    
  iconView->setIconList(bList->getIconList());
  connect(bList, SIGNAL(listIsChanged()), this, SLOT(updateBrushList()));
  connect(iconView, SIGNAL(iconSelectionChanged()), this, SLOT(updateIconData()));
  connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer *)));
  connect(btnDelete, SIGNAL(clicked()), this, SLOT(removeBrush()));
  connect(btnOpen, SIGNAL(clicked()), this, SIGNAL(loadFileBrush()));
  connect(btnBrowse, SIGNAL(clicked()), this, SIGNAL(browseFileBrush()));
  connect(iconView, SIGNAL(deleteImageSig()), this, SLOT(removeBrush()));
  connect(btnBL, SIGNAL(clicked()), this, SIGNAL(brushToLayerSig()));
  connect(btnLB, SIGNAL(clicked()), this, SIGNAL(layerToBrushSig()));
  connect(btnOpenWithPaint, SIGNAL(clicked()), this, SLOT(openWithPaintBtnClicked()));

  connect(sbOffX, SIGNAL(valueChanged(int)), this, SLOT(setOffsetX(int)));
  connect(sbOffY, SIGNAL(valueChanged(int)), this, SLOT(setOffsetY(int)));
  connect(cbAbs, SIGNAL(toggled(bool)), this, SLOT(setAbsOffset(bool)));
  QToolTip::add(btnOpen, ApaintConfig::getText(731));
  QToolTip::add(btnBrowse, ApaintConfig::getText(657));
  QToolTip::add(btnDelete, ApaintConfig::getText(732));
  QToolTip::add(btnBL, ApaintConfig::getText(725));
  QToolTip::add(btnLB, ApaintConfig::getText(724));
  QToolTip::add(btnOpenWithPaint, ApaintConfig::getText(827));
  btnOpen->setPixmap(QPixmap(getCommonPNG("document-open")));
  btnBrowse->setPixmap(QPixmap(getCommonPNG("document-browse")));
  btnDelete->setPixmap(QPixmap(getCommonPNG("edit-delete")));
  btnBL->setPixmap(QPixmap(getCommonPNG("1leftarrow")));
  btnLB->setPixmap(QPixmap(getCommonPNG("1rightarrow")));
  btnOpenWithPaint->setIconSet(QPixmap(getCommonPNG("ArahPaint4-22")));
}

BrushListWidget::~BrushListWidget()
{
  
}

void BrushListWidget::removeBrush()
{  
  ApaintBrushList *bList = ApaintBrushList::getObject();
  bList->removeBrush();
  iconView->setupContents(true);
}

void BrushListWidget::updateBrushList()
{
  QFontMetrics fm(ApaintConfig::getFontName());
  ApaintBrushList *bList = ApaintBrushList::getObject();

  bList->updateBrushList(fm.height(), false);  
  iconView->setupContents(true);
  updateIconData();
}

void BrushListWidget::layerChangedSlot(Layer *layer)
{
  updateIconData();
}

void BrushListWidget::updateIconData()
{
  ApaintBrushList *bList = ApaintBrushList::getObject();
  Layer *layer = doc->getLayer();
  if (!layer) return;
  bList->loadBrush(layer->imageBuf.numColors(), layer->imageBuf.colorTable());
}

void BrushListWidget::setOffsetX(int val)
{
  ApaintBrushList *bList = ApaintBrushList::getObject();
  bList->setOffsetX(val);
}

void BrushListWidget::setOffsetY(int val)
{
  ApaintBrushList *bList = ApaintBrushList::getObject();
  bList->setOffsetY(val);
}

void BrushListWidget::openWithPaintBtnClicked()
{  
 ApaintBrushList *bList = ApaintBrushList::getObject();  

 if (bList->isValid())
  {
   QProcess *proc;

   proc = new QProcess( this );
   proc->addArgument("./arahpaint4");
   proc->addArgument(bList->getBrushName());
   proc->start();
  }
}

void BrushListWidget::setAbsOffset(bool enable)
{
  ApaintBrushList *bList = ApaintBrushList::getObject();
  bList->setAbsOffset(enable); 
}

void BrushListWidget::prevBrush()
{
  iconView->goUp();
}

void BrushListWidget::nextBrush()
{
  iconView->goDown();
}



