/***************************************************************************
;                          colorpalette.h  -  description
                             -------------------
    begin                : Fri Apr 11 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef COLORPALETTE_H
#define COLORPALETTE_H

#include <qtable.h>
#include <qrect.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include "apaintimage.h"
#include "apaintdoc.h"  

/***********************ColorPalette CLASS***************************************************/

class ColorPalette : public QScrollView  {
  Q_OBJECT
public:
   ColorPalette(QWidget *parent, const char *name);
  ~ColorPalette();
  int colorNum() {return cNum;}
  QRgb *getColorTab(){ return colorTable;}
  QRgb *getTransColorTab();
  void enableColorDelete(bool val) { allowColorDelete = val; }
  void setColorTable(QRgb *tab, int num);
  void setVisibleColors(bool *visibleCol);
  int visibleColorNum();
  QRgb getSelColor(int &indx);
  int getColorPos();

signals:
  void colorChanged(QRgb rgb, int num);  
  void putColorUndo(QRgb col, int colIndx);
  
public slots:

  void setColor(QRgb rgb, bool doUndo);
  void setColorPos(int indx);
  void showColor(int i);
  void hideColor(int i);  

protected:

  void refreshCell(int col, int row);
  QRect getCell(int col, int row);
  void getCords(int x, int y, int *col, int *row);
  int numCols() {return nc;}
  int numRows() {return nr;}
  void setColorNum(int num) {cNum=num;}
  void setUpContents(int num);
  void keyPressEvent( QKeyEvent *e );
  virtual void cellClicked(ButtonState bs, int col, int row, bool doubleClick);
  virtual void drawContents(QPainter *p, int clipx, int clipy, int clipw, int cliph);
  virtual void contentsMousePressEvent (QMouseEvent *e );
  virtual void contentsMouseDoubleClickEvent (QMouseEvent *e );
  virtual QSize sizeHint() const;
  virtual void drawProperties(QPainter *p,  QRect r, int n);
  void viewportResizeEvent(QResizeEvent *re);
  int getColorPos(int col, int row);
  void getColRow(int i, int &col, int &row);
  void initColorTable(int num);
//  bool colorExist(QRgb col);
  void setValidTable(bool enable){ validTable = enable;}
  bool isValidTable() {return validTable;}
  virtual void setDoc(ApaintDoc *docp) {doc = docp;}

  QRgb *colorTable;
  int selCol, selRow;
  bool selectedIsChanged;    
  
private:
  
  QRgb transColorTab[256];
  int cNum;
  QPixmap colPix;
  int nr, nc; 
  bool allowColorDelete;
  bool visibleColors[256];
  bool validTable;
  ApaintDoc *doc;
};

class ColorPaletteImage : public ColorPalette  {
  Q_OBJECT
public:
   ColorPaletteImage(QWidget *parent, const char *name,  ApaintDoc *docp);
  ~ColorPaletteImage();
  void showUsedColors(bool enable);
  bool isShowingUsedColors(){ return shUsedCol;}
  void setProtectedColor(bool *tab);
  void setTransColor(bool *tab);
  void mergeSameColors();
  
signals:
  void bgFgChanged(int fgn, QRgb fgc, int bgn, QRgb bgc);
  
public slots:  
  void layerChangedSlot(Layer *sel);
  void setColor(QRgb rgb,  bool doUndo);
  
protected:
  void cellClicked(ButtonState bs, int col, int row, bool doubleClick);
  void drawProperties(QPainter *p,  QRect r, int n);
  QSize sizeHint() const;
  
private:
  bool usedColorsShow[256];
  Layer *selLayer;
  ApaintDoc *doc;
  bool docLoaded;
  bool shUsedCol;
  QPixmap locked;
  QPixmap transparent;
};



/*************************COLOR PALETTE DIALOG********************************/

class ColPaletteDlg : public QWidget
{
    Q_OBJECT

public:
    ColPaletteDlg( QWidget* parent , const char* name, ApaintDoc *docp);
    ~ColPaletteDlg();

    QRgb getColor(int indx);
    QRgb getSelColor(int &indx) {return colPWidget->getSelColor(indx);}
    int getColorPos();
    
signals:
     void colorChanged(QRgb rgb, int n);
     void refresh(int x, int y, int w, int h);
     void putColorUndo(QRgb c, int n);
     void bgFgChanged(int fgn, QRgb fgc, int bgn, QRgb bgc);
     
public slots:
     void setColorPos(int num);
     void setColor(QRgb rgb, bool doUndo);
     void layerChangedSlot(Layer *layer);
     void disableUsedColors();
     
protected slots:
     void showUsedColors();
     void showUsedColors(bool enable);
     void removeUnusedColors();
     void mergeColors();
     void addColor();
     void removeColor();

private:

    ColorPaletteImage* colPWidget;
    QPushButton* btnRefresh;
    QPushButton* btnRemove;
    QPushButton* btnMerge;
    QPushButton *btnPlus, *btnMinus;
    QVBoxLayout* ColPaletteDlgLayout;
    QHBoxLayout* Layout1;
    ApaintDoc *doc;
};

#endif
