/***************************************************************************
                          drawinginrepeatdlg.cpp  -  description
                             -------------------
    begin                : Tue Sep 16 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dialogDrawRepeat.h"
#include <qbuttongroup.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include "apaintdoc.h"
#include "apaintview.h"
#include "apaintconfig.h"
#include "apaint.h"

extern char *getCommonPNG(const char *pixName);

DrawingInRepeatDlg::DrawingInRepeatDlg(QWidget *parent, const char *name, ApaintDoc *docp):DrawingInRepeatDialogUI(parent, name)
{ 
 /*set captions...*/
 setCaption(ApaintConfig::getText(43));
 gbPreview->setTitle(ApaintConfig::getText(PREVIEW_MSG));
 bgRepMode->setTitle(ApaintConfig::getText(142));
 cbOff->setCurrentItem(0);
 cbOff->setCurrentText(ApaintConfig::getText(ABSOLUTE_MSG));
 bgOff->setTitle(ApaintConfig::getText(100));
 lbOff->setText(ApaintConfig::getText(147)+":");
 gbHorizontal->setTitle(ApaintConfig::getText(46));
 lbSelWidth->setText(ApaintConfig::getText(WIDTH_MSG)+":");
 lbPasteTimes->setText(ApaintConfig::getText(101)+":");
 gbVertical->setTitle(ApaintConfig::getText(47));
 lbSelHeight->setText(ApaintConfig::getText(HEIGHT_MSG)+":");
 lbPasteTimes_2->setText(ApaintConfig::getText(101)+":");
 cbNewImage->setText(ApaintConfig::getText(NEW_PICTURE_MSG));
 btnOK->setText(ApaintConfig::getText(OK_MSG));
 btnCancel->setText(ApaintConfig::getText(86));

 /* there must be a better way to reference child widgets */
 bgRepMode->find(0)->setPixmap(QPixmap(getCommonPNG("repeat-block")));
 bgRepMode->find(1)->setPixmap(QPixmap(getCommonPNG("repeat-brick")));
 bgRepMode->find(2)->setPixmap(QPixmap(getCommonPNG("repeat-drop")));

 makeNewImage = false;
 
 connect(btnOK, SIGNAL(clicked()), this, SLOT(okBtnClicked()));
 connect(btnCancel, SIGNAL(clicked()), this, SLOT(cancelBtnClicked()));
 connect(bgRepMode, SIGNAL(clicked(int)), this, SLOT(setRepeatMode(int)));
 connect(sbRepTimesWid, SIGNAL(valueChanged(int)), this, SLOT(setRepX(int)));
 connect(sbRepTimesHei, SIGNAL(valueChanged(int)), this, SLOT(setRepY(int)));
 connect(sbOffset, SIGNAL(valueChanged(int)), this, SLOT(offsetChanged(int)));
 connect(cbNewImage, SIGNAL(clicked()), this, SLOT(setNewImage()));
 connect(cbOff, SIGNAL(activated(int)), this, SLOT(setOffset(int)));
 Q_CHECK_PTR(docp);
 doc = docp;
 gbPreview->setColumnLayout(0, Qt::Vertical );
 vbox = new QVBoxLayout(gbPreview->layout());
 previewDoc = new ApaintDoc(this); 
 preview = new ApaintView(gbPreview, previewDoc); 
 vbox->addWidget(preview);
 connect(previewDoc, SIGNAL(newDocSig()), preview, SLOT(documentChangedSlot()));
 connect(previewDoc, SIGNAL(mainLayerChanged()), preview, SLOT(documentChangedSlot())); 
 previewDoc->load(getCommonPNG("repeatDraw"));
 mainLayer = previewDoc->getMainLayer();
 connect(preview, SIGNAL(mousePressSig(QMouseEvent*, QPainter*)), this, SLOT(mousePressedSlot(QMouseEvent*, QPainter*)));
 connect(preview, SIGNAL(zoomChanged(double, double)), this, SLOT(zoomChangedSlot(double, double)));
 connect(this, SIGNAL(refreshSig(int, int, int, int)), preview, SLOT(refresh(int, int, int, int)));
 connect(previewDoc, SIGNAL(refreshSig(int, int, int, int)), preview, SLOT(refresh(int, int, int, int)));
 scaleX = scaleY = 1;
 offsetMain = offsetPreview = 0;
 pviewPos = previewDoc->getLayerPos();
 setRepeatMode(Brick);
 connect(doc, SIGNAL(layerChanged(Layer*)), this, SLOT(layerChangedSlot(Layer*)));
}

DrawingInRepeatDlg::~DrawingInRepeatDlg()
{
  
}

void DrawingInRepeatDlg::layerChangedSlot(Layer* layer)
{
  if (layer->getType() == MainLayer)
   setupDialog();

}

void DrawingInRepeatDlg::setNewImage()
{
 makeNewImage  = cbNewImage->isOn();
}

void DrawingInRepeatDlg::copyTileData(Layer *dst, Layer *src)
{
  Tile *srcTile, *dstTile;

  QPtrList<Tile> &tileList= src->getTileList();
  QPtrList<Tile> &docTileList = dst->getTileList();
  for (int i=0; i<(int)min(docTileList.count(), tileList.count()); i++)
   {
    srcTile = tileList.at(i);
    dstTile = docTileList.at(i);
    dstTile->setMirrorHor(srcTile->mirrorHor());
    dstTile->setMirrorVert(srcTile->mirrorVert());
   } 
}

void DrawingInRepeatDlg::setLayerTiles(Layer *layer, int offset)
{
  if (getRepeatMode() == Brick)
     layer->createTiles(getRepX(), getRepY(), 0, 0);
  else if (getRepeatMode() == HorMove)
     layer->createTiles(getRepX(), getRepY(), offset, 0);
  else if (getRepeatMode() == VertMove)
     layer->createTiles(getRepX(), getRepY(), 0, offset); 
}

void DrawingInRepeatDlg::okBtnClicked()
{ 
 disconnect(doc, SIGNAL(layerChanged(Layer*)), this, SLOT(layerChangedSlot(Layer*)));
 Layer *layer = doc->getMainLayer();

/* this causes fabric modified on two times view repeat
  if (makeNewImage)
   doc->putUndoStack(ChangeLayerForm);
  else
   emit setTileUndo();
*/

 setLayerTiles(layer, offsetMain);
 copyTileData(layer, mainLayer);   
 if (makeNewImage)
  {
   doc->putUndoStack(ChangeLayerForm);
   doc->newImageFromTiles();
   resetDocTiles(previewDoc);
   setupDialog();
   close();
  }
 else
  {
   doc->createMainLayerTiles(0, 0, 0, 0, true);
   doc->refreshLayer();   
  }
 connect(doc, SIGNAL(layerChanged(Layer*)), this, SLOT(layerChangedSlot(Layer*)));
}

void DrawingInRepeatDlg::cancelBtnClicked()
{
 close(); 
}

void DrawingInRepeatDlg::setRepX(int wid)
{
 repx = wid;
 setViewWid(origPos.width()*repx);
 setUpLayer(previewDoc, offsetPreview);
}

void DrawingInRepeatDlg::setRepY(int hei)
{
  repy = hei;
  setViewHei(origPos.height()*repy);
  setUpLayer(previewDoc, offsetPreview);
}

void DrawingInRepeatDlg::setOffset(int indx)
{
  if (indx>0)
  {
    if (getRepeatMode() == HorMove)
      sbOffset->setValue(origPos.width() / (indx+1));
    else if (getRepeatMode() == VertMove)
      sbOffset->setValue(origPos.height() / (indx+1));
  }
}

void DrawingInRepeatDlg::offsetChanged(int val)
{
  offsetMain = val;
  if (getRepeatMode() == HorMove)
    offsetPreview =(int) ((double)val*((double)pviewPos.width() / (double)origPos.width()) + 0.5);
  else if (getRepeatMode() == VertMove)
    offsetPreview =(int) ((double)val*((double)pviewPos.height() / (double)origPos.height()) + 0.5);
  setUpLayer(previewDoc, offsetPreview);
}

void DrawingInRepeatDlg::setViewWid(int wid)
{
  lbWid->setText(QString("%1").arg(wid));
}

void DrawingInRepeatDlg::setViewHei(int hei)
{
  lbHei->setText(QString("%1").arg(hei)); 
}

void DrawingInRepeatDlg::setRepeatMode(int mode)
{
  rmode = (RepeatMode) mode;
  if (mode == Brick)
   bgOff->setEnabled(false);
  else
   bgOff->setEnabled(true);
  setUpLayer(previewDoc, offsetPreview);
}

DrawingInRepeatDlg::RepeatMode DrawingInRepeatDlg::getRepeatMode()
{
 return rmode; 
}

void DrawingInRepeatDlg::zoomChangedSlot(double zx, double zy)
{
  scaleX = zx;
  scaleY = zy;
}

void DrawingInRepeatDlg::refreshRect(QRect r)
{
 int x1, y1, x2, y2;

  x1=(int)(r.x()*scaleX);
  y1=(int)(r.y()*scaleY);
  x2=(int)((r.x()+r.width())*scaleX);
  y2=(int)((r.y()+r.height())*scaleY);

  emit refreshSig(x1, y1, x2-x1, y2-y1);
}

void DrawingInRepeatDlg::mousePressedSlot(QMouseEvent *e, QPainter *pc)
{
  Tile *tile;
  int x = (int)(e->x()/scaleX);
  int y = (int)(e->y()/scaleY);

  tile = mainLayer->pointInTile(x, y);
  if (!tile) return;

  bool mh = tile->mirrorHor();
  bool mv = tile->mirrorVert();

  if (!mh && !mv)  
   tile->setMirrorHor(true);
  else if (mh && !mv)
  {
   tile->setMirrorHor(false);
   tile->setMirrorVert(true);
  }
  else if (!mh && mv)
  {
   tile->setMirrorVert(true);
   tile->setMirrorHor(true);
  }
  else if (mh && mv)
  {
   tile->setMirrorVert(false);
   tile->setMirrorHor(false);
  }
  refreshRect(tile->getPosMain());
  refreshRect(tile->getPosSecond());
}

void DrawingInRepeatDlg::setUpLayer(ApaintDoc *adoc, int offset)
{
  if (getRepeatMode() == Brick)
    adoc->createMainLayerTiles(getRepX(), getRepY(), 0, 0, false);
  else if (getRepeatMode() == HorMove)
    adoc->createMainLayerTiles(getRepX(), getRepY(), offset, 0, false);
  else if (getRepeatMode() == VertMove)
    adoc->createMainLayerTiles(getRepX(), getRepY(), 0, offset, false);
}

void DrawingInRepeatDlg::resetDocTiles(ApaintDoc *d)
{
  d->createMainLayerTiles(1, 1, 0, 0, false);
}

void DrawingInRepeatDlg::setupDialog()
{
  int sizex, sizey, offx, offy;

  doc->mainLayer->getTileInfo(sizex, sizey, offx, offy);
  sbRepTimesWid->setValue(sizex);
  sbRepTimesHei->setValue(sizey);
  cbOff->setCurrentItem(0);
  if (offx == 0 && offy == 0)
  {
   bgRepMode->setButton(Brick);
   sbOffset->setValue(0);   
  }
  else if (offx!=0)
  {
    bgRepMode->setButton(HorMove);
    sbOffset->setValue(offx);
  }
  else
  {
    bgRepMode->setButton(VertMove);
    sbOffset->setValue(offy);
  }
  
  origPos = doc->mainLayer->realLayerPos();

  setViewWid(getRepX()*origPos.width());
  setViewHei(getRepY()*origPos.height());
  cbNewImage->setChecked(false);
  setNewImage();
}

void DrawingInRepeatDlg::showDlg()
{ 
  setupDialog();
  show();
}


