/***************************************************************************
                          textfontselect.cpp  -  description
                             -------------------
    begin                : Tue Sep 2 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "qwindowdefs.h"

#include "dialogFontSelect.h"
#include "qlineedit.h"
#include "qlistbox.h"
#include "qpushbutton.h"
#include "qcheckbox.h"
#include "qcombobox.h"
#include "qlayout.h"
#include "qvgroupbox.h"
#include "qhgroupbox.h"
#include "qlabel.h"
#include "qapplication.h"
#include "qfontdatabase.h"
#include "qstyle.h"
#include "qtextedit.h"
#include "qregexp.h"
#include "qradiobutton.h"
#include "qvbuttongroup.h"
#include "apaintconfig.h"
#include "qspinbox.h"
#include "qsize.h"
#include "qtoolbutton.h"

extern char *getCommonPNG(const char *pixName);

class QFontDialogPrivate
{
public:
    QFontDialogPrivate(){};
    QLabel * familyAccel;
    QLineEdit * familyEdit;
    QListBox * familyList;

    QLabel * styleAccel;
    QLineEdit * styleEdit;
    QListBox * styleList;

    QLabel * sizeAccel;
    QSpinBox * sizeEdit;
    QListBox * sizeList;

    QVButtonGroup * backColor;
    QRadioButton *transColor;
    QRadioButton *bgColor;
    
    QVGroupBox * effects;
    QCheckBox * strikeout;
    QCheckBox * underline;
    QLabel * colorAccel;
    QComboBox * color;

    QHGroupBox * sample;
    QTextEdit * sampleEdit;

    QLabel * scriptAccel;
    QComboBox * scriptCombo;

    QToolButton *alignLeft;
    QToolButton *alignRight;
    QToolButton *alignCenter;
    QToolButton *alignFill;

    QPushButton *insertBtn;
    QPushButton * cancel;

    QBoxLayout * buttonLayout;
    QBoxLayout * effectsLayout;
    QBoxLayout * sampleLayout;
    QBoxLayout * sampleEditLayout;
    QVBoxLayout *mainVLayout;

    QFontDatabase fdb;
    QString       family;
    QString       script;
    QString       style;
    QString       size;

    QStringList familyNames;
    QStringList scriptNames;
    QValueList<int> scriptScripts;
    QStringList scriptSamples;
    bool usingStandardSizes;
};


/*!
  \internal
  Constructs a standard font dialog.

  Use setFont() to set the initial font attributes.

  The \a parent, \a name, \a modal and \a f parameters are passed to
  the QDialog constructor.

  \sa getFont()
*/

TextFontSelect::TextFontSelect( QWidget *parent, const char *name,bool modal, WFlags f ) : QDialog( parent, name, modal, f )
{
    setCaption( ApaintConfig::getText(170));
    setSizeGripEnabled( TRUE );
    /*pointer reset*/
    d = new QFontDialogPrivate;
    // grid
    d->familyEdit = new QLineEdit( this, "font family I" );
    d->familyEdit->setFocusPolicy( StrongFocus );
    d->familyList = new QListBox( this, "font family II" );
    d->familyList->viewport()->setFocusProxy( d->familyEdit );

    d->familyAccel = new QLabel( d->familyEdit, ApaintConfig::getText(169), this, "family accelerator" );
    d->familyAccel->setIndent( 2 );

    d->styleEdit = new QLineEdit( this, "font style I" );
    d->styleEdit->setFocusPolicy( StrongFocus );
    d->styleList = new QListBox( this, "font style II" );
    d->styleList->viewport()->setFocusProxy( d->styleEdit );
    d->styleAccel = new QLabel( d->styleEdit, ApaintConfig::getText(168), this, "style accelerator" );
    d->styleAccel->setIndent( 2 );

    d->sizeEdit = new QSpinBox( this, "font size I" );
    d->sizeEdit->setFocusPolicy( StrongFocus );
    d->sizeList = new QListBox( this, "font size II" );
    d->sizeList->viewport()->setFocusProxy( d->sizeEdit );
    d->sizeAccel = new QLabel ( d->sizeEdit, ApaintConfig::getText(167), this, "size accelerator" );
    d->sizeAccel->setIndent( 2 );

    // effects box
    d->effects = new QVGroupBox( ApaintConfig::getText(633), this, "font effects" );
    d->strikeout = new QCheckBox( d->effects, "strikeout on/off" );
    d->strikeout->setText( ApaintConfig::getText(634));
    d->underline = new QCheckBox( d->effects, "underline on/off" );
    d->underline->setText( ApaintConfig::getText(635) );

    d->backColor = new QVButtonGroup(ApaintConfig::getText(616), this, "bgColorBox");
    d->backColor->setRadioButtonExclusive(true);
    d->transColor = new QRadioButton(ApaintConfig::getText(636), d->backColor);
    d->transColor->setChecked(true);
    d->bgColor = new QRadioButton(ApaintConfig::getText(178), d->backColor);
    

    d->sample = new QHGroupBox( ApaintConfig::getText(166), this, "sample text" );
    d->sampleEdit = new QTextEdit( d->sample, "r/w sample text" );

    // Note that the sample text is *not* translated with tr(), as the
    // characters used depend on the charset encoding.
    d->sampleEdit->setText( ApaintConfig::getText(637));

    d->sampleEdit->setAlignment( AlignLeft );
    d->sampleEdit->setVScrollBarMode(QScrollView::Auto);
    d->sampleEdit->setHScrollBarMode(QScrollView::Auto);
    d->scriptCombo = new QComboBox( TRUE, this, "font encoding" );
    d->scriptCombo->setFocusPolicy( StrongFocus );

    d->scriptAccel = new QLabel( d->scriptCombo, ApaintConfig::getText(638), this,"encoding label");
    d->scriptAccel->setIndent( 2 );

    QHBoxLayout *align = new QHBoxLayout(0, 0, 6);
    align->addItem(new QSpacerItem(0,0, QSizePolicy::Expanding, QSizePolicy::Minimum));
    d->alignLeft = new QToolButton(this);
    d->alignLeft->setIconSet(QIconSet(QPixmap(getCommonPNG("format-justify-left"))));
    d->alignLeft->setToggleButton(true);
    d->alignLeft->setOn(true);
    align->addWidget(d->alignLeft);
    d->alignRight = new QToolButton(this);
    d->alignRight->setIconSet(QIconSet(QPixmap(getCommonPNG("format-justify-right"))));
    d->alignRight->setToggleButton(true);
    align->addWidget(d->alignRight);
    d->alignCenter = new QToolButton(this);
    d->alignCenter->setIconSet(QIconSet(QPixmap(getCommonPNG("format-justify-center"))));
    d->alignCenter->setToggleButton(true);
    align->addWidget(d->alignCenter);
    d->alignFill = new QToolButton(this);
    d->alignFill->setIconSet(QIconSet(QPixmap(getCommonPNG("format-justify-fill"))));
    d->alignFill->setToggleButton(true);
    align->addWidget(d->alignFill);
    align->addItem(new QSpacerItem(0,0, QSizePolicy::Expanding, QSizePolicy::Minimum));

    d->usingStandardSizes = FALSE;

    connect(d->sampleEdit, SIGNAL(textChanged()), this, SLOT(setSizeLabel()));
    connect(d->backColor, SIGNAL(clicked(int)), this, SLOT(bgColorClicked(int)));
    
#if 0
    connect( d->familyList, SIGNAL(highlighted(const QString&)),
	     SLOT(familyHighlighted(const QString&)) );
#else
    connect( d->familyList, SIGNAL(highlighted(int)),
	     SLOT(familyHighlighted(int)) );
#endif
    connect( d->scriptCombo, SIGNAL(activated(int)),
	     SLOT(scriptHighlighted(int)) );
    connect( d->styleList, SIGNAL(highlighted(const QString&)),
	     SLOT(styleHighlighted(const QString&)) );
    connect( d->sizeList, SIGNAL(highlighted(const QString&)),
	     SLOT(sizeHighlighted(const QString&)) );

    connect( d->sizeEdit, SIGNAL(valueChanged( const QString &)),
	     SLOT(sizeChanged(const QString&)) );

    connect( d->strikeout, SIGNAL(clicked()),
	     SLOT(updateSample()) );
    connect( d->underline, SIGNAL(clicked()),
	     SLOT(updateSample()) );

    connect(d->alignLeft, SIGNAL(pressed()), this, SLOT(leftPressed()));
    connect(d->alignRight, SIGNAL(pressed()), this, SLOT(rightPressed()));
    connect(d->alignCenter, SIGNAL(pressed()), this, SLOT(centerPressed()));
    connect(d->alignFill, SIGNAL(pressed()), this, SLOT(fillPressed()));

    updateFamilies();
    if ( d->familyList->count() != 0 )
	    d->familyList->setCurrentItem( 0 );

/*
    QSize sz;
    sz = d->familyList->sizeHint();
    sz = d->styleList->sizeHint();
    sz = d->sizeList->sizeHint();
*/
    // grid layout
    QGridLayout * mainGrid = new QGridLayout( this, 9, 6, 12, 0 );

    mainGrid->addWidget( d->familyAccel, 0, 0 );
    mainGrid->addWidget( d->familyEdit, 1, 0 );
    mainGrid->addWidget( d->familyList, 2, 0 );

    mainGrid->addWidget( d->styleAccel, 0, 2 );
    mainGrid->addWidget( d->styleEdit, 1, 2 );
    mainGrid->addWidget( d->styleList, 2, 2 );

    mainGrid->addWidget( d->sizeAccel, 0, 4 );
    mainGrid->addWidget( d->sizeEdit, 1, 4 );
    mainGrid->addWidget( d->sizeList, 2, 4 );

    mainGrid->setColStretch( 0, 38 );
    mainGrid->setColStretch( 2, 24 );
    mainGrid->setColStretch( 4, 10 );

    mainGrid->addColSpacing( 1, 6 );
    mainGrid->addColSpacing( 3, 6 );
    mainGrid->addColSpacing( 5, 6 );

    mainGrid->addRowSpacing( 3, 12 );

    mainGrid->addMultiCellWidget( d->effects, 4, 6, 0, 0);
    mainGrid->addMultiCellWidget(d->backColor, 4, 6, 2, 2);

    mainGrid->addWidget( d->scriptAccel, 4, 4 );
    mainGrid->addWidget( d->scriptCombo, 5, 4 );

    mainGrid->addLayout(align, 6, 4);

    mainGrid->addMultiCellWidget( d->sample, 7, 7, 0, 4 );

    mainGrid->addRowSpacing( 8, 12 );

    QHBoxLayout *buttonBox = new QHBoxLayout;
    mainGrid->addMultiCell( buttonBox, 9, 9, 0, 4 );

    buttonBox->addStretch( 1 );

    mainGrid->setRowStretch( 1, 10 );
    mainGrid->setRowStretch( 7, 20 );

    
    d->insertBtn = new QPushButton(ApaintConfig::getText(639), this, "insert");
    buttonBox->addWidget(d->insertBtn);
    connect(d->insertBtn, SIGNAL(clicked()), this, SIGNAL(insertBtnClicked()));
    d->insertBtn->setFixedWidth( 80 );

    buttonBox->addSpacing( 12 );

    QString cancelt = modal ? ApaintConfig::getText(CANCEL_MSG) : ApaintConfig::getText(104);
    d->cancel = new QPushButton( cancelt, this, "cancel/close" );
    buttonBox->addWidget( d->cancel );
    connect( d->cancel, SIGNAL(clicked()), SIGNAL(closeSig()) );
    d->cancel->setFixedWidth( 80 );

    updateScripts();

    resize( 500, 360 );

    d->familyEdit->installEventFilter( this );
    d->styleEdit->installEventFilter( this );
    d->sizeEdit->installEventFilter( this );
    d->familyList->installEventFilter( this );
    d->styleList->installEventFilter( this );
    d->sizeList->installEventFilter( this );

    d->familyEdit->setFocus();

    transBg = true;
    lastFamily=0;
}

/*!
  \internal
 Destroys the font dialog and frees up its storage.
*/

TextFontSelect::~TextFontSelect()
{
    delete d;
    d = 0;
}

void TextFontSelect::closeEvent(QCloseEvent *e)
{
  emit closeSig();
  QDialog::closeEvent(e);  
}

void TextFontSelect::bgColorClicked(int which)
{
  if (which == 0)
   transBg = true;
  else
   transBg = false;
}

void TextFontSelect::showDlg()
{
  show();
  familyHighlighted(lastFamily);
}

QFont TextFontSelect::getFont( bool *ok, const QFont &initial,
			    QWidget *parent, const char* name)
{
    return getFont( ok, &initial, parent, name );
}


QFont TextFontSelect::getFont( bool *ok, QWidget *parent,const char* name)
{
    return getFont( ok, 0, parent, name );
}

QFont TextFontSelect::getFont( bool *ok, const QFont *def, QWidget *parent, const char* name)
{
    QFont result;
    if ( def )
	    result = *def;

    TextFontSelect *dlg = new TextFontSelect( parent, name, TRUE );

    Q_CHECK_PTR( dlg );
    if ( def )
	    dlg->setFont( *def );
    dlg->setCaption( ApaintConfig::getText(170));

    if ( dlg->exec() == QDialog::Accepted ) {
  	result = dlg->font();
  	if ( ok )
  	    *ok = TRUE;
    } else {
    	if ( ok )
  	    *ok = FALSE;
    }
    delete dlg;
    return result;
}

void TextFontSelect::setSizeLabel()
{
  QRect r = textSize(false, dummy);
  d->sample->setTitle(QString("%1(%2:%3 x %4)").arg(ApaintConfig::getText(166)).arg(ApaintConfig::getText(167)).arg(r.width()).arg(r.height()));
}

/*ret indx of the maxLen string in list*/
int TextFontSelect::getMaxLen(QStringList &strL)
{
  uint maxLen =0;
  int indx=0;
  for (int i=0; i<(int)strL.count(); i++)
  {
    if (maxLen<strL[i].length())
    {
     maxLen = strL[i].length();
     indx = i;
    }
  }
  return indx;
}


QStringList TextFontSelect::text()
{
  QStringList sl = QStringList::split(QRegExp("\n"), d->sampleEdit->text());
  for (int i=0; i<(int)sl.count(); i++)
  {
    if (sl[i].contains('\n'))
     sl[i].truncate(sl[i].length()-1);

    sl[i] = " "+sl[i]+" ";
  }
  return sl;
}

QRect TextFontSelect::textSize(bool useTab, QPtrList<QSize>& sizeTab)
{
 QStringList sl;
 QFontMetrics fm(font()) ;
 QRect bnd, trect;
 int wid, hei;
 
 sl = text(); 
 wid = hei =0;
 for (int i=0; i<(int)sl.count(); i++)
 {
  trect = QRect(QPoint(0, 0), fm.size(0, sl[i])+QSize(2,fm.descent()));
  wid = QMAX(trect.width(), wid);
  hei+=trect.height();
  if (useTab)
  {
   QSize *nsize = new QSize(trect.width(), trect.height());
   sizeTab.append(nsize);
  }
 }
 return QRect(0, 0, wid, hei);
}

QListBox * TextFontSelect::familyListBox() const
{
    return d->familyList;
}

/*!
    \internal
    Returns a pointer to the "font style" list box. This is mainly
    useful if you reimplement updateFontStyles();
*/

QListBox * TextFontSelect::styleListBox() const
{
    return d->styleList;
}

/*!
    \internal
    Returns a pointer to the "scripts combo" list box. This is mainly
    useful if you reimplement updateScripts();
*/

QComboBox * TextFontSelect::scriptCombo() const
{
    return d->scriptCombo;
}

/*!
    \internal
    Returns a pointer to the "font size" list box. This is mainly
    useful if you reimplement updateFontSizes();
*/

QListBox * TextFontSelect::sizeListBox() const
{
    return d->sizeList;
}

/*!
    \internal
    This slot is called if the user changes the font size.
    The size is passed in the \a s argument as a \e string.
*/

void TextFontSelect::sizeChanged( const QString &s )
{
    bool ok = FALSE;
    if ( d->size != s ) {
   (void) s.toInt( &ok );
  	if ( ok ) {
  	    d->size = s;
  	    updateSample();
   }
    }
}


/*!
    \internal
    An event filter to make the Up, Down, PageUp and PageDown keys work
    correctly in the line edits. The source of the event is the object
    \a o and the event is \a e.
*/

bool TextFontSelect::eventFilter( QObject * o , QEvent * e )
{
    if ( e->type() == QEvent::KeyPress ) {
  	QListBox * lb = 0;
  	QLineEdit * le = 0;
    QSpinBox *sb = 0;

  	if ( o == d->familyEdit ) {
  	    lb = d->familyList;
  	    le = d->familyEdit;
  	} else if ( o == d->styleEdit ) {
  	    lb = d->styleList;
  	    le = d->styleEdit;
  	} else if ( o == d->sizeEdit ) {
  	    lb = d->sizeList;
  	    sb = d->sizeEdit;
  	} else {
  	    return FALSE;
  	}

	QKeyEvent * k = (QKeyEvent *)e;
	if ( k->key() == Key_Up ||
	     k->key() == Key_Down ||
	     k->key() == Key_Prior ||
	     k->key() == Key_Next ) {
	    int ci = lb->currentItem();
	    (void)QApplication::sendEvent( lb, k );

	    if ( ci != lb->currentItem() && style().styleHint(QStyle::SH_FontDialog_SelectAssociatedText, this))
      {
        if ( o == d->sizeEdit)
         sb->selectAll();
        else
         le->selectAll();
      }
	    return TRUE;
	}
    } else if ( e->type() == QEvent::FocusIn && 	style().styleHint(QStyle::SH_FontDialog_SelectAssociatedText, this)) {
      	if ( o == d->familyEdit )
      	    d->familyEdit->selectAll();
      	else if ( o == d->styleEdit )
      	    d->styleEdit->selectAll();
      	else if ( o == d->sizeEdit )
      	    d->sizeEdit->selectAll();
          } else if ( e->type() == QEvent::MouseButtonPress ) {
      	if ( o == d->familyList )
      	    d->familyEdit->setFocus();
      	else if ( o == d->styleList )
      	    d->styleEdit->setFocus();
      	else if ( o == d->sizeList )
      	    d->sizeEdit->setFocus();
    }
    return QDialog::eventFilter( o, e );
}


#ifdef SHOW_FONTS_IN_FAMILIES
#include "qpainter.h"

class QListBoxFontText : public QListBoxText
{
    QFont cfont;
public:
    QListBoxFontText( const QString & text );
    ~QListBoxFontText() { }

    int	 height( const QListBox * ) const;
    int	 width( const QListBox * )  const;

protected:
    void  paint( QPainter * );
};

QListBoxFontText::QListBoxFontText( const QString & text )
    : QListBoxText(text), cfont(text)
{
}

int QListBoxFontText::height( const QListBox * ) const
{
    QFontMetrics fm(cfont);
    return QMAX( fm.lineSpacing() + 2, QApplication::globalStrut().height() );
}

int QListBoxFontText::width( const QListBox * )  const
{
    QFontMetrics fm(cfont);
    return QMAX( fm.width( text() ) + 6, QApplication::globalStrut().width() );
}

void QListBoxFontText::paint( QPainter *painter )
{
    painter->save();
    painter->setFont(cfont);
    QListBoxText::paint(painter);
    painter->restore();
}

#endif

/*!
  \internal
    Updates the contents of the "font family" list box. This
  function can be reimplemented if you have special requirements.
*/

void TextFontSelect::updateFamilies()
{
    d->familyNames = d->fdb.families();
    QStringList newList;
    QString s;
    QStringList::Iterator it = d->familyNames.begin();
#ifdef SHOW_FONTS_IN_FAMILIES
    int idx = 0;
#endif
    for( ; it != d->familyNames.end() ; it++ )
    {
      	s = *it;

    #if 0
    	if ( d->fdb.isSmoothlyScalable( *it ) )
    	    newList.append( s + "(TT)" );
    	else if ( d->fdb.isBitmapScalable( *it ) )
    	    newList.append( s + "(BT)" );
    	else
    #endif
    #ifdef SHOW_FONTS_IN_FAMILIES
    	    d->familyList->insertItem(new QListBoxFontText(s), idx++);
    #else
    	newList.append( s );
    #endif
    }
#ifndef SHOW_FONTS_IN_FAMILIES
    d->familyList->insertStringList( newList );
#endif
}

/*!
    \internal
    Updates the contents of the "font script" combo box. This
  function can be reimplemented if you have special requirements.
*/

void TextFontSelect::updateScripts()
{
    d->scriptCombo->clear();
    d->scriptNames.clear();
    d->scriptScripts.clear();

    QString scriptname;
    for (int i = 0; i < QFont::NScripts; i++)
    {
    	scriptname = QFontDatabase::scriptName((QFont::Script) i);
    	if (! scriptname.isEmpty()) {
    	    d->scriptNames += scriptname;
    	    d->scriptScripts += i;
    	}
    }

    if ( d->scriptNames.isEmpty() ) {
      #ifndef QT_NO_DEBUG
      	qWarning( "TextFontSelect::updateScripts: Internal error, "
      		  "no scripts for family \"%s\"",
      		  (const char *) d->family );
      #endif
      	return;
    }

    QStringList::Iterator it = d->scriptNames.begin();
    for ( ; it != d->scriptNames.end() ; ++it )
    	d->scriptCombo->insertItem( *it );
}

/*!
    \internal
    Updates the contents of the "font style" list box. This
  function can be reimplemented if you have special requirements.
*/

void TextFontSelect::updateStyles()
{
    d->styleList->clear();

    QStringList styles = d->fdb.styles( d->family );

    if ( styles.isEmpty() ) {
     #ifndef QT_NO_DEBUG
     	qWarning( "TextFontSelect::updateStyles: Internal error, "
     		  "no styles for family \"%s\" with script \"%s\"",
     		  (const char *) d->family, (const char *) d->script );
     #endif
     	return;
    }
    d->styleList->insertStringList( styles );
}

/*!
    \internal
    Updates the contents of the "font size" list box. This
  function can be reimplemented if you have special requirements.
*/

void TextFontSelect::updateSizes()
{
    //    usingStandardSizes = d->fdb.isScalable( d->family );

    d->sizeList->clear();
    QValueList<int> sizes = d->fdb.pointSizes( d->family,d->style );

    if ( sizes.isEmpty() ) {
     #ifndef QT_NO_DEBUG
     	qWarning( "TextFontSelect::updateSizes: Internal error, "
     		  "no pointsizes for family \"%s\" with script \"%s\"\n"
     		  "and style \"%s\"",
     		  (const char *) d->family, (const char *) d->script,
     		  (const char *) d->style );
     #endif
     	return;
    }
    int i;
    QString tmp;
    for( i = 0 ; (uint)i < sizes.count() ; i++ ) {
    	tmp.sprintf( "%i", sizes[i] );
    	d->sizeList->insertItem( tmp );
    }
    int maxVal;
    d->sizeEdit->setMinValue(sizes[0]);

    if (d->fdb.isScalable( d->family ))
      maxVal = 400;
    else
      maxVal = sizes[sizes.count()-1];

    d->sizeEdit->setMaxValue(maxVal);
}

/*!
    \internal
*/

void TextFontSelect::familyHighlighted( const QString &s )
{
    d->familyEdit->setText(d->familyList->text(d->familyList->currentItem()));
    if ( style().styleHint(QStyle::SH_FontDialog_SelectAssociatedText, this) && d->familyEdit->hasFocus() )
      	d->familyEdit->selectAll();

    d->family = s;
    if ( d->scriptCombo->count() != 0 )
    	scriptHighlighted( d->scriptCombo->currentItem() );
}

void TextFontSelect::familyHighlighted( int i )
{
    QString s = d->familyNames[i];
    familyHighlighted( s );
    lastFamily = i;
}

/*!
    \internal
*/

void TextFontSelect::scriptHighlighted( int index )
{
    scriptHighlighted( d->scriptNames[index] );
}

void TextFontSelect::scriptHighlighted( const QString &s )
{
    d->script = s;
    QString currentStyle = d->styleList->currentText();

    updateStyles();
    if ( d->styleList->count() != 0 )
    {
    	for ( int i = 0 ; i < (int)d->styleList->count() ; i++ )
     {
    	    if ( currentStyle == d->styleList->text(i) )
         {
         		d->styleList->setCurrentItem( i ); // Will call styleHighlighted
         		break;
    	    }
    	}
        if ( d->styleList->currentItem() == -1 )
	        d->styleList->setCurrentItem( 0 ); // Will call styleHighlighted
    }
}

/*!
    \internal
*/

void TextFontSelect::styleHighlighted( const QString &s )
{
    d->styleEdit->setText( s );
    if ( style().styleHint(QStyle::SH_FontDialog_SelectAssociatedText, this) && d->styleEdit->hasFocus() )
    	d->styleEdit->selectAll();

    d->style = s;

    if ( d->usingStandardSizes && d->fdb.isScalable( d->family ) )
    {
    	updateSample();
    	return;
     }
    int pSize = d->size.toInt();
    updateSizes();
    QString tmp;
    if ( d->sizeList->count() != 0 )
    {
    	int i;
    	for ( i = 0 ; i < (int)d->sizeList->count() - 1 ; i++ )
     {
    	    tmp = d->sizeList->text(i);
    	    if ( tmp.toInt() >= pSize )
    		break;
    	}
    	d->sizeList->setCurrentItem( i );
    }
}


/*!
    \internal
*/

void TextFontSelect::sizeHighlighted( const QString &s )
{
    d->sizeEdit->setValue( s.toInt() );
    if ( style().styleHint(QStyle::SH_FontDialog_SelectAssociatedText, this) && d->sizeEdit->hasFocus() )
	    d->sizeEdit->selectAll();

    d->size = s;
    updateSample();
}

enum match_t { MATCH_NONE=0, MATCH_LAST_RESORT=1, MATCH_DEFAULT=2, MATCH_FAMILY=3 };

/*!
  \internal
  Sets the font highlighted in the QFontDialog to font \a f.

  \sa font()
*/

void TextFontSelect::parseFontName(const QString &name, QString &foundry, QString &family)
{
    if ( name.contains('-') )
    {
    	int i = name.find('-');
    	foundry = name.left( i ).lower();
    	family = name.right( name.length() - i - 1 ).lower();
    }
    else if ( name.contains('[') && name.contains(']'))
    {
      	int i = name.find('[');
      	int li = name.findRev(']');

      	if (i < li)
       {
      	    foundry = name.mid(i + 1, li - i - 1).lower();
      	    if (name[i - 1] == ' ')
      	    	i--;
      	    family = name.left(i).lower();
      	}
    }
    else
    {
      	foundry = QString::null;
      	family = name.lower();
    }
}


void TextFontSelect::setFont( const QFont &f )
{
    QListBoxText *dummy = new QListBoxText( d->familyList );
    d->familyList->blockSignals( TRUE );
    d->familyList->setCurrentItem( dummy );
    d->familyList->blockSignals( FALSE );
    QString foundryName1, familyName1, foundryName2, familyName2;
    int bestFamilyMatch = -1;
    match_t bestFamilyType = MATCH_NONE;

    parseFontName(f.family(), foundryName1, familyName1);

    QStringList::Iterator it;
    it = d->familyNames.begin();
    int i = 0;
    for( ; it != d->familyNames.end() ; ++it, ++i )
    {
       parseFontName(*it, foundryName2, familyName2);

      	//try to match..
      	if (foundryName1 == foundryName2 && familyName1 == familyName2)
        {
      	    d->familyList->setCurrentItem(i);
      	    delete dummy;
      	    dummy = 0;
      	    i = -1;
      	    break;
      	}

      	//and try some fall backs
      	match_t type = MATCH_NONE;
      	switch(bestFamilyType) {
      	case MATCH_NONE:
      	    if(familyName2 == f.lastResortFamily()) {
      		type = MATCH_LAST_RESORT;
      		break;
      	    }
      	case MATCH_LAST_RESORT:
      	    if(familyName2 == f.defaultFamily()) {
      		type = MATCH_DEFAULT;
      		break;
      	    }
      	case MATCH_DEFAULT:
      	    if(familyName2 == familyName1) {
      		type = MATCH_FAMILY;
      		break;
      	    }
      	case MATCH_FAMILY: //already got the best
      	    break;
      	}
      	if(type != MATCH_NONE)
      	{
      	    bestFamilyType = type;
      	    bestFamilyMatch = i;
      	}
    }

    if (i != -1 && bestFamilyType != MATCH_NONE) {
    	d->familyList->setCurrentItem(bestFamilyMatch);
    } else if ( dummy ) {
    	d->familyList->setCurrentItem( 0 );
    }
    delete dummy;
    dummy = 0;

    QString styleString = d->fdb.styleString( f );
    if ( !styleString.isEmpty() && d->styleList->count() != 0 ) {
    	for ( i = 0 ; i < (int)d->styleList->count() ; i++ )
      {
    	    if ( styleString == d->styleList->text(i) )
         {
         		d->styleList->setCurrentItem( i );
    		    break;
    	    }
    	}
    }

    if ( d->sizeList->count() != 0 )
    {
     	int pSize = QFontInfo(f).pointSize();
     	for ( i = 0 ; i < (int)d->sizeList->count() - 1 ; i++ )
      {
     	    QString tmp = d->sizeList->text(i);
     	    if ( tmp.toInt() >= pSize )
     		break;
     	}
     	d->sizeList->setCurrentItem( i );
     	d->sizeEdit->setValue( pSize  );
    }

    d->strikeout->setChecked( f.strikeOut() );
    d->underline->setChecked( f.underline() );

    updateSample();
}

/*!
  \internal
  Returns the font which the user has chosen.

  \sa setFont()
*/

// nothing to do for now...
bool TextFontSelect::isRawNameCorrect(QString rname, QString &newFont)
{
  QStringList sl = QStringList::split(":", rname);
  if (sl.count()>1) return false;
  sl = QStringList::split("-", rname);
  if (sl.count()<2) return false;

  return false;
}

QFont TextFontSelect::font()
{
    int pSize = d->size.toInt();
    if ( pSize == 0 )
    	pSize = 12;     
    QFont f;
    f.setRawMode(true);
    f = d->fdb.font( d->family, d->style, pSize );
    
    f.setStrikeOut( d->strikeout->isChecked() );
    f.setUnderline( d->underline->isChecked() );
    
    return f;
}

void TextFontSelect::updateSample()
{
    d->sampleEdit->setFont(font());    
    d->sampleEdit->setMinimumSize(QSize(200, 100));
    emit fontHighlighted(font());
    setSizeLabel();
}

void TextFontSelect::emitSelectedFont()
{
    emit fontSelected(font());
}

/*!
  \fn void TextFontSelect::fontSelected( const QFont & font )
  \internal

  This signal is emitted when the user has chosen a font and clicked OK.
  The font that was selected is passed in \a font.
*/

/*!
  \fn void TextFontSelect::fontHighlighted( const QFont & font )
  \internal

  This signal is emitted when the user changed a setting in the dialog.
  The font that is highlighted is passed in \a font.
*/

void TextFontSelect::leftPressed()
{
 if (d->alignLeft->isOn()) d->alignLeft->toggle();
 if (d->alignRight->isOn()) d->alignRight->toggle();
 if (d->alignCenter->isOn()) d->alignCenter->toggle();
 if (d->alignFill->isOn()) d->alignFill->toggle();
 align = Qt::AlignLeft;
 emit insertBtnClicked();
}

void TextFontSelect::rightPressed()
{
 if (d->alignLeft->isOn()) d->alignLeft->toggle();
 if (d->alignRight->isOn()) d->alignRight->toggle();
 if (d->alignCenter->isOn()) d->alignCenter->toggle();
 if (d->alignFill->isOn()) d->alignFill->toggle();
 align = Qt::AlignRight;
 emit insertBtnClicked();
}

void TextFontSelect::centerPressed()
{
 if (d->alignLeft->isOn()) d->alignLeft->toggle();
 if (d->alignRight->isOn()) d->alignRight->toggle();
 if (d->alignCenter->isOn()) d->alignCenter->toggle();
 if (d->alignFill->isOn()) d->alignFill->toggle();
 align = Qt::AlignCenter;
 emit insertBtnClicked();
}

void TextFontSelect::fillPressed()
{
 if (d->alignLeft->isOn()) d->alignLeft->toggle();
 if (d->alignRight->isOn()) d->alignRight->toggle();
 if (d->alignCenter->isOn()) d->alignCenter->toggle();
 if (d->alignFill->isOn()) d->alignFill->toggle();
 align = Qt::AlignJustify;
 emit insertBtnClicked();
}

int TextFontSelect::aligning() const
{
 return align;
}

