/***************************************************************************
                          dialogIconView.cpp  -  description
                             -------------------
    begin                : Tue Jan 6 2004
    copyright            : (C) 2003-2014 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <limits.h>
#include <unistd.h>
#include <sys/stat.h>
#include <time.h>
#include <qcursor.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qspinbox.h>
#include <qlineedit.h>
#include <qtooltip.h>
#include <qmessagebox.h>
#include <qprocess.h>
#include "dialogIconView.h"
#include "widgetIconView.h"
#include "iconViewData.h"
#include "apaintio.h"
#include "qpushbutton.h"
#include "apaintconfig.h"
#include "apaintio.h"
#include "apaintUtil.h"

extern char *getCommonPNG(const char *pixName);

static int disableOK = FALSE; /* ugly hack to disable OK on enter in text box */

void IconViewDlg::setSubDirIcon()
{
 btnSubDir->setIconSet(QPixmap(iList->getIncludeSubDir() ? getCommonPNG("treeLeafs") : getCommonPNG("treeNoLeafs")));
}

IconViewDlg::IconViewDlg(QWidget *parent, const char *name, ApaintDoc *docp):IconViewDialogUI(parent, name)
{
 Q_CHECK_PTR(docp);
 doc = docp;
 curDir = "";
 captionStr = "";

 setGeometry(100, 100, ApaintConfig::initialBrowseSizeXpix(), ApaintConfig::initialBrowseSizeYpix());

 cbDirInfo->setPixmap(QPixmap(getCommonPNG("folder-info-32")));
 QToolTip::add(cbDirInfo,  ApaintConfig::getText(833));
 cbDirectory->setText(ApaintConfig::getText(FILE_SELECTION_DIRECTORY_MSG));
 btnClose->setPixmap(QPixmap(getCommonPNG("close")));
 QToolTip::add(btnClose,  ApaintConfig::getText(86));
 gbFilter->setTitle(ApaintConfig::getText(661));
 lbSort->setText(ApaintConfig::getText(658));
 cbRaport->setText(ApaintConfig::getText(54));
 cbName->setText(ApaintConfig::getText(81));
 cbSuffix->setText(ApaintConfig::getText(806));
 cbSize->setText(ApaintConfig::getText(167));
 cbFileSize->setText(ApaintConfig::getText(659));
 cbFilter->setText(ApaintConfig::getText(661));
 btnOk->setText(ApaintConfig::getText(OK_MSG));
 lbFileName->setText(ApaintConfig::getText(81));
 cbDate->setPixmap(QPixmap(getCommonPNG("date")));
 gbFilter->setCaption(ApaintConfig::getText(661));

 QToolTip::add(cbDate, ApaintConfig::getText(660));
 QToolTip::add(btnDirUp, ApaintConfig::getText(750));
 QToolTip::add(btnReload, ApaintConfig::getText(751));
 QToolTip::add(btnHome, ApaintConfig::getText(754));
 QToolTip::add(btnDelete, ApaintConfig::getText(14));
 QToolTip::add(btnRename, ApaintConfig::getText(712));
 QToolTip::add(btnSubDir, ApaintConfig::getText(749));
 QToolTip::add(btnOpenWithPaint, ApaintConfig::getText(827));
 btnDirUp->setIconSet(QPixmap(getCommonPNG("go-up")));
 btnReload->setIconSet(QPixmap(getCommonPNG("reload")));
 btnHome->setIconSet(QPixmap(getCommonPNG("go-home")));
 btnDelete->setIconSet(QPixmap(getCommonPNG("edit-delete")));
 btnRename->setIconSet(QPixmap(getCommonPNG("rename")));
 btnOpenWithPaint->setIconSet(QPixmap(getCommonPNG("ArahPaint4-22")));

 connect(btnOk, SIGNAL(clicked()), this, SLOT(okBtnClicked()));
 connect(btnClose, SIGNAL(clicked()), this, SLOT(closeBtnClicked()));

 connect(btnDirUp, SIGNAL(clicked()), this, SLOT(dirUpBtnClicked()));
 connect(btnReload, SIGNAL(clicked()), this, SLOT(reloadBtnClicked()));
 connect(btnHome, SIGNAL(clicked()), this, SLOT(homeBtnClicked()));
 connect(btnRename, SIGNAL(clicked()), this, SLOT(renameBtnClicked()));
 connect(btnDelete, SIGNAL(clicked()), this, SLOT(deleteBtnClicked()));
 connect(btnSubDir, SIGNAL(clicked()), this, SLOT(subDirClicked()));
 connect(btnOpenWithPaint, SIGNAL(clicked()), this, SLOT(openWithPaintBtnClicked()));
 
 iList = new IconList(4, 100, 100);
 listView->setIconList(iList);
 connect(listView, SIGNAL(iconSelected()), this, SLOT(iconSelectedSlot()));
 connect(cbRaport, SIGNAL(toggled(bool)), this, SLOT(showRepeat(bool)));
 connect(cbName, SIGNAL(toggled(bool)), this, SLOT(showIconName(bool)));
 connect(cbSuffix, SIGNAL(toggled(bool)), this, SLOT(showIconSuffix(bool)));
 connect(cbSize, SIGNAL(toggled(bool)), this, SLOT(showIconSize(bool)));
 connect(cbFileSize, SIGNAL(toggled(bool)), this, SLOT(showFileSize(bool)));
 connect(cbFilter, SIGNAL(toggled(bool)), this, SLOT(showFilter(bool)));
 connect(cbDate, SIGNAL(toggled(bool)), this, SLOT(showDTime(bool)));
 connect(cbDirInfo, SIGNAL(toggled(bool)), this, SLOT(showDirInfo(bool)));
 connect(cbDirectory, SIGNAL(toggled(bool)), this, SLOT(showDirectory(bool)));

 cbSort->insertItem(ApaintConfig::getText(81));
 cbSort->insertItem(ApaintConfig::getText(721));
 cbSort->insertItem(ApaintConfig::getText(664));
 cbSort->insertItem(ApaintConfig::getText(662));
 cbSort->insertItem(ApaintConfig::getText(663));
 cbSort->insertItem(ApaintConfig::getText(659));
 cbSort->insertItem(ApaintConfig::getText(660));
 cbSort->insertItem(ApaintConfig::getText(17));
 connect(cbSort, SIGNAL(activated(int)), this, SLOT(sortIcons(int)));
 
 cbFilter1->insertItem(ApaintConfig::getText(NONE_MSG));
 cbFilter1->insertItem(ApaintConfig::getText(SIZE_X_MSG));
 cbFilter1->insertItem(ApaintConfig::getText(SIZE_Y_MSG));
 cbFilter1->insertItem(ApaintConfig::getText(SIZE_XY_MSG));
 cbFilter1->insertItem(ApaintConfig::getText(DAYS_OLD_MSG));
 cbFilter1->insertItem(ApaintConfig::getText(COLORS_MSG));
 connect(cbFilter1, SIGNAL(activated(int)), this, SLOT(filter1Changed(int)));
 connect(cbComp1, SIGNAL(activated(int)), this, SLOT(filter1Changed(int)));
 connect(sbFilter1, SIGNAL(valueChanged(int)), this, SLOT(filter1Changed(int)));

 cbFilter2->insertItem(ApaintConfig::getText(NONE_MSG));
 cbFilter2->insertItem(ApaintConfig::getText(SIZE_X_MSG));
 cbFilter2->insertItem(ApaintConfig::getText(SIZE_Y_MSG));
 cbFilter2->insertItem(ApaintConfig::getText(SIZE_XY_MSG));
 cbFilter2->insertItem(ApaintConfig::getText(DAYS_OLD_MSG));
 cbFilter2->insertItem(ApaintConfig::getText(COLORS_MSG));
 connect(cbFilter2, SIGNAL(activated(int)), this, SLOT(filter2Changed(int)));
 connect(cbComp2, SIGNAL(activated(int)), this, SLOT(filter2Changed(int)));
 connect(sbFilter2, SIGNAL(valueChanged(int)), this, SLOT(filter2Changed(int)));

 cbFilter3->insertItem(ApaintConfig::getText(NONE_MSG));
 cbFilter3->insertItem(ApaintConfig::getText(SIZE_X_MSG));
 cbFilter3->insertItem(ApaintConfig::getText(SIZE_Y_MSG));
 cbFilter3->insertItem(ApaintConfig::getText(SIZE_XY_MSG));
 cbFilter3->insertItem(ApaintConfig::getText(DAYS_OLD_MSG));
 cbFilter3->insertItem(ApaintConfig::getText(COLORS_MSG));
 connect(cbFilter3, SIGNAL(activated(int)), this, SLOT(filter3Changed(int)));
 connect(cbComp3, SIGNAL(activated(int)), this, SLOT(filter3Changed(int)));
 connect(sbFilter3, SIGNAL(valueChanged(int)), this, SLOT(filter3Changed(int)));

 cbFilter4->insertItem(ApaintConfig::getText(NONE_MSG));
 cbFilter4->insertItem(ApaintConfig::getText(SIZE_X_MSG));
 cbFilter4->insertItem(ApaintConfig::getText(SIZE_Y_MSG));
 cbFilter4->insertItem(ApaintConfig::getText(SIZE_XY_MSG));
 cbFilter4->insertItem(ApaintConfig::getText(DAYS_OLD_MSG));
 cbFilter4->insertItem(ApaintConfig::getText(COLORS_MSG));
 connect(cbFilter4, SIGNAL(activated(int)), this, SLOT(filter4Changed(int)));
 connect(cbComp4, SIGNAL(activated(int)), this, SLOT(filter4Changed(int)));
 connect(sbFilter4, SIGNAL(valueChanged(int)), this, SLOT(filter4Changed(int)));
 connect(edFilter, SIGNAL(textChanged(const QString&)), this, SLOT(filterTextChanged(const QString&)));

 connect(listView, SIGNAL(deleteImageSig()), this, SLOT(deleteImage()));
 connect(listView, SIGNAL(renameImageSig()), this, SLOT(renameImage()));
 // signal emited when text in comboBox is activated 
 connect(comboBoxPath, SIGNAL(activated(int)), this, SLOT(comboPathChanged(int)));

 showFilter(ApaintConfig::startBrowseWithFilter()); 
 cbFilter->setChecked(ApaintConfig::startBrowseWithFilter());
}

IconViewDlg::~IconViewDlg()
{
 
}

void IconViewDlg::setupGUI()
{
 iList->setIsBrushList(false);

 cbName->setChecked(ApaintConfig::getBrowseImageDisplay() & IMAGE_ICON_FILENAME);
 cbSuffix->setEnabled(ApaintConfig::getBrowseImageDisplay() & IMAGE_ICON_FILENAME);
 cbSuffix->setChecked(!(ApaintConfig::getBrowseImageDisplay() & IMAGE_ICON_NO_SUFFIX));
 cbSize->setChecked(ApaintConfig::getBrowseImageDisplay() & IMAGE_ICON_IMAGESIZE);
 cbFileSize->setChecked(ApaintConfig::getBrowseImageDisplay() & IMAGE_ICON_FILESIZE);
 cbRaport->setChecked(ApaintConfig::getBrowseImageDisplay() & IMAGE_ICON_REPEAT);
 cbDate->setChecked(ApaintConfig::getBrowseImageDisplay() & IMAGE_ICON_FILEDATE);
 cbDirInfo->setChecked(ApaintConfig::getBrowseImageDisplay() & IMAGE_ICON_DIR_INFO);
 cbDirectory->setChecked(ApaintConfig::getBrowseImageDisplay() & IMAGE_ICON_SUB_DIR);
}

void IconViewDlg::showDlg(QString startDir, LoadType ltyp)
{
 static int first = TRUE;
 
 ltype = ltyp;
 if (ltype == MainImage)
  captionStr = ApaintConfig::getText(LOAD_PICTURE_MSG);
 else if (ltype == LayerImage)
  captionStr = ApaintConfig::getText(LOAD_LAYER_MSG);
 else if ((ltype == ColorPImage) || (ltype == ColormapImage) || (ltype == ColormapDitherImage))
  captionStr = ApaintConfig::getText(61);
 else if (ltype == BrushImage)
  captionStr = ApaintConfig::getText(722);

 curDir = startDir;
 if (curDir.right(1)!="/")
   curDir.append("/");
 if (first)
  {
   homeDir = curDir;
   first = FALSE;
  }
 dir.setPath(curDir);  
 setupData(".");
 setupGUI();
 setSubDirIcon();
 show();
}

void IconViewDlg::updateDialog()
{
 QString title = captionStr;
 
 if ((iList->getNumImages() + iList->getNumDirs()) == iList->getNumIcons())
  title += " " + ApaintConfig::getText(FOUND_MSG) + QString(" %1").arg(iList->getNumImages());
 else
  title += " " + ApaintConfig::getText(FOUND_MSG) + QString(" %1(%2)").arg(iList->getNumIcons() - iList->getNumDirs()).arg(iList->getNumImages());

 if ((ApaintConfig::getBrowseImageDisplay() & IMAGE_ICON_FILESIZE) && iList->getNumIcons())
  title += " ["+iList->getTotalFileSize()+"]";

 setCaption(title);
 comboBoxPath->setCurrentText(curDir);
 btnSubDir->setEnabled(iList->getNumDirs() || iList->getIncludeSubDir());
 cbDirInfo->setEnabled(iList->getNumDirs());
 if (iList->getIncludeSubDir())
  {
   cbDirectory->show();
   cbDirInfo->hide();
  }
 else
  {
   cbDirectory->hide();
   cbDirInfo->show();
  }
}

void IconViewDlg::setupData(QString newDir)
{
  QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
  if (newDir == "..")
   {
     dir.cdUp();
     curDir = dir.path();
     if (curDir.right(1)!="/")
       curDir.append("/");
    /* disable subdirectory browse if we go one directory up */
    iList->setIncludeSubDir(false);
    setSubDirIcon();
    setImageBrowseIncludeSubDir(iList->getIncludeSubDir());
   }
  else if (newDir == ".")
   {
     /*do nothing*/
   }
  else
   {
    curDir.append(newDir);
    if (curDir.right(1)!="/")
     curDir.append("/");
    dir.setPath(curDir);
   }
  iList->setToNewDir(curDir, listView->fontMetrics().height(), true);
  listView->setupContents(true);
  updateDialog();
  QApplication::restoreOverrideCursor();
  emit setImagePathSig(curDir);
}

void IconViewDlg::setupDataWithNewDir(QString newDir)
{
 /* do not change the directory to paths which do not exist or we do not have write access */
 if ((access((char *)newDir.local8Bit().data(), R_OK)!=0))
  {
   QApplication::beep();
//   edPath->setText(curDir);
   comboBoxPath->setCurrentText(curDir);
   return;
  }
  QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );

  curDir = newDir;
  if (curDir.right(1)!="/")
    curDir.append("/");
  dir.setPath(curDir);

  iList->setToNewDir(curDir, listView->fontMetrics().height(), true);
  listView->setupContents(true);
  updateDialog();
  QApplication::restoreOverrideCursor();
  emit setImagePathSig(curDir);
}

void IconViewDlg::comboPathChanged(int index)
{
 setupDataWithNewDir(comboBoxPath->currentText());
 disableOK = TRUE;
}

void IconViewDlg::iconSelectedSlot()
{
  IconData *ic  =  iList->getIconData(iList->selectedIcon());
  if (!ic) return;
  if (ic->isImage())
   {
    QString file;
    imageInfo *iI = ic->getImageInfo();
    if (iI->subDirID)
     file = QString("%1%2/%3").arg(curDir).arg(QString::fromLocal8Bit(getImageSubDir(iI->subDirID))).arg(ic->getName());
    else
     file = QString("%1%2").arg(curDir).arg(ic->getName());

    if (ltype == MainImage)
     emit loadDocSig(file);
    else if (ltype == LayerImage)
     emit loadLayerSig(file);
    else if (ltype == ColorPImage)
     emit loadColorPImage(file);
    else if (ltype == ColormapImage)
     emit loadColormapImage(file);
    else if (ltype == ColormapDitherImage)
     emit loadColormapDitherImage(file);
    else if (ltype == BrushImage)
     emit loadBrushSig(file);
   }
  else if (ic->isDir())
   {
    char filen[500];

    /* don't enter directory if we can't read it */
    sprintf(filen, "%s/%s", (strlen(getImageBrowsePath())==1) ? "" : getImageBrowsePath(), ic->getName().local8Bit().data());
    if (access(filen, R_OK | X_OK)!=0)
     QApplication::beep();
    else
     setupData(ic->getName());
   }
}

void IconViewDlg::okBtnClicked()
{
 if (disableOK)
  disableOK = FALSE;
 else
  iconSelectedSlot();
 close();
}

void IconViewDlg::closeBtnClicked()
{
 close();
}

void IconViewDlg::dirUpBtnClicked()
{
 setupData("..");
}

void IconViewDlg::reloadBtnClicked()
{
 setupData(".");
}

void IconViewDlg::homeBtnClicked()
{
 curDir = homeDir;
 setupData(".");
}

void IconViewDlg::renameBtnClicked()
{
 renameImage();
}

void IconViewDlg::deleteBtnClicked()
{
 deleteImage();
}

void IconViewDlg::openWithPaintBtnClicked()
{
 IconData *ic  =  iList->getIconData(iList->selectedIcon());

 if (!ic)
  return;
 if (ic->isImage())
  {
   QString file;
   QProcess *proc;

   imageInfo *iI = ic->getImageInfo();
   if (iI->subDirID)
    file = QString("%1%2/%3").arg(curDir).arg(QString::fromLocal8Bit(getImageSubDir(iI->subDirID))).arg(ic->getName());
   else
    file = QString("%1%2").arg(curDir).arg(ic->getName());

   proc = new QProcess( this );
   proc->addArgument("./arahpaint4");
   proc->addArgument(file);
   proc->start();
  }
 else
  QApplication::beep();
}

void IconViewDlg::subDirClicked()
{
 iList->setIncludeSubDir(!iList->getIncludeSubDir());
 setImageBrowseIncludeSubDir(iList->getIncludeSubDir());
 setSubDirIcon();
 setupDataWithNewDir(comboBoxPath->currentText());
}

void IconViewDlg::showRepeat(bool enable)
{
 ApaintConfig::setBrowseImageDisplayBit(IMAGE_ICON_REPEAT, enable);
 listView->setupContents(true);
}

void IconViewDlg::showIconName(bool enable)
{
 ApaintConfig::setBrowseImageDisplayBit(IMAGE_ICON_FILENAME, enable);
 cbSuffix->setEnabled(enable);
 listView->setupContents(true);
}

void IconViewDlg::showIconSuffix(bool enable)
{
 ApaintConfig::setBrowseImageDisplayBit(IMAGE_ICON_NO_SUFFIX, !enable);
 listView->setupContents(true);
}

void IconViewDlg::showIconSize(bool enable)
{
 ApaintConfig::setBrowseImageDisplayBit(IMAGE_ICON_IMAGESIZE, enable);
 listView->setupContents(true); 
}

void IconViewDlg::showFileSize(bool enable)
{
 ApaintConfig::setBrowseImageDisplayBit(IMAGE_ICON_FILESIZE, enable);
 listView->setupContents(true);
 updateDialog(); /* needed to update window title */
}

void IconViewDlg::showDTime(bool enable)
{
 ApaintConfig::setBrowseImageDisplayBit(IMAGE_ICON_FILEDATE, enable);
 listView->setupContents(true);
}

void IconViewDlg::showDirInfo(bool enable)
{
 ApaintConfig::setBrowseImageDisplayBit(IMAGE_ICON_DIR_INFO, enable);
 if (enable)
   setupData(".");
 else
  listView->setupContents(true);
}

void IconViewDlg::showDirectory(bool enable)
{
 ApaintConfig::setBrowseImageDisplayBit(IMAGE_ICON_SUB_DIR, enable);
 listView->setupContents(true);
}

void IconViewDlg::sortIcons(int typ)
{
 switch(typ)
  {
   case 0:
    iList->sortIcons(NameSort);    
    break;
   case 1:
    iList->sortIcons(FileTypeSort);
    break;
   case 2:
    iList->sortIcons(SizeSort);
    break;
   case 3:
    iList->sortIcons(SizeXSort);
    break;
   case 4:
    iList->sortIcons(SizeYSort);
    break;
   case 5:
    iList->sortIcons(FileSizeSort);
    break;
   case 6:
    iList->sortIcons(DTimeSort);
    break;
   case 7:
    iList->sortIcons(ColorNumSort);
    break;
  }
 listView->setupContents(true);
}

void IconViewDlg::showFilter(bool enable)
{
 swFilter = enable;
 iList->useFilter(enable);
 if (enable)
  {
   gbFilter->show();
   edFilter->setFocus();
  }
 else
  gbFilter->hide();
 listView->setupContents(true);
 ApaintConfig::setBrowseWithFilter(enable);
}

/*dont use dummy!!!*/
void IconViewDlg::filter1Changed(int dummy)
{
  if (!swFilter) return;
  iList->setFilter(0, (IconList::FilterType) cbFilter1->currentItem(), (IconList::FilterOp) cbComp1->currentItem(),
                    sbFilter1->value());
  listView->setupContents(true);
  updateDialog();
}

void IconViewDlg::filter2Changed(int dummy)
{
  if (!swFilter) return;
  iList->setFilter(1, (IconList::FilterType) cbFilter2->currentItem(), (IconList::FilterOp)cbComp2->currentItem(),
                  sbFilter2->value());
  listView->setupContents(true);
  updateDialog();
}

void IconViewDlg::filter3Changed(int dummy)
{
  iList->setFilter(2, (IconList::FilterType) cbFilter3->currentItem(), (IconList::FilterOp)cbComp3->currentItem(),
                  sbFilter3->value());
  listView->setupContents(true);
  updateDialog();
}

void IconViewDlg::filter4Changed(int dummy)
{
  if (!swFilter) return;
  iList->setFilter(3, (IconList::FilterType) cbFilter4->currentItem(), (IconList::FilterOp)cbComp4->currentItem(),
                  sbFilter4->value());
  listView->setupContents(true);
  updateDialog();
}

void IconViewDlg::filterTextChanged(const QString& s)
{
  if (!swFilter) return;
  iList->setFilterText(s);
  listView->setupContents(true);
  updateDialog();
}

void IconViewDlg::setFont(QFont& nfont)
{
  listView->setFont(nfont);
}

void IconViewDlg::deleteImage()
{
  if (!iList->iconNum())
   return;
  /* delete directory, without warning... */
  if (iList->getIconData(iList->selectedIcon())->isDir())
   {
    char filen[500];
    int dirs, files;

    sprintf(filen, "%s/%s", getImageBrowsePath(), iList->getIconData(iList->selectedIcon())->getName().local8Bit().data());
    if ((access(filen, W_OK)!=0))
     { QMessageBox::information(this, ApaintConfig::getText(8), ApaintConfig::getText(835)); return; }

    countDirFiles(filen, &dirs, &files);
    if (dirs || files)
     { QMessageBox::information(this, ApaintConfig::getText(8), ApaintConfig::getText(834)); return; }
    else
     if (rmdir(filen)!=0)
      { QMessageBox::information(this, ApaintConfig::getText(8), ApaintConfig::getText(835)); return; }
    setupData(".");
    return;
   }

  QString askUser = QString("%1: %2").arg(ApaintConfig::getText(506)).arg(iList->getIconData(iList->selectedIcon())->getName());
  int ret = QMessageBox::warning(this, ApaintConfig::getText(9), askUser,
                                       ApaintConfig::getText(1), ApaintConfig::getText(CANCEL_MSG));
  if (ret == 0)
   {
    iList->removeIconAndFile(iList->selectedIcon());
    listView->setupContents(true);
    updateDialog();
   } 
}

void IconViewDlg::renameImage()
{
  if (iList->getIconData(iList->selectedIcon())->isDir()) return;

  QString fname = iList->getIconData(iList->selectedIcon())->getName();
  QString tellUser = QString("%1: %2").arg(ApaintConfig::getText(712)).arg(fname);
  if (EnterTextDialog::getText(this, ApaintConfig::getText(712), "", tellUser, fname))
  {
    iList->renameIconAndFile(iList->selectedIcon(), fname);
    listView->refresh();
  }
}

void IconViewDlg::setDirectoryHistoryList(const QStringList& directoryList)
{
 comboBoxPath->clear();
 comboBoxPath->insertStringList(directoryList, 0);
}
