/***************************************************************************
                          scalerotdialog.cpp  -  description
                             -------------------
    begin                : Fri May 30 2003
    copyright            : (C) 2003-2014 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dialogScaleRotate.h"
#include <qcombobox.h>
#include <qlabel.h>
#include <qslider.h>
#include <qspinbox.h>
#include <qtabwidget.h>
#include <qgroupbox.h>
#include <qvalidator.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include "dialogRepeatDimen.h"
#include "apaintconfig.h"

/************************SCALE WIDGET************************/

ScaleWidget::ScaleWidget(QWidget *parent, const char *name, RepeatDimDlg *repeatDlg): ScaleImgWidgetUI(parent, name, 0)
{
 useDensity = false;
 lbOrigxy->setText(ApaintConfig::getText(620)+":");
 lbOrigy->setText(ApaintConfig::getText(HEIGHT_MSG)+":");
 lbNewWid->setText(ApaintConfig::getText(621)+":");
 lbNewHei->setText(ApaintConfig::getText(HEIGHT_MSG)+":");
 cbResizeCont->setText(ApaintConfig::getText(58));
 cbKeepRatio->setText(ApaintConfig::getText(175));
 btnDensity->setText(ApaintConfig::getText(622));
 
 origWidth = perWid = pxWid = 0 ;
 origHeight = perHei = pxHei = 0 ;
 canSetVal = true; /* set when one spinbox changes value on another */
 type = cbType->currentItem();
 repDlg = repeatDlg;
 Q_CHECK_PTR(repDlg);
 setVisualData();
 connect(cbType, SIGNAL(activated(int)), this, SLOT(changeType(int)));
 connect(sbWid, SIGNAL(valueChanged(int)), this, SLOT(widChanged(int)));
 connect(sbHei, SIGNAL(valueChanged(int)), this, SLOT(heiChanged(int)));
 connect(cbKeepRatio, SIGNAL(clicked()), this, SLOT(keepRatioClicked()));
 connect(btnDensity, SIGNAL(clicked()), this, SLOT(densityClicked()));
}


ScaleWidget::~ScaleWidget()
{
}

void ScaleWidget::changeType(int val)
{
 type = val;
 setVisualData(); 
}

void ScaleWidget::keepRatioClicked()
{
 if (!cbKeepRatio->isChecked()) return;

 if (sbWid->value()!=origWidth)
  widChanged(sbWid->value());
 else if (sbHei->value()!=origHeight)
  heiChanged(sbHei->value());
}

void ScaleWidget::densityClicked()
{  
 useDensity = true;
 widChanged(sbWid->value());
 useDensity = false;
}

void ScaleWidget::widChanged(int val)
{
 double v, aspect, densityRatio, wid, hei;
 int sysx, sysy;
 double systemRatio;

 aspect = 1.0;
 if (type == 0)
  {
   if (!pxWid) pxWid = val;
   aspect = (double) val / (double) origWidth;
   pxWid = val;
   v = ((double)pxWid / (double)origWidth) * 100;
   perWid = (int) v;
  }
 else
  {
   perWid = val;
   v = (double)origWidth*((double)perWid /(double)100);
   pxWid = (int)v;
  }

 if (canSetVal && (keepAspectRatio() || useDensity))
  {
   canSetVal = false;
   if (useDensity)
    {
     repDlg->getDensity(wid, hei);
     repDlg->getSystem(sysx, sysy);
     systemRatio = (double)sysx / (double)sysy;
     densityRatio = (hei/wid) * systemRatio;
    }
   else
    densityRatio = 1.0;

   if (type == 0)
    sbHei->setValue((int)(origHeight*aspect*densityRatio+0.5));
   else
    sbHei->setValue((int)(perWid*densityRatio+0.5));
   canSetVal = true;
  }
}

void ScaleWidget::heiChanged(int val)
{
 double v, aspect;
 aspect = 1.0;

 if (type == 0)
  {
   if (!pxHei) pxHei = val;
   aspect = (double) val / (double) origHeight;
   pxHei = val;
   v= ((double)pxHei / (double)origHeight) * 100;
   perHei = (int) v;
  }
 else
  {
   perHei = val;
   v = (double)origHeight*((double)perHei / (double)100);
   pxHei = (int) v;
  }

 if (canSetVal && keepAspectRatio())
  {
   canSetVal = false;
   if (type == 0)
     sbWid->setValue((int)(origWidth*aspect+0.5));
   else
     sbWid->setValue(perHei);
   canSetVal = true;
  }
}

void ScaleWidget::changeSize(int wid, int hei)
{
 int oldType = type;
 bool tmp = canSetVal;

 type = 0;
 canSetVal = false;
 widChanged(wid);
 heiChanged(hei);
 type = oldType;
 setVisualData();
 canSetVal = tmp;
}

void ScaleWidget::setVisualData()
{
 bool tmp = canSetVal;

 lbOldWid->setText(QString("%1").arg(origWidth));
 lbOldHei->setText(QString("%1").arg(origHeight));
 canSetVal = false;

 if (type == 0)/*px*/
  {
   sbWid->setValue(pxWid);
   sbHei->setValue(pxHei);
  }
 else /*%*/
  {
   sbWid->setValue(perWid);
   sbHei->setValue(perHei);
  }
 canSetVal = tmp;
}

int ScaleWidget::getWidth()
{
 return pxWid;
}

int ScaleWidget::getHeight()
{
 return pxHei;
}


void ScaleWidget::resetData()
{
 changeSize(origWidth, origHeight);
}

void ScaleWidget::setUp(int iwid, int ihei)
{  
 origWidth = pxWid = iwid;
 origHeight = pxHei = ihei;
 perWid = 100;
 perHei = 100;
 setVisualData();
}

/*if keepAspectRatio is ON and OKBtn is clicked, value in w and h fields is not updated because there is not focus change!*/
/*we call this func before Btn signal is send*/
void ScaleWidget::updateFocus()
{
 if (!keepAspectRatio())
  return;

  sbHei->setFocus();
  sbWid->setFocus();
}

/*********************ROTATE WIDGET**************************************/

RotateWidget::RotateWidget(QWidget *parent, const char *name):RotateImageWidgetUI(parent, name)
{
 lbAngle->setText(ApaintConfig::getText(50));
 QDoubleValidator *valid = new QDoubleValidator(0.0, 360.0, 1, this);
 valid->setRange(0, 360, 1);
 leDeg->setValidator(valid);

 connect(btnRotate0, SIGNAL(clicked()), this, SLOT(rotate0Clicked()));
 connect(btnRotate45, SIGNAL(clicked()), this, SLOT(rotate45Clicked()));
 connect(btnRotate90, SIGNAL(clicked()), this, SLOT(rotate90Clicked()));
 connect(btnRotate135, SIGNAL(clicked()), this, SLOT(rotate135Clicked()));
 connect(btnRotate180, SIGNAL(clicked()), this, SLOT(rotate180Clicked()));
 connect(btnRotate225, SIGNAL(clicked()), this, SLOT(rotate225Clicked()));
 connect(btnRotate270, SIGNAL(clicked()), this, SLOT(rotate270Clicked()));
 connect(btnRotate315, SIGNAL(clicked()), this, SLOT(rotate315Clicked()));

 connect(slAngle, SIGNAL(valueChanged(int)), this, SLOT(changeValue10(int)));
 connect(leDeg, SIGNAL(textChanged(const QString&)), this, SLOT(setValue(const QString&)));
 changeValue(0);
}

RotateWidget::~RotateWidget()
{

}

void RotateWidget::rotate0Clicked()
{  
 changeValue(0);
// emit ScaleRotDialog::okClicked();
//  ScaleRotDialog::okBtnClicked();
// emit okClicked();
}

void RotateWidget::rotate45Clicked()
{  
 changeValue(45);
}

void RotateWidget::rotate90Clicked()
{  
 changeValue(90);
}

void RotateWidget::rotate135Clicked()
{  
 changeValue(135);
}

void RotateWidget::rotate180Clicked()
{  
 changeValue(180);
}

void RotateWidget::rotate225Clicked()
{  
 changeValue(225);
}

void RotateWidget::rotate270Clicked()
{  
 changeValue(270);
}

void RotateWidget::rotate315Clicked()
{  
 changeValue(315);
}

void RotateWidget::setValue(const QString& val)
{
 angle = val.toDouble();
 disconnect(leDeg, SIGNAL(textChanged(const QString&)), this, SLOT(setValue(const QString&)));
 changeValue(angle);
 connect(leDeg, SIGNAL(textChanged(const QString&)), this, SLOT(setValue(const QString&)));
}

void RotateWidget::changeValue(double val)
{
 if (val<0)
  val = 360+val;
 slAngle->setValue((int)(val*10+0.5));
 if (leDeg->text() == "")
  changeValue10(slAngle->value());
}

void RotateWidget::changeValue10(int val)
{
 if (val<0)
  val = 360+val;
 leDeg->setText(QString("%1").arg((double)val/10.0));
}


/*********************SCALEROT DIALOG**********************************************/

ScaleRotDialog::ScaleRotDialog(QWidget *parent , const char *name, RepeatDimDlg *repeatDlg): QDialog(parent, name) 
{
 dlgMode = All;
 setCaption(ApaintConfig::getText(667));
 vbl = new QVBoxLayout(this);  
 contourVisible = averageVisible = false;

 setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));

 gbScale = new QGroupBox(this, "gbScale");
 gbScale->setColumnLayout(0, Qt::Vertical );  
 gbScale->setTitle(ApaintConfig::getText(606));
 gbScale->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));
 scaleBox = new QVBoxLayout(gbScale->layout());

 gbRotate = new QGroupBox(this, "bgRotate");
 gbRotate->setColumnLayout(0, Qt::Vertical );
 gbRotate->setTitle(ApaintConfig::getText(85));
 gbRotate->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));
 rotBox = new QVBoxLayout(gbRotate->layout());

 swid = new ScaleWidget(gbScale, "scale widget", repeatDlg);
 scaleBox->addWidget(swid);
 rwid = new RotateWidget(gbRotate, "rotate widget");
 rotBox->addWidget(rwid);

 gbFilter = new QGroupBox(this, "bgFilter");
 gbFilter->setColumnLayout(0, Qt::Horizontal );
 hblFilt = new QHBoxLayout(gbFilter->layout());
 lbFilter = new QLabel(gbFilter);
 lbFilter->setText(ApaintConfig::getText(717)+":");
 cbFilter = new QComboBox(gbFilter);
 QSpacerItem* spaceFilt = new QSpacerItem( 5, 0, QSizePolicy::Expanding , QSizePolicy::Minimum);
 cbContour = new QCheckBox(ApaintConfig::getText(36), gbFilter);
 cbAverage = new QCheckBox(ApaintConfig::getText(41), gbFilter);

 hblFilt->addWidget(lbFilter);
 hblFilt->addWidget(cbFilter);
 hblFilt->addItem(spaceFilt);
 hblFilt->addWidget(cbContour);
 hblFilt->addWidget(cbAverage);

 setFilterBox();

 okBtn = new QPushButton(ApaintConfig::getText(OK_MSG), this);
 okBtn->setFocusPolicy(ClickFocus);
 resetBtn = new QPushButton(ApaintConfig::getText(625), this);
 resetBtn->setFocusPolicy(ClickFocus);
 closeBtn = new QPushButton(ApaintConfig::getText(86), this);
 closeBtn->setFocusPolicy(ClickFocus);
 QSpacerItem* spacer = new QSpacerItem( 30, 0, QSizePolicy::Expanding , QSizePolicy::Minimum);  
 hbl = new QHBoxLayout();
 hbl->addItem(spacer);
 hbl->addWidget(okBtn);
 hbl->addWidget(resetBtn);
 hbl->addWidget(closeBtn);

 vbl->addWidget(gbScale);
 vbl->addWidget(gbRotate);
 vbl->addWidget(gbFilter);
 vbl->addLayout(hbl);
 connect(okBtn, SIGNAL(clicked()), this, SLOT(okBtnClicked()));
 connect(resetBtn, SIGNAL(clicked()), this, SLOT(resetBtnClicked()));
 connect(closeBtn, SIGNAL(clicked()), this, SLOT(closeBtnClicked()));
 connect(cbFilter, SIGNAL(activated(int)), this, SLOT(filterChanged(int)));
 connect(cbContour, SIGNAL(clicked()), this, SIGNAL(contourRotateClicked()));
 connect(cbAverage, SIGNAL(clicked()), this, SIGNAL(averageClicked()));

 connect(rwid->btnRotate0, SIGNAL(clicked()), this, SLOT(okBtnClicked()));
 connect(rwid->btnRotate45, SIGNAL(clicked()), this, SLOT(okBtnClicked()));
 connect(rwid->btnRotate90, SIGNAL(clicked()), this, SLOT(okBtnClicked()));
 connect(rwid->btnRotate135, SIGNAL(clicked()), this, SLOT(okBtnClicked()));
 connect(rwid->btnRotate180, SIGNAL(clicked()), this, SLOT(okBtnClicked()));
 connect(rwid->btnRotate225, SIGNAL(clicked()), this, SLOT(okBtnClicked()));
 connect(rwid->btnRotate270, SIGNAL(clicked()), this, SLOT(okBtnClicked()));
 connect(rwid->btnRotate315, SIGNAL(clicked()), this, SLOT(okBtnClicked()));
}

ScaleRotDialog::~ScaleRotDialog()
{
}

void ScaleRotDialog::resetBtnClicked()              
{
 swid->resetData();
 rwid->resetData();
 emit okClicked();
}

void ScaleRotDialog::okBtnClicked()
{
 swid->updateFocus();
 emit okClicked();
}

void ScaleRotDialog::closeBtnClicked()
{
 close(); 
}              


void ScaleRotDialog::setFilterBox()
{
 cbFilter->insertItem(ApaintConfig::getText(718), 0);
 cbFilter->insertItem(ApaintConfig::getText(719), 1);
 cbFilter->insertItem(ApaintConfig::getText(720), 2);  
// cbFilter->insertItem(ApaintConfig::getText(675)+" 2X", 3);
// cbFilter->insertItem(ApaintConfig::getText(675)+" 3X", 4);
// cbFilter->insertItem(ApaintConfig::getText(675)+" 4X", 5);
}

void ScaleRotDialog::filterChanged(int active)
{
 setRotateContour(contourVisible);
 setAverage(averageVisible);
}

int ScaleRotDialog::getFilterNum()
{
 return cbFilter->currentItem();
}

        
void ScaleRotDialog::setResizeContents(bool enable)
{
 swid->setResizeContents(enable);
}

void ScaleRotDialog::changeSize(int wid, int hei)
{
  swid->changeSize(wid, hei);
}

void ScaleRotDialog::changeAngle(double angle)
{
 rwid->changeValue(angle);
}

void ScaleRotDialog::setSize(int wid, int hei)
{
 swid->setUp(wid, hei);
}

void ScaleRotDialog::setAngle(double val)
{
  rwid->changeValue(val);
}

void ScaleRotDialog::setRotateContour(bool on)
{
 contourVisible = on;
 if (on && getFilterNum()>0)
  cbContour->show();
 else
  cbContour->hide();
}

void ScaleRotDialog::setAverage(bool on)
{
 averageVisible = on;
 if (on)
  cbAverage->show();
 else
  cbAverage->hide();
}

bool ScaleRotDialog::doRotateContour()
{
 return(cbContour->isChecked() && contourVisible);
}

bool ScaleRotDialog::doAverage()
{
 return(cbAverage->isChecked());
}

void ScaleRotDialog::getSize(QSize &size)
{
 size.setWidth(swid->getWidth());
 size.setHeight(swid->getHeight());
}

void ScaleRotDialog::closeEvent(QCloseEvent *e)
{
 setupDlg(All);
 emit closeSig();
 QDialog::closeEvent(e);
}

bool ScaleRotDialog::isScaleMode()
{
 return(dlgMode == ScaleType || dlgMode == All);
}

bool ScaleRotDialog::isRotateMode()
{
 return(dlgMode == RotateType || dlgMode == All);
}

void ScaleRotDialog::setupDlg(TransDlgType type)
{
 if (type == All)
  {
   gbScale->show();
   gbRotate->show();
  }
 else if (type == ScaleType)
  {
   gbScale->show();
   gbRotate->hide();
  }
 else if (type == RotateType)
  {
   gbScale->hide();
   gbRotate->show();   
  }
 dlgMode = type;
}

