/***************************************************************************
                          scalerotdialog.h  -  description
                             -------------------
    begin                : Fri May 30 2003
    copyright            : (C) 2003-2014 Arahne, d.o.o.
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SCALEROTDIALOG_H
#define SCALEROTDIALOG_H

#include <qdialog.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qcheckbox.h>
#include "apaintdoc.h"
#include <qspinbox.h>
#include "widget_ui_rotateImage.h"
#include "widget_ui_scaleImage.h"

class QTabWidget;
class RepeatDimDlg;
class QGroupBox; 
  
class ScaleWidget : public ScaleImgWidgetUI {
  Q_OBJECT
public:
 ScaleWidget(QWidget *parent, const char *name, RepeatDimDlg *repeatDlg);
 ~ScaleWidget();
 int getWidth();
 int getHeight();
 void setUp(int iwid, int ihei);
 void resetData();
 bool resizeContents() {return(cbResizeCont->isChecked());}
 bool keepAspectRatio() {return(cbKeepRatio->isChecked());}
 void setResizeContents(bool enable) { cbResizeCont->setEnabled(enable);}
 void checkResizeContents(bool onOff) { cbResizeCont->setChecked(onOff);}
 void updateFocus();
 
public slots:
   void changeType(int val);
   void widChanged(int val);
   void heiChanged(int val);
   void changeSize(int wid, int hei);
   void keepRatioClicked();
   void densityClicked();
   
private:
   void setVisualData();

//   ScaleRotDialog *parent;
   int origWidth, origHeight;
   int perWid, perHei, pxWid, pxHei;
   int type;
   bool canSetVal;
   RepeatDimDlg *repDlg;
   bool useDensity;
};

class RotateWidget : public RotateImageWidgetUI {
  Q_OBJECT
public:
 RotateWidget(QWidget *parent, const char *name);
 ~RotateWidget();

 double getAngle() { return angle;}
 void changeValue(double val);
public slots:
 
 void changeValue10(int val);
 void resetData() { changeValue(0); }

protected slots:
 void setValue(const QString& val);
 void rotate0Clicked();
 void rotate45Clicked();
 void rotate90Clicked();
 void rotate135Clicked();
 void rotate180Clicked();
 void rotate225Clicked();
 void rotate270Clicked();
 void rotate315Clicked();
 
private:
 double angle;
};
    
class ScaleRotDialog : public QDialog  {
  Q_OBJECT
public: 
	ScaleRotDialog(QWidget *parent , const char *name, RepeatDimDlg *repeatDlg);
	~ScaleRotDialog();

  enum TransDlgType{All=0, ScaleType, RotateType};
  
  void getSize(QSize &size);
  double getAngle() {return rwid->getAngle();}
  bool resizeContents() {return(swid->resizeContents());}
  void setResizeContents(bool enable);
  void checkResizeContents(bool onOff) {swid->checkResizeContents(onOff);}
  bool keepAspectRatio() {return(swid->keepAspectRatio());}
  void setAngle(double val);
  void setupDlg(TransDlgType type);
  bool isScaleMode();
  bool isRotateMode();
  int getFilterNum();
  void setRotateContour(bool on);
  void setAverage(bool on);
  bool doRotateContour();
  bool doAverage();
  
signals:

  void okClicked();
  void resetClicked();
  void closeSig();
  void averageClicked();
  void contourRotateClicked();  

public slots:

  void setSize(int wid, int hei);
  void changeSize(int wid, int hei);
  void changeAngle(double angle);
  void resetBtnClicked();
  void okBtnClicked();
  void closeBtnClicked();
  void filterChanged(int active);
     
protected:
  void closeEvent(QCloseEvent *e);

    
private:
  void setFilterBox();
  
  QPushButton *okBtn, *resetBtn, *closeBtn;
  QVBoxLayout *vbl, *scaleBox, *rotBox;
  QHBoxLayout *hbl, *hblFilt;
  ScaleWidget *swid;
  RotateWidget *rwid;
  QLabel *lbFilter;
  QComboBox *cbFilter;
  QGroupBox *gbScale, *gbRotate, *gbFilter;
  TransDlgType dlgMode;
  bool contourVisible, averageVisible;
  QCheckBox *cbContour, *cbAverage;
};

#endif
