/***************************************************************************
                          bsplinetool.cpp  -  description
                             -------------------
    begin                : Thu Dec 4 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// the main algorithm for drawing curves was taken from Xpaint: splineOp.c

#include "toolBSpline.h"
#include "apaintdrawrepeat.h"
#include <math.h>
#include "apaintUtil.h"

extern char *getCommonPNG(const char *pixName);

#define MAXPOINTS   250
#define SUBDIV 50
#define SUBDIV2 SUBDIV*2
#define SQR(x) ((x)*(x))


BSplineTool::BSplineTool(DrawInRepeat *drawp, ToolType type, PropObject *propObj):ToolBase(type, propObj)
{ 
  outPtsNum = halfOutPtsNum =  0;
  cur=QCursor(ArrowCursor);
  setToggleBtn(QPixmap(getCommonPNG("spline")), tr("Curve Tool"));
  Q_CHECK_PTR(drawp);
  draw = drawp;
  if (propObj)
   setUsepWidget(true);
  im1 = new QPoint[MAXPOINTS];
  im2 = new QPoint[MAXPOINTS];
  bpt = new bool[MAXPOINTS];

  inPts = new QPoint[MAXPOINTS];
  outPts = QPointArray(2*SUBDIV+1);
  halfOutPts = QPointArray(SUBDIV+1);
  resetState();
}

BSplineTool::~BSplineTool()
{
  
}

void BSplineTool::showCommonProp(bool enable)
{
  CommonProp *cProp = CommonProp::getObject();
  cProp->setAspectDrawProp(enable);
  cProp->setThickLinesProp(enable);
}


void BSplineTool::resetState()
{
  drawStarted = false;
  nPoints = 0;
  for (int i=0; i<MAXPOINTS; i++)
  {
   bpt[i] = false;
  }
}

void BSplineTool::begin()
{
  resetState();
}

void BSplineTool::calcCurve(QPoint *points, QPointArray &output, int &outputNum, int &outputNumHalf,
                            QPoint *interm1, QPoint *interm2, bool *breakpt)
{
    int ni, i, j, i0, i1, n;
    int u, v;
    float ax=0.0, ay=0.0, bx, by, cx, cy, r=0.0, s, t;
    int len=0;
    
   if (nPoints == 1)
   {
    output.setPoint(0, points[0].x(), points[0].y());
    output.setPoint(1, points[1].x(), points[1].y());
    outputNum = 2;
    outputNumHalf = 1;
    return;
   }
   n = j = nPoints;
   i0 = n-2;
   if (i0<0)
     i0 = 0;
   for (i = i0; i < j; i++)
    {
        s = r;
        bx = ax;
        by = ay;
        ax = points[i+1].x() - points[i].x();
        ay = points[i+1].y() - points[i].y();
        r = ax*ax + ay*ay + 1E-9;
        ax = ax/r; ay = ay/r;
        r = sqrt(r)/3.0;
        if (i > i0) {
           if (breakpt[i])
           {
               t = 3.0*r*r;
               interm1[i].setX(points[i].x() + (short)(t*ax));
               interm1[i].setY(points[i].y() + (short)(t*ay));
               t = 3.0*s*s;
               interm2[i-1].setX(points[i].x() - (short)(t*bx));
               interm2[i-1].setY(points[i].y() - (short)(t*by));
           }
           else
           {
               cx = (ax+bx)/2;
               cy = (ay+by)/2;
               t = sqrt(cx*cx + cy*cy + 1E-9);
               cx = cx/t;
               cy = cy/t;
               interm1[i].setX(points[i].x() + (short)(r*cx));
               interm1[i].setY(points[i].y() + (short)(r*cy));
               interm2[i-1].setX(points[i].x() - (short)(s*cx));
               interm2[i-1].setY(points[i].y() - (short)(s*cy));
           }
           if (i == 1)
           {
               t = 3.0*s*s;
               interm1[0].setX(interm2[0].x() - (short)(t*bx));
               interm1[0].setY(interm2[0].y() - (short)(t*by));
           } 
           if (i == n-1)
           {
               t = 3.0*r*r;
               interm2[i].setX(interm1[i].x() + (short)(t*ax));
               interm2[i].setY(interm1[i].y() + (short)(t*ay));
           }           
         
        }
    }

    i0 = n-2;
    i1 = n;

        
    len+=(int)sqrt(SQR(points[n-2].x()-points[n-1].x())+SQR(points[n-2].y()-points[n-1].y()));
    len+=(int)sqrt(SQR(points[n-1].x()-points[n].x())+SQR(points[n-1].y()-points[n].y()))/5;
    
    outputNum = QMIN(QMAX(3,len), SUBDIV2);
    outputNumHalf = outputNum/2;
     
    output.setPoint(0, points[i0].x(), points[i0].y());
    ni=1;
    for (i = i0; i <i1; i++)
    {       
       for (j = 1; j <=outputNumHalf; j++)
       {
         t = ((float) j)/((float) outputNumHalf);
         s = 1.0-t;
         u = (int) (s*s*(s*points[i].x()+3.0*t*interm1[i].x())+ t*t*(t*points[i+1].x()+3.0*s*interm2[i].x()));
         v = (int) (s*s*(s*points[i].y()+3.0*t*interm1[i].y())+ t*t*(t*points[i+1].y()+3.0*s*interm2[i].y()));
         output.setPoint(ni,  u, v);
         ni++;
       }
    }
    output.setPoint(outputNum-1, points[n].x(), points[n].y());
}

void BSplineTool::drawCurve(QPainter *p, bool doRefresh, bool final)
{
  if (nPoints == 1)
    draw->drawLine(p, outPts.point(0).x(), outPts.point(0).y(), outPts.point(1).x(), outPts.point(1).y(), doRefresh, true, final);
  else
    draw->drawPolygon(p, outPts, outPtsNum, doRefresh, true, final);
}

void BSplineTool::saveCurve(QPainter *p, bool finish)
{     
 p->setRasterOp(CopyROP);

 if (finish)
 {
   drawCurve(p, true, finish);
 }
 else
 {
  halfOutPts.putPoints(0, halfOutPtsNum+1, outPts);
  draw->drawPolygon(p, halfOutPts, halfOutPtsNum+1, true, true, true);
 }
}


void BSplineTool::mousePressed(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
  int xi=(int)(e->x()/zoomX);
  int yi=(int)(e->y()/zoomY);
  

  QPen pen= QPen(pi->pen());
  pen.setJoinStyle(Qt::RoundJoin);
  pi->setPen(pen);
  btnPressed = e->button();
  if (btnPressed == MidButton) return;
  if (btnPressed == LeftButton)
  {
    if (!nPoints)
    {
      inPts[0].setX(xi);
      inPts[0].setY(yi);
      inPts[1].setX(xi);
      inPts[1].setY(yi);
      
      nPoints = 1;
      drawStarted = true;
      calcCurve(inPts, outPts, outPtsNum, halfOutPtsNum, im1, im2, bpt);
      drawCurve(pi, false, false);
    }
    else if (nPoints == 1)
    {
      pi->setRasterOp(NotROP);
      drawCurve(pi, false, false);
      nPoints++;
      inPts[nPoints].setX(xi);
      inPts[nPoints].setY(yi);
      calcCurve(inPts, outPts, outPtsNum, halfOutPtsNum, im1, im2, bpt);
      drawCurve(pi, true, false);
    }
    else if (nPoints == MAXPOINTS-1)
    {
      saveCurve(pi, true);
      resetState();
    }
    else
    {
      pi->setRasterOp(NotROP);
      drawCurve(pi, false, false);
      saveCurve(pi, false);
      pi->setRasterOp(NotROP);
      nPoints++;
      inPts[nPoints].setX(xi);
      inPts[nPoints].setY(yi);
      calcCurve(inPts, outPts, outPtsNum, halfOutPtsNum, im1, im2, bpt);
      drawCurve(pi, true, false);
    }
  }
  else if (btnPressed == RightButton)
  {
    saveCurve(pi, true);
    resetState();
  }
}

void BSplineTool::mouseMoved(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
  if (!drawStarted) return;
  QPen pen= QPen(pi->pen());
  pen.setJoinStyle(Qt::RoundJoin);
  pi->setPen(pen);
  pi->setRasterOp(NotROP);
  int xi=(int)(e->x()/zoomX);
  int yi=(int)(e->y()/zoomY);

  if (xi == inPts[nPoints-1].x() && yi == inPts[nPoints-1].y()) return;
  drawCurve(pi, false, false);
  inPts[nPoints].setX(xi);
  inPts[nPoints].setY(yi);
  calcCurve(inPts, outPts, outPtsNum, halfOutPtsNum, im1, im2, bpt);
  drawCurve(pi, true, false);
     
}

