/***************************************************************************
                          cleanupTool.cpp  -  description
                             -------------------
    begin                : et feb 26 2004
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <math.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include "toolCleanup.h"
#include "apaintdrawrepeat.h"
#include "apaintdoc.h"
#include "apaintconfig.h"
#include "apaintimageUtil.h"
#include "apaintUtil.h"

extern char *getCommonPNG(const char *pixName);

#define SMALL_R   5
#define BIG_R     10

#define SQR(a) ((a)*(a))


CleanupProp::CleanupProp(QWidget *parent, const char *name):CleanupPropDialogUI(parent, name)
{
  lbAverage->setText(ApaintConfig::getText(673));
  lbDraw->setText(ApaintConfig::getText(672));
  cbBorder->setText(ApaintConfig::getText(674));
  cbSmooth->setText(ApaintConfig::getText(675));
  sbDraw->setValue(SMALL_R);
  sbAverage->setValue(BIG_R);
  connect(sbDraw, SIGNAL(valueChanged(int)), this, SLOT(updateAverage(int)));
  connect(sbAverage, SIGNAL(valueChanged(int)), this, SLOT(updateDraw(int)));
}


void CleanupProp::updateAverage(int val)
{
  if (sbAverage->value() <= val)
    sbAverage->setValue(val+1);
}

void CleanupProp::updateDraw(int val)
{
  if (sbDraw->value()>=val)
   sbDraw->setValue(val-1);
}

CleanupTool::CleanupTool(DrawInRepeat *drawp, ToolType type, ApaintDoc *docp, PropObject *propObj):ToolBase(type, propObj)
{
  doBorder = doSmooth = false;  
  setToggleBtn(QPixmap(getCommonPNG("draw-eraser")), tr("Cleanup Tool"));
  if (propObj)
  {
   setUsepWidget(true);
   cProp = (CleanupProp *) propObj->pWidget();
  }
  Q_CHECK_PTR(drawp);
  Q_CHECK_PTR(docp);
  draw = drawp;
  doc = docp;
  btnPressed = false;
  aspectDrawSetting = false;
  setAutoScroll(false);
}

void CleanupTool::begin()
{
  CommonProp *cp = CommonProp::getObject();
  aspectDrawSetting = cp->doAspectDraw();
  if (aspectDrawSetting)
   cp->enableAspectDrawProp(false);
}

void CleanupTool::end()
{
 CommonProp *cp = CommonProp::getObject();
 if (!cp->doAspectDraw() && aspectDrawSetting)
   cp->enableAspectDrawProp(true);
}

void CleanupTool::cleanUp(QPainter *p, int xin, int yin, int rout, int rin, ApaintImage &img, bool final, bool *pColors, QRect layerPos)
{
 int left, right, up, down, xp, yp;
 uint col=0;
 double ax, ay, dax, day;
 
 ax = dax = doc->getAspectX();
 ay = day = doc->getAspectY();
 if ((ax == 1) && (ay == 1))
  {
   ax = aspectX;
   ay = aspectY;
  }
 
 xin-=layerPos.x();
 yin-=layerPos.y();
 
 if (doc->getLayer()->isUsingTiles())
  draw->transPoint(xin, yin, &xp, &yp);
 else
  {
   xp = xin;
   yp = yin;
  }

 doSmooth = cProp->cbSmooth->isChecked();
 

 left =  xp-rout;
 right = xp+rout;
 up =    yp-rout;
 down =  yp+rout;

 
 if (left<0) left=0;
 if (up<0) up=0;
 if (left>img.width()-1) return;
 if (up>img.height()-1) return;

 if (right>img.width()-1) right = img.width()-1;
 if (down>img.height()-1) down = img.height()-1;
 if (right<0) return;
 if (down<0) return;
 
 if ((doSmooth && cProp->cbBorder->isChecked()) || !doSmooth)
  col = getAverageColor(xp, yp, QRect(left, up, right-left+1, down-up+1), rout, img);
  
 left =  xp-rin;
 right = xp+rin;
 up =    yp-rin;
 down =  yp+rin;
 
 if (left<0) left=0;
 if (up<0) up=0;
 if (left>img.width()-1) return;
 if (up>img.height()-1) return;

 if (right>img.width()-1) right = img.width()-1;
 if (down>img.height()-1) down = img.height()-1;
 if (right<0) return;
 if (down<0) return;
 
 if ((doSmooth && cProp->cbBorder->isChecked()) || !doSmooth)
  changeAreaToColor(xp, yp, col, QRect(left, up, right-left+1, down-up+1), rin, img, pColors);

 if (doSmooth)
  {
   if (img.type() == LOAD_CMAP)
    ApaintImageUtil::averageContour(img, left, up, right-left+1, down-up+1, pColors, false, rin, ax, ay);
   else if (img.type() == LOAD_TRUE_COL)
    ApaintImageUtil::averageContour24thickness(img, left, up, right-left+1, down-up+1, false, rin, rin);
  }

 QRect drec = QRect((int)((left+layerPos.x())*dax), (int)((up+layerPos.y())*day),
                    (int)((right-left)*dax), (int)((down-up)*day));
 p->fillRect(drec.x(), drec.y(), drec.width(), drec.height(), QBrush(white, SolidPattern));
 
 draw->refreshDirect(left+layerPos.x(), up+layerPos.y(), right-left, down-up, false, false, final);
}

uint CleanupTool::getAverageColor(int xp, int yp, QRect area, int r, ApaintImage &img)
{

 int x, y, numPix, r2;
 uint red, green, blue;
 uint col=0;
 ushort colTab[256];
 int numAll, indx;
 double ratio;
 
 /*init*/
 red = green = blue = 0;
 numPix = 0;
 r2 = r*r; 
 memset(colTab, 0, img.numColors()*sizeof(ushort));
 numAll = 0;
 doBorder = false;
 /*calc*/ 
 for (y=area.top(); y<area.bottom(); y++) 
 {
  if (img.type() == LOAD_TRUE_COL)
   {
    for (x=area.left(); x<area.right(); x++)
     {
      if (((x-xp)*(x-xp)+(y-yp)*(y-yp))<r2)
       {
        col = img.pixel(x, y);
        red+=qRed(col);
        green+=qGreen(col);
        blue+=qBlue(col);
        numPix++;
       }
     }
   }
  else if (img.type() == LOAD_CMAP)
   for (x=area.left(); x<area.right(); x++)
    if (((x-xp)*(x-xp)+(y-yp)*(y-yp))<r2)
     colTab[img.pixelIndex(x, y)]++;
  }
   
  if (img.type() == LOAD_TRUE_COL)
   {
    red/=numPix;
    green/=numPix;
    blue/=numPix;
    col = qRgb(red, green, blue) ;
   }
  else if (img.type() == LOAD_CMAP)
   {
    col = 0;
    for (x=0; x<img.numColors(); x++)
     {
      numAll+= colTab[x];
      if (colTab[col]<colTab[x])
       col = x;
     }
    indx = 0;
    for (x=0; x<img.numColors(); x++)
     {
      ratio = (double)colTab[x] / (double)numAll;
      if ((int)(ratio*100)>=20 && indx<2)
       {
	bIndex[indx] = x;
	indx++;
       }
     }
    if (indx == 2 && cProp->cbBorder->isChecked())
     doBorder = true;
   }
 return(col);
}

int CleanupTool::colLen(uint c1, uint c2)
{
   int redc, greenc, bluec;
   
   redc=QABS(qRed(c1) - qRed(c2));
   greenc=QABS(qGreen(c1) - qGreen(c2));
   bluec=QABS(qBlue(c1) - qBlue(c2));
   
   return ((int)sqrt(SQR(redc)+SQR(greenc)+SQR(bluec)));

}

void CleanupTool::changeAreaToColor(int xp, int yp, uint col, QRect area, int r, ApaintImage& img, bool *pColors)
{
  int x, y, r2, len1, len2;
  QRgb *ctab;
  double ax = doc->getAspectX();
  double ay = doc->getAspectY();
  if ((ax == 1) && (ay == 1))
  {
    ax = aspectX;
    ay = aspectY;
  }
  
  r2 = r*r;
  ctab = img.colorTable();
  for (y=area.top(); y<area.bottom(); y++)
  {
    if (doBorder)
    {       
       for (x=area.left(); x<area.right(); x++)
       {
         if (SQR((x-xp)*ax)+SQR((y-yp)*ay)<r2 || (r2 == 1))
         {                          
             len1 = colLen(ctab[img.pixelIndex(x, y)], ctab[bIndex[0]]);
             len2 = colLen(ctab[img.pixelIndex(x, y)], ctab[bIndex[1]]);
             if (len1>len2)
              img.setPixel(x, y, bIndex[1]);
             else
              img.setPixel(x, y, bIndex[0]);
         }
       }
    }
    else
    {
       if (img.type() == LOAD_CMAP)
       {
          for (x=area.left(); x<area.right(); x++)
          {
           if (!pColors[img.pixelIndex(x, y)])
             if (SQR((x-xp)*ax)+SQR((y-yp)*ay)<r2 || (r2 == 1))
                 img.setPixel(x, y, col);
           }
       }
       else if (img.type() == LOAD_TRUE_COL)
       {
          for (x=area.left(); x<area.right(); x++)
          {
             if (SQR((x-xp)*ax)+SQR((y-yp)*ay)<r2 || (r2 == 1))
                img.setPixel(x, y, col);
          }
       }
    }
  }
}

void CleanupTool::drawPointer(QPainter *p, int x, int y)
{
  int sx, sy, bx, by;

  double ax = doc->getAspectX();
  double ay = doc->getAspectY();
  if ((ax == 1) && (ay == 1))
  {
    ax = aspectX;
    ay = aspectY;
  }

  sx = (int)(cProp->getDrawDiameter() / ax);
  sy = (int)(cProp->getDrawDiameter() / ay);
  bx = (int)(cProp->getAverageDiameter() / ax);
  by = (int)(cProp->getAverageDiameter() / ay);

  p->setRasterOp(NotROP);
  p->setPen(QPen(QColor(128, 128,128)));
  p->drawEllipse((int)(x-sx*zoomX), (int)(y-sy*zoomY), (int)(2*sx*zoomX), (int)(2*sy*zoomY));
  if (!cProp->cbSmooth->isChecked() || cProp->cbBorder->isChecked())
    p->drawEllipse((int)(x-bx*zoomX), (int)(y-by*zoomY), (int)(2*bx*zoomX), (int)(2*by*zoomY));

}

void CleanupTool::mousePressed(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
  if (e->button() == LeftButton)
   btnPressed = true;
  int xin = (int)(e->pos().x()/zoomX);
  int yin = (int)(e->pos().y()/zoomY);

  Layer *layer = doc->getLayer();
  if (layer)
  {
    cleanUp(pi, xin, yin, cProp->getAverageDiameter(), cProp->getDrawDiameter(),
            layer->imageBuf, false, layer->getProtectColors(), layer->layerPos());
  }

}

void CleanupTool::mouseMoved(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
  
  int xin = (int)(e->pos().x()/zoomX);
  int yin = (int)(e->pos().y()/zoomY);

  pc->setRasterOp(NotROP);
  Layer *layer = doc->getLayer();
  if (layer && btnPressed)
    cleanUp(pi, xin, yin, cProp->getAverageDiameter(),
            cProp->getDrawDiameter(), layer->imageBuf, false, layer->getProtectColors(), layer->layerPos());
 

}

void CleanupTool::mouseReleased(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
  
  int xin = (int)(e->pos().x()/zoomX);
  int yin = (int)(e->pos().y()/zoomY);
  
  btnPressed = false;
  /*wid = 1 and hei = 1 becouse we just want to save UNDO data; the rest is calculated in cleanUp function*/
  draw->refreshDirect(xin, yin, 1, 1, false, false, true);
}

