/***************************************************************************
                          filltool.cpp  -  description
                             -------------------
    begin                : Wed Mar 5 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <qcheckbox.h>
#include <qlayout.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <apaintconfig.h>
#include <math.h>
#include "toolFill.h"
#include "apaintdrawrepeat.h"
#include "apaintdoc.h"
#include "apaintBrushList.h"
#include "apaintUtil.h"

extern char *getCommonPNG(const char *pixName);

#define MAX 30000   /* max depth of stack */

typedef struct {short y, xl, xr, dy;} Segment;

#define PUSH(Y, XL, XR, DY) if ((Y+(DY)>=0) && (sp<stack+MAX) && (Y+(DY)<=hei)) \
    {sp->y = Y; sp->xl = XL; sp->xr = XR; sp->dy = DY; sp++; } /* push new segment on stack */ 

#define POP(Y, XL, XR, DY) /* pop segment off stack */ \
    {sp--; Y = sp->y+(DY = sp->dy); XL = sp->xl; XR = sp->xr;}

FillToolProp::FillToolProp(QWidget *parent, const char *name):QWidget(parent, name)
{
 QGridLayout *gbox = new QGridLayout(this, 2, 8);
 QButtonGroup *bg1 = new QButtonGroup(ApaintConfig::getText(793), this);

 cbFillToBorder = new QCheckBox(ApaintConfig::getText(726), this);
 cb8fill = new QCheckBox(ApaintConfig::getText(740), this);
 cbWholeImage = new QCheckBox(ApaintConfig::getText(739), this);

 gbox->addMultiCellWidget(cbFillToBorder, 0, 0, 0, 2);
 gbox->addMultiCellWidget(cb8fill, 1, 1, 0, 1);
 gbox->addMultiCellWidget(cbWholeImage, 2, 2, 0, 2);

 connect(cbFillToBorder, SIGNAL(toggled(bool)), this, SIGNAL(fillToBorderToggle(bool)));
 connect(cb8fill, SIGNAL(toggled(bool)),  this, SIGNAL(fill8way(bool)));
 connect(cbWholeImage, SIGNAL(toggled(bool)),  this, SIGNAL(fillWholeImage(bool)));

 lbTreshold = new QLabel(ApaintConfig::getText(727)+" ", this);
 gbox->addWidget(lbTreshold, 4, 0);
 sbTreshold = new QSpinBox(0, 128, 1, this);
 sbTreshold->setValue(0);
 gbox->addWidget(sbTreshold, 4, 1);

 lbFillPercentage = new QLabel(ApaintConfig::getText(668)+"(%) ", this);
 gbox->addWidget(lbFillPercentage, 5, 0);
 sbFillPercentage = new QSpinBox(1, 100, 1, this);
 sbFillPercentage->setValue(100);
 gbox->addWidget(sbFillPercentage, 5, 1);

 bg1->setColumnLayout(0, Qt::Vertical );
 bg1->layout()->setSpacing( 2 );
 bg1->layout()->setMargin( 2 );

 gbox->addMultiCellWidget(bg1, 5, 5, 0, 2);

 QSpacerItem* spacer = new QSpacerItem(6, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
 gbox->addItem(spacer, 7, 0);
}

FillToolProp::~FillToolProp()
{

}

int FillToolProp::getTreshold()
{
  return  sbTreshold->value();
}

int FillToolProp::getFillPercentage()
{
  return  sbFillPercentage->value();
}
    
FillTool::FillTool(DrawInRepeat *drawp, ApaintDoc *docp, ToolType type, PropObject *propObj):ToolBase(type, propObj)
{
 cur=QCursor(CrossCursor); 
 setToggleBtn(QPixmap(getCommonPNG("color-fill")), tr("Fill"));
 bList = ApaintBrushList::getObject();
 if (propObj)
 {
  setUsepWidget(true);
  fillProp = (FillToolProp *) propObj->pWidget();
  connect(fillProp, SIGNAL(fillToBorderToggle(bool)), this, SLOT(enableFillToBorder(bool)));
  connect(fillProp, SIGNAL(fill8way(bool)), this, SLOT(enableFill8way(bool)));
  connect(fillProp, SIGNAL(fillWholeImage(bool)), this, SLOT(enableFillWholeImage(bool)));
 }
 Q_CHECK_PTR(drawp);
 Q_CHECK_PTR(docp);
 draw = drawp;
 doc = docp;
 protCol = 0;
 doFillToBorder = do8wayFill = aspectDrawSetting = doWholeImage = false;
}

FillTool::~FillTool(){
}

void FillTool::begin()
{
  CommonProp *cp = CommonProp::getObject();
  aspectDrawSetting = cp->doAspectDraw();
  if (aspectDrawSetting)
   cp->enableAspectDrawProp(false);
}

void FillTool::end()
{
 CommonProp *cp = CommonProp::getObject();
 if (!cp->doAspectDraw() && aspectDrawSetting)
   cp->enableAspectDrawProp(true);
}

void FillTool::enableFillToBorder(bool enable)
{
  doFillToBorder = enable;
}

void FillTool::enableFill8way(bool enable)
{
  do8wayFill = enable;
}

void FillTool::enableFillWholeImage(bool enable)
{
  doWholeImage = enable;
}

void FillTool::mousePressed(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
 mousePressedDoit(e, pc, pi);
}

void FillTool::mousePressedDoit(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
  int x,y, treshold;
  QRect area;
  Layer *selLayer = doc->getLayer();
  if (!selLayer) return;
  bool useBrush;
  ApaintImage bImg = bList->getBrush();  

  if (bImg.isNull())
   useBrush = false;
  else
   useBrush = bList->useBrush();
  if (bList->isColorTabDifferent())
   {
    doc->putUndoStack(ChangeColorTable);
    bList->mergeColorTables(selLayer);
    emit colorTableChanged(selLayer);
   }
  if (useBrush)
   brushMask = bList->getBrushMask();
    
  treshold = fillProp->getTreshold()*3;
  if (useBrush || doFillToBorder || treshold)
   {
    validMask = QImage(selLayer->imageBuf.width(), selLayer->imageBuf.height(), 1, 2,  QImage::LittleEndian);
    validMask.fill(0);
    useMaskFill = TRUE;
   }
  else
   useMaskFill = FALSE;
  transTable = bList->getTransTable();
  QRect br = selLayer->layerPos();
  btnPressed = e->button();
  if (btnPressed == MidButton)
   return;
  protCol = selLayer->getProtectColors();
  x=(int)(e->x()/zoomX)-br.x();
  y=(int)(e->y()/zoomY)-br.y();
  pi->translate(selLayer->drawAreaPos().x(), selLayer->drawAreaPos().y());

  if (doWholeImage)
   {
    area = selLayer->layerPos();
    if (btnPressed == LeftButton)
     area = fillWholeImage(x, y, area, selLayer->getFgCol(), selLayer->getFgIndx(), selLayer->getBgCol(), 
                     selLayer->getBgIndx(), treshold, pi, selLayer->imageBuf, useBrush, &bImg, fillProp->getFillPercentage());
    else
     area = fillWholeImage(x, y, area, selLayer->getBgCol(), selLayer->getBgIndx(), selLayer->getFgCol(),
                     selLayer->getFgIndx(), treshold, pi, selLayer->imageBuf, useBrush, &bImg, fillProp->getFillPercentage()); 
   }
  else
   {
    if ((fillProp->getFillPercentage()<100) && (selLayer->imageBuf.depth()==8) && (selLayer->imageBuf.numColors()<255))
     {
      int i, j, wid, hei, xin, yin;
      int fillPercentage = fillProp->getFillPercentage();
      unsigned char fgcol, bgcol;
      unsigned char *line;

      /* temporary fill it with color 255, and then replace it with the noise area */
      /* use rand for color value in the hope it will be different than any of the colors from the palette */
      /* not very beautiful solution, but should be good enough, and it is a rarely used function */
      area = fill(x,y, selLayer->layerPos(), rand(), 255, rand(),
        255, do8wayFill, doFillToBorder, treshold, pi, selLayer->imageBuf, FALSE, &bImg);

      wid = selLayer->imageBuf.width();
      hei = selLayer->imageBuf.height();

      fgcol = selLayer->getFgIndx();
      bgcol = selLayer->getBgIndx();

      if (doc->getLayer()->isUsingTiles())  
       draw->transPoint(x, y, &xin, &yin);
      else
       {
	xin = x;
	yin = y;
       }

      if (useBrush)
       for (j=0; j<hei; j++)
	{
         line = selLayer->imageBuf.scanLine(j);
	 for (i=0; i<wid; i++)
	  if (line[i]==255)
	   {
	    if ((rand()%100)<fillPercentage)
             setPixColBrush(i, j, xin, yin, pi, selLayer->imageBuf, &bImg);
	    else
	     line[i] = bgcol;
	   }
	}
      else
       for (j=0; j<hei; j++)
	{
         line = selLayer->imageBuf.scanLine(j);
	 for (i=0; i<wid; i++)
	  if (line[i]==255)
	   line[i] = ((rand()%100)<fillPercentage) ? fgcol : bgcol;
	}
     }
    else
     {
      if (btnPressed == LeftButton)
        area = fill(x,y, selLayer->layerPos(), selLayer->getFgCol(), selLayer->getFgIndx(), selLayer->getBgCol(),
          selLayer->getBgIndx(), do8wayFill, doFillToBorder, treshold, pi, selLayer->imageBuf, useBrush, &bImg);
      else
	area = fill(x,y, selLayer->layerPos(), selLayer->getBgCol(), selLayer->getBgIndx(), selLayer->getFgCol(),
          selLayer->getFgIndx(), do8wayFill, doFillToBorder, treshold, pi, selLayer->imageBuf, useBrush, &bImg);
     }
   }

  if (area.isValid())
    draw->refreshImageDraw(area.x()+br.x(), area.y()+br.y(), area.width(), area.height(), true);

  validMask = QImage();
}

// ZZZZ
bool FillTool::validPoint(int xin, int yin, uchar red, uchar green, uchar blue, uint bgCol, uchar bgColPtr,
                          int treshold, bool diffColor, ApaintImage& img)
{
 int x = xin;
 int y = yin;
 uint col;
 unsigned char index;

 QRect pos = doc->getLayerPos();
 if (!pos.contains(xin+pos.x(), yin+pos.y()))
  return false;

 if (doc->getLayer()->isUsingTiles())
  draw->transPoint(xin, yin, &x, &y);
 if (useMaskFill)
  if (PIX_ON(validMask, x, y))
   return false;

 index = img.pixelIndex(x,y);
 col = img.pixel(x,y);

 if (diffColor)
  {
   if (img.depth()>8)
    return (bgCol!=col);
   else
    {
     if (protCol)
      if (protCol[index])
       return(false);
     return (bgColPtr!=index);
    }
  }
 else
  {
   if (img.depth()==8)
    {
     if (protCol)
      if (protCol[index])
       return(false);
     if (equalToFillColor[index])
      return(false);
    }
   if (treshold)
    return (((QABS(qRed(col) - red) + QABS(qGreen(col) - green) + QABS(qBlue(col) - blue) ) <= treshold));
   else
    return ((qRed(col)==red) && (qGreen(col)==green) && (qBlue(col)==blue));
  }
}

void FillTool::setPixCol(int xin, int yin, uint col, int colPtr, QPainter *p, ApaintImage& img)
{
 int x=xin;
 int y=yin;

 if (doc->getLayer()->isUsingTiles())
  draw->transPoint(xin, yin, &x, &y);

  img.setPixel(x,y, (img.depth()>8) ? col : colPtr);
  p->drawPoint((int)(x*doc->getAspectX()), (int)(y*doc->getAspectY()));
  if (useMaskFill)
   SET_PIX(validMask, x, y);
}



// ZZZZZ
void FillTool::setPixColBrush(int xin, int yin, int xd, int yd, QPainter *p, ApaintImage& img, ApaintImage *brushImg)
{
  int x=xin;
  int y=yin;
  int xsrc, ysrc;
  uint col;

  if (doc->getLayer()->isUsingTiles())
   draw->transPoint(xin, yin, &x, &y);
  if (bList->getAbsOffset())
   {
    xsrc = (x - bList->getOffsetX() + brushImg->width()*MAX_IMAGE) % brushImg->width();
    /* absolute offset must start from bottom */
    ysrc = (y + bList->getOffsetY() +  (brushImg->height() - (img.height()%brushImg->height())) ) % brushImg->height();
   }
  else
   {
    xsrc = (x - bList->getOffsetX() + brushImg->width()*MAX_IMAGE - (xd % brushImg->width())) % brushImg->width();
    ysrc = (y + bList->getOffsetY() + brushImg->height()*MAX_IMAGE - (img.height()%brushImg->height()) - (yd % brushImg->height()) - 1) % brushImg->height();
   }

  if (PIX_ON(brushMask, xsrc, ysrc))
   {
    if (img.type() == LOAD_TRUE_COL)
     {
      col = brushImg->pixel(xsrc, ysrc);
      img.setPixel(x, y, col);
     }
    else
     if (img.type() == LOAD_CMAP)
      {
       col = brushImg->pixelIndex(xsrc, ysrc);
       img.setPixel(x, y, transTable[col]);
      }
     p->drawPoint((int)(x*doc->getAspectX()), (int)(y*doc->getAspectY()));
   }
  SET_PIX(validMask, x, y);
}

/* Fill whole image. */
QRect FillTool::fillWholeImage(int x, int y, QRect pos, uint setCol, int setColPtr, uint bgCol, int bgColPtr,
                              int treshold, QPainter *p, ApaintImage& img, bool useBrush, ApaintImage *brushImg, int fillPercentage)
{
 int wid, hei, xin, yin;
 uint col=0;
 int colPtr=0;
 uchar red, green, blue;

/* we need should do the operation on base, not on the repeated image */
 wid = img.width();
 hei = img.height();
 if (doc->getLayer()->isUsingTiles())  
  draw->transPoint(x, y, &xin, &yin);
 else
  {
   xin = x;
   yin = y;
  }

 if (img.depth()>8)
  {
   col = img.pixel(xin,yin); 
   if (col == setCol && !useBrush) return QRect();
   red =   qRed(col);
   green = qGreen(col);
   blue =  qBlue(col);
  }
 else
  {
   colPtr = img.pixelIndex(xin, yin);
   if (useBrush && (colPtr==setColPtr)) /* trick so that we can fill with brush regardless of selected draw color */
    setColPtr = (setColPtr+1)%img.numColors();
   if ((!useBrush) && (colPtr == setColPtr)) return QRect();
/*   if (protCol && protCol[setColPtr]) return QRect(); filling with protected color must be allowed... */
   red = qRed(img.color(colPtr));
   green = qGreen(img.color(colPtr));
   blue = qBlue(img.color(colPtr));
   for (int i=0; i<img.numColors(); i++)
    equalToFillColor[i] = (img.color(setColPtr)==img.color(i));
  }

// if (x<0 || x>wid || y<0 || y>hei) return QRect(); /* wrong, reqeuests it to be within area */
 /* the point clicked to, translated to base image, needs to be within the base image */
 if (xin<0 || xin>wid || yin<0 || yin>hei) return QRect();

 if (useBrush)
  {
   if (fillPercentage==100)
    {
     for (y = 0; y < hei; y++)
      for (x=0; x < wid; x++)
       if (validPoint(x,y, red, green, blue, bgCol, bgColPtr, treshold, false, img))
	setPixColBrush(x, y, xin, yin, p, img, brushImg);
    }
   else
    {
     for (y = 0; y < hei; y++)
      for (x=0; x < wid; x++)
       if (validPoint(x,y, red, green, blue, bgCol, bgColPtr, treshold, false, img))
        if ((rand()%100)<fillPercentage)
	 setPixColBrush(x, y, xin, yin, p, img, brushImg);
    }
  }
 else
  {
   if (fillPercentage==100)
    {
     for (y = 0; y < hei; y++)
      for (x=0; x < wid; x++)
       if (validPoint(x,y, red, green, blue, bgCol, bgColPtr, treshold, false, img))
	setPixCol(x, y, setCol, setColPtr, p, img);
    }
   else
    {
     for (y = 0; y < hei; y++)
      for (x=0; x < wid; x++)
       if (validPoint(x,y, red, green, blue, bgCol, bgColPtr, treshold, false, img))
        if ((rand()%100)<fillPercentage)
	 setPixCol(x, y, setCol, setColPtr, p, img);
    }
  }
 return pos;
}


QRect FillTool::fill(int x, int y, QRect pos, uint setCol, int setColPtr, uint bgCol, int bgColPtr,
                     bool do8way, bool fillToBorder, int treshold, QPainter *p,
                     ApaintImage& img, bool useBrush, ApaintImage *brushImg)
{
int l, x1, x2, dy, wid, hei, xmin, ymin, xmax, ymax, xin, yin;
Segment stack[MAX], *sp = stack;	/* stack of filled segments */
uint col=0;
int colPtr=0;
uchar red, green, blue;

 wid = pos.width();
 hei = pos.height(); //must use hei variable because of PUSH macro

 xmin = wid;
 ymin = hei;
 xmax = ymax = 0;

 if (doc->getLayer()->isUsingTiles())  
   draw->transPoint(x, y, &xin, &yin);
 else
  {
   xin = x;
   yin = y;
  }

 if (img.depth()>8)
  {
   col = fillToBorder ? setCol : img.pixel(xin,yin); /* read pv at seed point */
   if (img.pixel(xin,yin)==setCol && !useBrush) return QRect();
   red =   qRed(col);
   green = qGreen(col);
   blue =  qBlue(col);
  }
 else
  {
   colPtr = fillToBorder ? setColPtr : img.pixelIndex(xin,yin);
   if (img.pixelIndex(xin,yin)==setColPtr && !useBrush) return QRect();
/*   if (protCol && protCol[setColPtr]) return QRect(); filling with protected color must be allowed... */

   red = qRed(img.color(colPtr));
   green = qGreen(img.color(colPtr));
   blue = qBlue(img.color(colPtr));

   for (int i=0; i<img.numColors(); i++)
    equalToFillColor[i] = useBrush ? FALSE : (img.color(setColPtr)==img.color(i));
  }  
//    if (x<0 || x>wid || y<0 || y>hei) return QRect();

 /*
 if (do8way)
 {
   fill8Recur(xin, yin, xin, yin, pos, setCol, setColPtr, red, green, blue, bgCol, bgColPtr, fillToBorder, treshold, p,
              img, useBrush, brushImg,  xmin, ymin, xmax, ymax);
   xmin=QMAX(0,xmin);
   ymin=QMAX(0,ymin);
   return QRect(xmin, ymin, QMAX(0,xmax-xmin)+1, QMAX(0,ymax-ymin)+1);
 } 
 */
 PUSH(y, x, x, 1);         /* needed in some cases */
 PUSH(y+1, x, x, -1);      /* seed segment (popped 1st) */

 while (sp>stack)
  {
   /* pop segment off stack and fill a neighboring scan line */
   POP(y, x1, x2, dy);
   /*
    * segment of scan line y-dy for x1<=x<=x2 was previously filled,
    * now explore adjacent pixels in scan line y
    */
   if (y>ymax) ymax=y;
   if (y<ymin) ymin=y;
   if (useBrush)
    for (x=x1; x>=0 && validPoint(x,y, red, green, blue, bgCol, bgColPtr, treshold, fillToBorder, img); x--)
     setPixColBrush(x, y, xin, yin, p, img, brushImg);
   else
    for (x=x1; x>=0 && validPoint(x,y, red, green, blue, bgCol, bgColPtr, treshold, fillToBorder, img); x--)
     setPixCol(x, y, setCol, setColPtr, p, img);

   if (x<xmin) xmin=x;

   if (x>=x1)
    goto skip;
   l = x+1;
   if (l<x1)
    PUSH(y, l, x1-1, -dy);   /* leak on left? */

   if (do8way && x>0)
    {
     PUSH(y, x, l, -dy);
     PUSH(y, x, l, dy);
    }

   x = x1+1;
   do
    {
     if (useBrush)
      for (; x<=wid && validPoint(x,y, red, green, blue, bgCol, bgColPtr, treshold, fillToBorder, img); x++)
       setPixColBrush(x, y, xin, yin, p, img, brushImg);
/* must not have else here, or fill to border does not work... */
      for (; x<=wid && validPoint(x,y, red, green, blue, bgCol, bgColPtr, treshold, fillToBorder, img); x++)             
       setPixCol(x, y, setCol, setColPtr, p, img);

     if (x>xmax)
      xmax=x;
     PUSH(y, l, x-1, dy);

     if (do8way && x<wid)
      {
       PUSH(y, x-1, x, -dy);
       PUSH(y, x-1, x, dy);
      }

     if (x>(x2+1))
      PUSH(y, x2+1, x-1, -dy);   // leak on right?         

skip:for (x++; x<=x2 && (!validPoint(x,y, red, green, blue, bgCol, bgColPtr, treshold, fillToBorder, img)); x++);
     l = x;
   } while (x<=x2);  
 }
 xmin=QMAX(0,xmin);
 ymin=QMAX(0,ymin);
 return QRect(xmin, ymin, QMAX(0,xmax-xmin), QMAX(0,ymax-ymin));
}
