/***************************************************************************
                          toolFreeTransform.cpp  -  description
                             -------------------
    begin                : Wed Sep 8 2004
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "toolFreeTransform.h"
#include <qpainter.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <math.h>
#include "apaintdoc.h"
#include "apaintconfig.h"
#include "apaintUtil.h"

extern char *getCommonPNG(const char *pixName);

#define CHECK_PT_SIZE  5
#define SQR(x) ((x)*(x))

FreeTransformDialog::FreeTransformDialog(QWidget *parent, const char *name):FreeTransformDialogUI(parent, name)
{
  setCaption(ApaintConfig::getText(736));
  btnOk->setText(ApaintConfig::getText(3));
  btnClose->setText(ApaintConfig::getText(86));
  connect(btnOk, SIGNAL(clicked()), this, SIGNAL(okClicked()));
  connect(btnClose, SIGNAL(clicked()), this, SIGNAL(closeSig())); 
}

FreeTransformDialog::~FreeTransformDialog()
{
  
}

void FreeTransformDialog::closeEvent(QCloseEvent *e)
{
  emit closeSig();
}
        
FreeTransformTool::FreeTransformTool(ToolType type, ApaintDoc *docp, FreeTransformDialog *fdlgp, PropObject *propObj):ToolBase(type, propObj)
{
 setToggleBtn(QPixmap(getCommonPNG("transform-perspective")), tr("unskew"));
 toolIsActive = false;
 Q_CHECK_PTR(docp);
 doc = docp;
 if (propObj)
 {
  setUsepWidget(true);
 }
 ptSelected = -1;
 Q_CHECK_PTR(fdlgp);
 fdlg = fdlgp;
 connect(fdlg, SIGNAL(okClicked()),this, SLOT(doTransform()));
 connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer *)));
}

FreeTransformTool::~FreeTransformTool()
{

}

void FreeTransformTool::begin()
{
  ToolBase::begin();
  toolIsActive = true;
  resetTool();
  fdlg->show();
}

void FreeTransformTool::resetTool()
{
  Layer *layer = doc->getLayer();
  QRect b = layer->layerPos();
  ptSelected = -1;
  qPts[0].setX(b.left());
  qPts[0].setY(b.top());
  qPts[1].setX(b.right());
  qPts[1].setY(b.top());
  qPts[2].setX(b.right());
  qPts[2].setY(b.bottom());
  qPts[3].setX(b.left());
  qPts[3].setY(b.bottom());
  layer->showBorder(false);
  doc->refreshLayer(getRefreshArea());
}

void FreeTransformTool::end()
{
 if (!toolIsActive)
  return;

  ToolBase::end();
  toolIsActive = false;
  Layer *layer = doc->getLayer();
  layer->showBorder(true);
  doc->refreshLayer(getRefreshArea());
  doc->refreshLayer();
  fdlg->hide();
}

void FreeTransformTool::layerChangedSlot(Layer *layer)
{
  if (toolIsActive)
   resetTool(); 
}

void FreeTransformTool::drawPoints(QPainter *p)
{
  int i;
  QPoint zqPts[4];

  if (!toolIsActive) return;
  
  p->setRasterOp(Qt::XorROP);
  p->setPen(QPen(QColor(128, 128,0),QMAX(2, (int)(zoomX/2))));
  
  for (i=0; i<4; i++)
  {
    zqPts[i].setX((int)(qPts[i].x()*zoomX));
    zqPts[i].setY((int)(qPts[i].y()*zoomY));
  }
  
  for (i=0; i<3; i++)
  {
    p->drawLine(zqPts[i].x(), zqPts[i].y(), zqPts[i+1].x(), zqPts[i+1].y());
    p->drawRect(zqPts[i].x()-CHECK_PT_SIZE, zqPts[i].y()-CHECK_PT_SIZE, 2*CHECK_PT_SIZE,  2*CHECK_PT_SIZE);
  }
  p->drawLine(zqPts[3].x(), zqPts[3].y(), zqPts[0].x(), zqPts[0].y());
  p->drawRect(zqPts[3].x()-CHECK_PT_SIZE, zqPts[3].y()-CHECK_PT_SIZE, 2*CHECK_PT_SIZE,  2*CHECK_PT_SIZE);
}

void FreeTransformTool::afterRefresh(QPainter *p)
{
  drawPoints(p);
}

QRect FreeTransformTool::getRefreshArea()
{
  int minx, miny, maxx, maxy;

  minx = miny = INT_MAX;
  maxx = maxy = 0;
  for (int i=0; i<4; i++)
  {
    if (minx > qPts[i].x()) minx = qPts[i].x();
    if (miny > qPts[i].y()) miny = qPts[i].y();
    if (maxx < qPts[i].x()) maxx = qPts[i].x();
    if (maxy < qPts[i].y()) maxy = qPts[i].y();    
  }
  return QRect(minx-CHECK_PT_SIZE, miny-CHECK_PT_SIZE, maxx-minx+2*CHECK_PT_SIZE, maxy-miny+2*CHECK_PT_SIZE);
}

int FreeTransformTool::getSelPoint(int x, int y)
{
  int len, i;
  for (i=0; i<4; i++)
  {
    len = (int)sqrt(SQR(x-qPts[i].x())+ SQR(y-qPts[i].y()));
    if (len <= (max(3, CHECK_PT_SIZE/((zoomX+zoomY)/2))))
     return i;
  }
  return -1;
}

//It does forward transformation from rect to selected quad.
void FreeTransformTool::doTransform()
{
  double RQ[3][3];
  double QR[3][3];
  QPoint rp[4];
  
  Layer *layer = doc->getLayer();
  if (!layer) return;
  QRect pos = getRefreshArea();
  
  for (int i=0; i<4; i++)
  {
    rp[i].setX(qPts[i].x()-pos.x());
    rp[i].setY(qPts[i].y()-pos.y());
  }
  QRect oldPos = getRefreshArea();
  PerspectiveTransform::quad2rect(0, 0, pos.width(), pos.height(), rp, RQ ,QR);
  doc->putUndoStack(ChangeLayerForm);
  doc->layerPerspectiveXForm(pos.x(), pos.y(), pos.width(), pos.height(), RQ);
  doc->refreshLayer(oldPos);
}

void FreeTransformTool::mousePressed(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
  int x = (int) (e->pos().x()/zoomX);
  int y = (int) (e->pos().y()/zoomY);
  ptSelected = getSelPoint(x, y);
}

void FreeTransformTool::mouseMoved(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
  int x = (int) (e->pos().x()/zoomX);
  int y = (int) (e->pos().y()/zoomY);
  if (ptSelected != -1)
  {
   drawPoints(pc);
   qPts[ptSelected].setX(x);
   qPts[ptSelected].setY(y);
   drawPoints(pc);
  }
}

void FreeTransformTool::mouseReleased(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
  ptSelected = -1;
}



