#include "widgetRuler.h"

#include <qpainter.h>
#include <qfont.h>
#include "apaintconfig.h"
#include <cmath>

#define FIX_WIDTH   30
#define SMALL_TICK  FIX_WIDTH/5
#define MEDIUM_TICK FIX_WIDTH/3
#define BIG_TICK FIX_WIDTH/2

#define SMALL_START  FIX_WIDTH - SMALL_TICK
#define SMALL_END    FIX_WIDTH
#define MEDIUM_START FIX_WIDTH - MEDIUM_TICK
#define MEDIUM_END   FIX_WIDTH
#define BIG_START    FIX_WIDTH - BIG_TICK
#define BIG_END      FIX_WIDTH

#define TICK_RES    5
#define BIG_SIZE    10

/*RULERINFO*/

RulerInfo::RulerInfo(QWidget *parent, const char* name):QFrame(parent, name)
{
 mStyle = Ruler::Pixel;
 init();
}

RulerInfo::RulerInfo(QWidget *parent, const char* name, Ruler::MetricStyle style):QFrame(parent, name)
{
 mStyle = style;
 init();
}

void RulerInfo::init()
{
  usingMetrics = ApaintConfig::useMetricSystem(); 
}

RulerInfo::~RulerInfo()
{
  
}

/*
void RulerInfo::setMetricSystem(bool enable)
{
  usingMetrics = enable;
  if (mStyle == Ruler::Inch && enable)
   mStyle = Ruler::Centim;
  else if (mStyle == Ruler::Centim && !enable)
   mStyle = Ruler::Inch;
   
  emit styleChanged(mStyle);
  repaint();
}
*/

void RulerInfo::mousePressEvent(QMouseEvent *e)
{
 if (mStyle != Ruler::Pixel)
   mStyle = Ruler::Pixel;
 else if (usingMetrics)
   mStyle = Ruler::Centim;
 else
   mStyle = Ruler::Inch;

 emit styleChanged(mStyle);
 repaint();
}

void RulerInfo::drawContents(QPainter *p)
{
  p->setBrush( p->backgroundColor() );
  QString info;
  switch(mStyle)
   {
    case Ruler::Pixel: info = ApaintConfig::getText(713); break;
    case Ruler::Centim: info = "cm"; break;
    case Ruler::Inch: info = "Inch"; break;
   }
  p->drawText(1, fontMetrics().height(), info);
}

/* RULER*/
Ruler::Ruler(Orientation orient, QWidget *parent, const char *name, WFlags f)
  : QFrame(parent, name, f)
{  
  orientRuler = orient;
  if (orient == Horizontal)
    setFixedHeight(FIX_WIDTH);
  else
    setFixedWidth(FIX_WIDTH);
  init();
}

void Ruler::init()
{
 // setFrameStyle(WinPanel | Raised); 
  sx = sy = 1;
  swSmall=true;
  swMedium=true;
  swBig=true;
  range.setValue(0);
  swPointer = true;
  showPointer(true);
  setNumOffset(1);
  drawNumRevers(false);
  setEndOffset(0);
  setTickMult(1);
  setRange(0,0);
  setOffset(0);
  setMetrics(Pixel, 0);
  setScale(1,1);   
  setBigTickNum(BIG_SIZE);
}


Ruler::~Ruler()
{
}

void Ruler::setBestTickMult()
{
  if ((int)(resolution*sx)>=5)
    tickMult = 1;
  else
    tickMult = (int)(5.0/(resolution*sx)+0.5);
}

void Ruler::setTickMult(int n)
{
  tickMult = n;
}

void Ruler::setBigTickNum(int num)
{
  bigTickNum = num;
  medTickNum = num/2;
  setDpi(dpi);
}

void Ruler::setMetrics(MetricStyle style, double dpiIn)
{
  mStyle = style;
  setDpi(dpiIn);
}

void Ruler::setDpi(double dpiIn)
{
 if (dpiIn == 0)
  dpi = 100;
 else
  dpi = dpiIn;   


 dpc = dpi/2.54;

 switch(mStyle)
 {
   case Pixel:
    setResolution(TICK_RES);
   break;
   case Inch:
    setResolution(dpi/(double)bigTickNum);
    setBestTickMult();
   break;
   case Centim:
    setResolution(dpc/(double)bigTickNum);
    setBestTickMult();
   break;
 }
 update(contentsRect());  
}


void Ruler::setNumOffset(int num)
{
  numOffset = num;
}

void Ruler::setResolution(double res)
{
  resolution = res;
}

void Ruler::setRange(int min, int max)
{
  range.setRange(min, max);
  update(contentsRect());
}

void Ruler::setValue(int val)
{
  if (val<range.minValue() || val>range.maxValue())
   return;
  if (swPointer)
   {
    drawPointer(range.value());
    drawPointer(val);
   }  
  range.setValue(val);
}

void Ruler::setOffset(int val)
{
  offset = val;
  update(contentsRect());
}

void Ruler::setScale(double x, double y)
{
  sx = x;
  sy = y;
  if (mStyle == Pixel)
  {
    setTickMult(1);
    if (sx <= 0.5)
    {
      showSmallTick(false);
      showMediumTick(false);
      showBigTick(true);
      if (sx <= 0.1)
       setResolution(80);
      else if (sx < 0.3)
       setResolution(40);
      else
       setResolution(20);
    }
    else if (sx < 1)
    {
      showSmallTick(false);
      showMediumTick(true);
      showBigTick(true);
      setResolution(10);
    }
    else if (sx>5)
    {
      showSmallTick(true);
      showMediumTick(true);
      showBigTick(true);
      setResolution(1);
    }
    else if (sx>=3)
    {
      showSmallTick(true);
      showMediumTick(true);
      showBigTick(true);
      setResolution(3);
    }
    else if (sx>=1)
    {
      showSmallTick(true);
      showMediumTick(true);
      showBigTick(true);
      setResolution(TICK_RES);
    }   
  }
  else
  {
    if (sx <= 0.5)
    {
      showSmallTick(false);
      showMediumTick(false);
      showBigTick(true);
    }
    else if (sx < 1)
    {
      showSmallTick(false);
      showMediumTick(true);
      showBigTick(true);
    }
    else if (sx >=1)
    {
      showSmallTick(true);
      showMediumTick(true);
      showBigTick(true);
    }
    setBestTickMult();    
  }
  update(contentsRect());
}

void Ruler::showSmallTick(bool s)
{
  if (s == swSmall) return;
  swSmall = s;
  update(contentsRect());
}

void Ruler::showMediumTick(bool s)
{
  if (s == swMedium) return;
  swMedium = s;
  update(contentsRect());
}

void Ruler::showBigTick(bool s)
{
  if (swBig == s) return;
  swBig = s;
  update(contentsRect());
}

void Ruler::showPointer(bool s)
{
  swPointer = s;
}

bool Ruler::drawSmall()
{
  return swSmall;
}

bool Ruler::drawMedium(int pos)
{
 if (numRevers)
  return(swMedium && (((int)(range.maxValue()/sy) - pos)%medTickNum == 0));
 else
  return(swMedium && (pos%medTickNum == 0));
}

bool Ruler::drawBig(int pos)
{
 if (numRevers)
  return (swBig && (((int)(range.maxValue()/sy) - pos)%bigTickNum == 0));
 else
  return (swBig && (pos%bigTickNum == 0));
}

void Ruler::drawNumRevers(bool enable)
{
  numRevers = enable;
}

void Ruler::setEndOffset(int endOff)
{
  endOffset = endOff;
}


void Ruler::drawPointer(int val)
{
 QPointArray pa = QPointArray(3);
 int value = val-offset;

 if (swPointer)
  {
   QPainter p(this);

   p.setClipRect(contentsRect());
   p.setRasterOp(XorROP);
   if (orientRuler == Horizontal)
    pa.setPoints(3, value-5, FIX_WIDTH-10, value+5, FIX_WIDTH-10, value, FIX_WIDTH-3);
   else
    pa.setPoints(3, FIX_WIDTH- 10, value-5, FIX_WIDTH- 10, value+5, FIX_WIDTH-3, value);
   p.setBrush( p.backgroundColor() );
   p.drawPolygon( pa );
  }
}

void Ruler::drawContents(QPainter *p)
{
  int val,startDraw, endDraw, xpos;
  int cnt, cntOld, addDraw;
  bool tick;
  double add;

  /* because of gray border around */
  addDraw = 1;
   
  add = startDraw = endDraw  = 0;
  cntOld = -1;
  
  if (orientRuler == Horizontal)
   {
    startDraw = QMAX(contentsRect().x(), range.minValue());
    endDraw = QMIN(width()-endOffset, range.maxValue()+1-offset);//+1 so that the last num is also drawn
    add = resolution*sx*tickMult;
   }
  else if (orientRuler == Vertical)
   {
    startDraw = QMAX(contentsRect().x(), range.minValue());   
    endDraw = QMIN(height()-endOffset, range.maxValue()+1-offset);//+1 so that the last num is also drawn
    add = resolution*sy*tickMult;   
   }
  if (!add) return;
  
  if (orientRuler == Vertical)
   {
    p->setBrush( p->backgroundColor() );
    p->setBackgroundMode(Qt::OpaqueMode);
   }  
  
  if (orientRuler == Horizontal)
   for (val = startDraw; val<endDraw; val++)
    {
      cnt = (double)ceil(val+offset-1)/add;
      if (cnt != cntOld)
       {
	tick = true;
	cntOld = cnt;
       }
      else
       tick = false;

      if (!tick) continue;
      if (drawSmall())
       p->drawLine(val+addDraw, SMALL_START, val+addDraw, SMALL_END);
      if (drawMedium(cnt))
       p->drawLine(val+addDraw, MEDIUM_START, val+addDraw, MEDIUM_END);
      if (drawBig(cnt))
      {
        p->drawLine(val+addDraw, BIG_START, val+addDraw, BIG_END);
        if (sx>0)
        {
          QString txt;

          if (mStyle == Pixel)
           txt = QString("%1").arg(cnt*((int)resolution)*tickMult);
          else  if(mStyle == Centim || mStyle == Inch)
           txt = QString("%1").arg((int)(cnt/bigTickNum)*tickMult);

          if (!val)
           p->drawText(fontMetrics().width(txt), fontMetrics().height()+contentsRect().y(), txt); 
          else
           {
            xpos = val - fontMetrics().width(txt)/2;
            p->drawText(xpos, fontMetrics().height()+contentsRect().y(), txt);
           }
        }
       }
     }
  cntOld = -1;

 if (orientRuler == Vertical)
  for (val=endDraw; val>=startDraw; val--)
   {
     cnt = ceil((double)(val+offset-2)/add);
     if (cnt != cntOld)
      {
       tick = true;
       cntOld = cnt;
      }
     else
      tick = false;

     if (!tick) continue;

     if (drawSmall())
       p->drawLine(SMALL_START, val, SMALL_END, val);
     if (drawMedium(cnt))
      p->drawLine(MEDIUM_START, val, MEDIUM_END, val);
     if (drawBig(cnt))
      {
        p->drawLine(BIG_START, val, BIG_END, val);
        if (sy>0)
        {
          QString txt;
           if (mStyle == Pixel)
            txt = QString("%1").arg((int)(floor(range.maxValue()/add-cnt)*((int)resolution)*tickMult));
          else  if(mStyle == Centim || mStyle == Inch)
            txt = QString("%1").arg((int)(floor(range.maxValue()/add-cnt)/bigTickNum)*tickMult);

          if (val < fontMetrics().height())
            p->drawText(contentsRect().x(), fontMetrics().height(), txt);
          else
            p->drawText(contentsRect().x(), val, txt);
        }
      }
    }
 drawPointer(range.value());
}

void Ruler::mouseDoubleClickEvent(QMouseEvent *event)
{
  QPoint* point = new QPoint(event->x(), event->y());
  Orientation guideOrientation = Horizontal;
  if (orientRuler == Vertical)
   guideOrientation = Horizontal;
  else if (orientRuler == Horizontal)
   guideOrientation = Vertical;
  emit mouseDoubleClicked(point, guideOrientation);
}
