#ifndef KRULER_H
#define KRULER_H

#include <qframe.h>
#include <qrangecontrol.h>


class Ruler : public QFrame
{
  Q_OBJECT  
public:

  /**
   * The types of units used.
   **/
  enum MetricStyle { Pixel = 0, Inch, Centim };
  static const int numMetrics = 3;
  
  Ruler(Orientation orient, QWidget *parent=0, const char *name=0, WFlags f=0);
  
  ~Ruler();

   void setRange(int min, int max);
   void setValue(int val);
   void setOffset(int val);
   void showSmallTick(bool s);
   void showMediumTick(bool s);
   void showBigTick(bool s);
   void showPointer(bool s);
   void setResolution(double res);
   void setNumOffset(int num);
   void drawNumRevers(bool enable);
   void setEndOffset(int endOff);
   void setMetrics(MetricStyle style, double dpiIn);
   void setDpi(double dpiIn);   
   
   //Guide change
  virtual void mouseDoubleClickEvent(QMouseEvent *event); 
   
signals:
  void mouseDoubleClicked(QPoint *, Orientation);
  //Guide change end
public slots:
   void setScale(double x, double y);
protected:

  virtual void drawContents(QPainter *);
  void drawPointer(int val);
  void setBigTickNum(int num);
  void setTickMult(int n);
  void setBestTickMult();
  
private:
  void init();
  bool drawSmall();
  bool drawMedium(int pos);
  bool drawBig(int pos);
  
private:
  bool showpointer;
  int fontWidth; // ONLY valid for vertical rulers
  double sx, sy;
  QRangeControl range;
  int offset;
  bool swSmall, swMedium, swBig;
  Orientation orientRuler;
  bool swPointer;
  double resolution;
  int numOffset;//number you start counting, ususally 0
  bool numRevers;
  int endOffset;
  MetricStyle mStyle;
  double dpi, dpc;
  int bigTickNum, medTickNum;
  int tickMult; /*count only tickMult - tick*/
};


/*RulerInfo*/

class RulerInfo: public QFrame
{
   Q_OBJECT  
public:
  RulerInfo(QWidget *parent, const char* name);
  RulerInfo(QWidget *parent, const char* name, Ruler::MetricStyle style);
  ~RulerInfo();
  
  Ruler::MetricStyle getStyle() {return mStyle;}
//  void setMetricSystem(bool enable);

signals:
  void styleChanged(Ruler::MetricStyle style);
protected:

  virtual void drawContents(QPainter *p);
  virtual void mousePressEvent(QMouseEvent *e); 

private:
 void init();

 Ruler::MetricStyle mStyle;
 bool usingMetrics;
 
};

#endif
