/* pbmp.c */

#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "print.h"

#ifdef __MINGW32__
#include <windows.h>
#endif

static FILE *fout;
static char *fname;
static int printModeType;
static int paddedLineBytes;
static int fullHei;
static unsigned char *mirrorBuffer = NULL;

static void putShort(int i)
{
 putc(i & 0xff, fout);
 putc((i>>8) & 0xff, fout);
}

static void putLong32(unsigned int i)
{
 putc( i & 0xff, fout);
 putc((i>>8)  & 0xff, fout);
 putc((i>>16) & 0xff, fout);
 putc((i>>24), fout);
}

static int initPrint(int printType, int complexImage, int xpixels,
	int ypixels, int xdpi, int ydpi, int xmargin, int ymargin)
{
 int cmapColors=0;
 int bitsPerPixel;
 int filedes = -1;

 printModeType = printType;
 switch (printModeType)
  {
   case IMG_BW:
    cmapColors = 2;
    paddedLineBytes = (((xpixels+7)/8 + 3)>>2)<<2;
    bitsPerPixel = 1;
    break;
   case IMG_CMAP:
    cmapColors = 256;
    paddedLineBytes = ((xpixels + 3)>>2)<<2;
    bitsPerPixel = 8;
    break;
   case IMG_TRUE_COL:
    cmapColors = 0;
    paddedLineBytes = ((xpixels*3 + 3)>>2)<<2;
    bitsPerPixel = 24;
    break;
   default:
    return(PRT_INTERNAL_ERROR);
  }

#ifdef __MINGW32__
 char tempPath[MAX_PATH + 1];

 if (GetTempPathA(MAX_PATH + 1, tempPath) == 0)
  {
   // Failed to get temp path.
   perror("plib: failed to get temp path");
   return (PRT_IO_ERROR);
  }

 // This is allocated in advance because of the code below.
 char tempFilename[MAX_PATH + 1];

 fname = tempFilename;
 if (GetTempFileNameA(tempPath, "prt", 1, fname) == 0)
  {
   perror("plib: failed to create tmpfile");
   return(PRT_IO_ERROR);
  }

 fout = fopen(fname, "wb");
 fname = strdup(fname);
#else
 fname = strdup("/tmp/prtXXXXXX");
 if ((filedes=mkstemp(fname)) < 0 )
  return(PRT_IO_ERROR);
 else
  fout = fdopen(filedes, "wb");
#endif

 fputc('B', fout); /* verson 3 "BM" header */
 fputc('M', fout);
 putLong32(14 + 40 + cmapColors*4 + paddedLineBytes*ypixels); /* total external file length */
 putShort(0);
 putShort(0);
 putLong32(14 + 40 + cmapColors*4); /* image data offset from start of file */
 /* end of WIN3XHEAD */

 /* start WIN3XINFOHEAD */
 putLong32(40); /* length of this header */
 putLong32(xpixels);
 putLong32(ypixels);
 putShort(1); /* number of planes; always 1!!!??? */
 putShort(bitsPerPixel); /* can be 1, 4, 8, 24 */
 putLong32(0); /* no compression */
 putLong32(paddedLineBytes*ypixels); /* size of image data */
 putLong32((int)((xdpi*(100/2.54))/1000)); /* number of horizontal pixels per meter */
 putLong32((int)((ydpi*(100/2.54))/1000)); /* number of vertical pixels per meter */
 putLong32(cmapColors); /* number of colors in colormap */
 putLong32(cmapColors); /* all colors in colormap are significant */
 /* end of WIN3XINFOHEAD */
 fullHei = ypixels; /* remember this so we can mirror the image later on */
 return(PRT_OK);
}

static int printBand(int xpixels, char *data, int bpl, int ypos,
	int lines, char bandType, unsigned char *cmap)
{
 int x, y, yy;
 int val;
 unsigned char *mirrorBufferPtr=NULL;

 if (data==NULL)
  return(PRT_INTERNAL_ERROR);
 if (printModeType<bandType)
  return(PRT_WRONG_IMG);

 if (ypos==0) /* need to output the colormap here */
  {
   if (printModeType==IMG_BW)
    {
     fputc(255, fout); fputc(255, fout); fputc(255, fout); fputc(0, fout); /* white */
     fputc(0, fout); fputc(0, fout); fputc(0, fout); fputc(0, fout); /* black */
    }
   if (printModeType==IMG_CMAP)
    {
     for (x=0; x<256; x++)
      {
       fputc(cmap[x*3+2], fout); /* blue */
       fputc(cmap[x*3+1], fout); /* green */
       fputc(cmap[x*3], fout); /* red */
       fputc(0, fout); 
      }
    }
   if (fullHei!=lines)
    {
     mirrorBuffer = calloc(paddedLineBytes, fullHei); /* use calloc to avoid warnings of uninitialised data on padding */
     if (!mirrorBuffer)
      return(PRT_NO_MEMORY);
    }
  }

 switch (printModeType)
  {
   case IMG_BW: /* just BW image possible, anyway */
    for (y=0; y<lines; y++)
     if (fullHei==lines)
      {
       fwrite(&data[(lines-y-1)*bpl], 1, (xpixels+7)/8, fout);
       for (x = paddedLineBytes - (xpixels+7)/8; x; x--)
	fputc(0, fout);
      }
     else
      memcpy(&mirrorBuffer[(fullHei-ypos-y-1)*paddedLineBytes], &data[y*bpl], (xpixels+7)/8);
    break;
   case IMG_CMAP:
    for (y=0; y<lines; y++)
     {
      if (bandType==IMG_CMAP)
       {
        if (fullHei==lines)
         fwrite(&data[(lines-y-1)*bpl], 1, xpixels, fout);
        else
	 memcpy(&mirrorBuffer[(fullHei-ypos-y-1)*paddedLineBytes], &data[y*bpl], xpixels);
       }
      else /* the IMG_BW case */
       {
        if (fullHei==lines)
	 {
	  yy = (lines-y-1)*bpl;
          for (x=0; x<xpixels; x++)
	   fputc((data[yy+(x>>3)]&(128>>(x&7))) ? 0 : 1, fout);
	 }
	else
	 {
	  mirrorBufferPtr = &mirrorBuffer[(fullHei-ypos-y-1)*paddedLineBytes];
	  yy = y*bpl;
          for (x=0; x<xpixels; x++)
	   *mirrorBufferPtr++ = (data[yy+(x>>3)]&(128>>(x&7))) ? 0 : 1;
	 }
       }
      if (fullHei==lines)
       for (x = paddedLineBytes - xpixels; x; x--)
	fputc(0, fout);
     }
    break;

   case IMG_TRUE_COL:
    for (y=0; y<lines; y++)
     {
      yy = (fullHei==lines) ? (lines-y-1)*bpl : y*bpl;
      if (fullHei!=lines)
       mirrorBufferPtr = &mirrorBuffer[(fullHei-ypos-y-1)*paddedLineBytes];
      switch (bandType)
       {
	case IMG_BW:
         if (fullHei==lines)
          for (x=0; x<xpixels; x++)
	   {
	    val = (data[yy+(x>>3)]&(128>>(x&7))) ? 0 : 255;
	    fputc(val, fout); fputc(val, fout); fputc(val, fout);
	   }
	 else
          for (x=0; x<xpixels; x++)
	   {
	    val = (data[yy+(x>>3)]&(128>>(x&7))) ? 0 : 255;
	    *mirrorBufferPtr++ = val; *mirrorBufferPtr++ = val; *mirrorBufferPtr++ = val; 
	   }
	break;
        case IMG_CMAP:
         if (fullHei==lines)
          for (x=0; x<xpixels; x++)
	   {
	    val = ((unsigned char) data[yy+x])*3;
	    fputc(cmap[val+2], fout); /* blue */
	    fputc(cmap[val+1], fout); /* green */
	    fputc(cmap[val], fout); /* red */
	   }
	 else
          for (x=0; x<xpixels; x++)
	   {
	    val = ((unsigned char) data[yy+x])*3;
	    *mirrorBufferPtr++ = cmap[val+2]; /* blue */
	    *mirrorBufferPtr++ = cmap[val+1]; /* green */
	    *mirrorBufferPtr++ = cmap[val]; /* red */
	   }
         break;
        case IMG_TRUE_COL_DITH:
        case IMG_TRUE_COL:
	 if (fullHei==lines)
          for (x=0; x<xpixels; x++)
	   {
	    fputc(data[yy+2], fout); /* blue */
	    fputc(data[yy+1], fout); /* green */
	    fputc(data[yy], fout); /* red */
	    yy+=3;
	   }
	 else
          for (x=0; x<xpixels; x++)
	   {
	    *mirrorBufferPtr++ = data[yy+2]; /* blue */
	    *mirrorBufferPtr++ = data[yy+1]; /* green */
	    *mirrorBufferPtr++ = data[yy]; /* red */
	    yy+=3;
	   }
        break;
       }
      if (fullHei==lines)
       for (x = paddedLineBytes - xpixels*3; x; x--)
	fputc(0, fout);
     }
    break;
   default:
    return(PRT_INTERNAL_ERROR);
  }
 return(PRT_OK);
}

static int endPrint(int printStatus)
{
 if (printStatus==PRT_OK)
  {
   if (mirrorBuffer) /* write out the accumulated data in one go */
    fwrite(mirrorBuffer, fullHei, paddedLineBytes, fout);
   fflush(fout);
   fclose(fout);
   printStatus = enqueuePrintFile(fname);
  }
 else
  fclose(fout);
 /* just cleanup; called even if printing fails */
 if (mirrorBuffer)
  free(mirrorBuffer);
 mirrorBuffer=NULL;
 removeFileFreeFilename(fname, printStatus);
 return(printStatus);
}

/* fill in the preallocated printer modes */
static void getPrtInfo(printerMode *printerMod, int *numModes,
	printerDensity *prtDensityX, int *numDensitX,
	printerDensity *prtDensityY, int *numDensitY,
	int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
	int *dpiVariable, char **prtName, char **profileName)
{
 int i;

 printerMod[0].printType = IMG_TRUE_COL;
 printerMod[1].printType = IMG_CMAP;
 printerMod[2].printType = IMG_BW;

 *numModes=3;

 *numDensitX = *numDensitY = 7;
 for (i=0; i<6; i++)
  {
   prtDensityX[i].pixels = prtDensityY[i].pixels = MAX_PRINT;
   prtDensityX[i].dpi = prtDensityY[i].dpi = 100000*(i+1);
  }
 prtDensityX[6].pixels = prtDensityY[6].pixels = MAX_PRINT;
 prtDensityX[6].dpi = prtDensityY[6].dpi = 720000;


 *nonprintableTop=0;
 *nonprintableBottom=0;
 *nonprintableLeft=0;
 *dpiVariable=TRUE;
 *prtName="BMP";
 *profileName=NULL;
}

/* register PbmPlus "printer" */
void registerBMP()
{
 registerPrinter(*getPrtInfo, *initPrint, *printBand, *endPrint);
}
